# UPB e-shop

This is the main project through which assignments from the UPB subject (STU FEI Bratislava) will be fulfilled. 
## Prerequisites

Before you can run this project, ensure you have the following installed:

- [Python](https://www.python.org/downloads/)
- [Docker](https://www.docker.com/get-started)
- [Docker Compose](https://docs.docker.com/compose/install/)
- [Git](https://git-scm.com/downloads) (for cloning the repository)

## Getting Started

Follow these steps to set up and run the project:

1. **Clone the repository to your local machine:**

   ```bash
   git clone https://github.com/MarekGalik/UPB-eshop.git
   cd UPB-eshop/
   
2. **Build and start Docker containers:**
   ```bash
   docker-compose up -d --build

3. **Access the Django development server and React frontend:**
   
     Open your web browser and go to http://localhost:8000/admin/ with default credentials admin@admin.com:admin to access the Django development server.
To access the React frontend go to http://localhost:3000/.

## Local machine development
### Django local machine development
Following steps are optional, do it only if you want to develop Django backend locally.
4. **Shut down web container:**
   ```bash
    docker stop be
   ```
5. **Create and activate a virtual environment (optional but recommended):**
   ```bash
   python -m venv venv
   source venv/bin/activate # On Windows: venv\Scripts\activate
6. **Install project dependencies:**
   ```bash
   pip install -r django_backend/requirements.txt
7. **Change HOST in
   [settings.py](django_backend/django_backend/settings.py)**
   ```bash
    # 'HOST': 'db',  # running server in docker container
    'HOST': 'localhost',  # running server locally (db in docker container)
   ```
8. **Run server locally:**

   ```bash
    python django_backend/manage.py runserver
   ```
9. **Access the Django local development server:**
   
     Open your web browser and go to http://localhost:8000/admin/ with default credentials admin@admin.com:admin. Now server is running on your machine with Docker db.


### React frontend local machine development
Following steps are optional, do it only if you want to develop React frontend locally. You can do this with Django backend running either in Docker container or locally.
4. **Shut down docker container:**
   ```bash
    docker stop fe
   ```
5. **Check if you already have downloaded `npm` by running this command:**
   ```commandline
    npm -v
   ```
6. **If you do not have npm, you probably need to [install Node.js](https://nodejs.org/en/download/).**
   - Verify the installation and [install](https://nodejs.org/en/download/) if needed:
   ```commandline
    node -v
   ```
7. **You also need to have Vite.js installed. You can do this by running:**
   ```commandline
    npm install -g create-vite
   ```
8. **Before running the server you have to install all dependencies from package.json. Change the directory and run this command:**
   ```commandline
    cd react_frontend/
    npm install
   ```
9. **Once you have successfully installed npm and all dependencies, run this command:**
   ```commandline
    npm run dev
   ```
10. **Access the React frontend local web app:**
   
   After running previous commands, you will see something like this in the terminal:
   ```commandline
    VITE v4.5.0  ready in 1089 ms
      ➜  Local:   http://localhost:3000/
      ➜  Network: use --host to expose
      ➜  press h to show help
   ```
   Open your web browser and go to http://localhost:3000/ to access the React frontend web app.
