from django.db import models
from django.core.exceptions import ValidationError

from django.contrib.auth.models import AbstractUser


class CustomUser(AbstractUser):
    first_name = models.CharField(max_length=30)
    last_name = models.CharField(max_length=30)
    address = models.CharField(max_length=100)
    email = models.EmailField(unique=True)
    phone = models.CharField(max_length=15)
    # Specify related_name to avoid clashes
    groups = models.ManyToManyField('auth.Group', related_name='customuser_set')
    user_permissions = models.ManyToManyField('auth.Permission', related_name='customuser_set')

    def __str__(self):
        return self.username

    def save(self, *args, **kwargs):
        if not self.email.strip():
            raise ValidationError("Email cannot be empty.")
        if CustomUser.objects.filter(username=self.email).exclude(id=self.id).exists():
            raise ValidationError("User with this email already exists.")
        self.username = self.email
        super().save(*args, **kwargs)
