import base64
import binascii
import json
from cryptography.fernet import Fernet, InvalidToken
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import padding
from django_backend.settings import PRIVATE_KEY
from rest_framework.response import Response
from rest_framework.status import HTTP_422_UNPROCESSABLE_ENTITY


def assymetric_encrypt(data, public_key=None):
    symmetric_key = Fernet.generate_key()
    f = Fernet(symmetric_key)
    encrypted_data = f.encrypt(json.dumps(data).encode('utf-8'))
    if public_key is None:
        public_key = PRIVATE_KEY.public_key()
    encrypted_key = public_key.encrypt(
        symmetric_key,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )
    return {'encrypted_data': encrypted_data, 'encrypted_key': base64.b64encode(encrypted_key)}


def symmetric_encrypt(data, symmetric_key=None):
    if symmetric_key is None:
        symmetric_key = Fernet.generate_key()
    f = Fernet(symmetric_key)
    encrypted_data = f.encrypt(json.dumps(data).encode('utf-8'))
    return encrypted_data


def decrypt(encrypted_data, encrypted_key):
    try:
        encrypted_key = base64.b64decode(encrypted_key)
    except binascii.Error:
        return {'error': 'Integrity check in key failed.'}

    try:
        symmetric_key = decrypt_symmetric_key(encrypted_key=encrypted_key)
    except ValueError:
        return {'error': 'Integrity check in key failed.'}

    try:
        decrypted_data = decrypt_data(symmetric_key=symmetric_key, encrypted_data=encrypted_data)
    except InvalidToken:
        return {'error': 'Integrity check in data failed.'}

    return {'decrypted_data': decrypted_data}


def decrypt_symmetric_key(encrypted_key):
    return PRIVATE_KEY.decrypt(
        encrypted_key,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )


def decrypt_data(symmetric_key, encrypted_data):
    f = Fernet(symmetric_key)
    decrypted_data = f.decrypt(encrypted_data)
    return json.loads(decrypted_data)


def decrypt_request_body(request):
    encrypted_key = base64.b64decode(request.data.get("symmetric_key", ""))
    encrypted_data = base64.b64decode(request.data.get("data", ""))
    try:
        symmetric_key = decrypt_symmetric_key(encrypted_key=encrypted_key)
    except ValueError:
        return Response({'detail': 'Integrity check in key failed.'}, status=HTTP_422_UNPROCESSABLE_ENTITY)
    try:
        decrypted_data = decrypt_data(symmetric_key=symmetric_key, encrypted_data=encrypted_data)
    except InvalidToken:
        return Response({'detail': 'Integrity check in data failed.'}, status=HTTP_422_UNPROCESSABLE_ENTITY)

    return {'data': decrypted_data, 'key': symmetric_key}


