from rest_framework import serializers
from .models import Order, OrderItem
from ..access_control.serializers import UserRegistrationSerializer
from ..products.serializers import ProductSerializer


class OrderItemSerializer(serializers.ModelSerializer):
    total_price = serializers.SerializerMethodField()
    product = serializers.SerializerMethodField('get_product')


    def get_total_price(self, obj):
        return str(obj.quantity * obj.product.price)

    def get_product(self, obj):
        return ProductSerializer(obj.product).data

    class Meta:
        model = OrderItem
        fields = ('product', 'quantity', 'total_price')


class OrderSerializer(serializers.ModelSerializer):
    products = serializers.SerializerMethodField('get_products')
    total_price = serializers.SerializerMethodField('get_total_price')
    user_information = serializers.SerializerMethodField('get_user_information')
    order_id = serializers.IntegerField(source='id')

    def get_user_information(self, obj):
        return UserRegistrationSerializer(obj.user).data

    def get_products(self, obj):
        qs = OrderItem.objects.filter(order__pk=obj.id)
        return OrderItemSerializer(qs, many=True).data

    def get_total_price(self, obj):
        qs = OrderItem.objects.filter(order__pk=obj.id)
        return str(sum(float(item['total_price']) for item in OrderItemSerializer(qs, many=True).data))

    class Meta:
        model = Order
        fields = ('order_id', 'user_information', 'products', 'total_price', 'created_at')
