from django.db import models
from django.core.validators import MaxValueValidator, MinValueValidator

from ..access_control.models import CustomUser
from ..products.models import Product


class ProductReview(models.Model):
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE)
    product = models.ForeignKey(Product, on_delete=models.CASCADE)
    rating = models.IntegerField(
        validators=[
            MinValueValidator(0, message="Rating must be at least 0."),
            MaxValueValidator(5, message="Rating must be at most 5.")
        ]
    )
    review = models.TextField(max_length=512)
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"Review by {self.user} for {self.product}"

    class Meta:
        unique_together = ('user', 'product')