from rest_framework import serializers
from .models import ProductReview
from ..access_control.models import CustomUser
from ..access_control.serializers import UserRegistrationSerializer
from ..products.models import Product


class ProductReviewSerializer(serializers.ModelSerializer):
    user_information = serializers.SerializerMethodField('get_user_information')

    def get_user_information(self, obj):
        return UserRegistrationSerializer(CustomUser.objects.get(id=obj.user.id)).data

    class Meta:
        model = ProductReview
        fields = ['id', 'user_information', 'product', 'rating', 'review', 'created_at']
        read_only_fields = ['id', 'user', 'created_at']

    def validate_product(self, value):
        # Ensure that the product exists
        try:
            Product.objects.get(pk=value)
        except Product.DoesNotExist as e:
            raise serializers.ValidationError("The specified product does not exist.") from e

        return value
