import React, {useEffect, useState} from "react";
import Card from "react-bootstrap/Card";
import {useNavigate} from "react-router-dom";
import Table from "react-bootstrap/Table";

import ListGroup from "react-bootstrap/ListGroup";
import Form from "react-bootstrap/Form";
import Button from "react-bootstrap/Button";
import {
    decrypt_products,
    encrypt_data,
    is_valid_public_key_pem,
    is_valid_text_data,
    send_pem_get_orders
} from "./utils.ts";
import {useShoppingCart} from "../context/ShoppingCartContext.tsx";
import Row from "react-bootstrap/Row";
import Col from "react-bootstrap/Col";
import {forEach} from "react-bootstrap/ElementChildren";
import Encryption_types from "./encryption_types.ts";
import {Container, FormControl, Modal} from "react-bootstrap";
import {jsPDF} from "jspdf";


export function ClientZoneForm() {

    const [show, setShow] = useState(false);
    const [public_key, setPublicKey] = useState<string>('');
    const [private_key, setPrivateKey] = useState<string>('');
    const [text_data, setTextData] = useState<string>('');
    const [error, setError] = useState(null);
    const [got_orders, setGotOrders] = useState(false);
    const [encrypted_data, setEncryptedData] = useState<string>('');
    const [encrypted_key, setEncryptedKey] = useState<string>('');
    const [decrypted_data, setDecryptedData] = useState(null);
    const [isDecrypted, setDecrypted] = useState(false)
    const navigate = useNavigate();
    const [textfields, setTextfields] = useState({});
    const [ratingFields, setRatingfields] = useState({});
    const [loading, setLoading] = useState(false);
    const [review_success, setReviewSuccess] = useState(false);

    const handleClose = () => setShow(false);
    const handleShow = () => {
        setShow(true);
        setLoading(false);
        setError(null);
        setReviewSuccess(false);
    };
    const handlePKTextareaChange = (event: React.ChangeEvent<HTMLTextAreaElement>) => {
        setPublicKey(event.target.value);
    };
    const handleSKTextareaChange = (event: React.ChangeEvent<HTMLTextAreaElement>) => {
        setPrivateKey(event.target.value);
    };
    const handleDataTextareaChange = (event: React.ChangeEvent<HTMLTextAreaElement>) => {
        setTextData(event.target.value);
    };
    const handleReviewTextareaChange = (order_id) => (event: React.ChangeEvent<HTMLTextAreaElement>) => {
        const updatedTextfields = {
            ...textfields,
            [order_id]: event.target.value,
        };
        setTextfields(updatedTextfields);
        //console.log(updatedTextfields);
        //console.log(event.target.value);
    };

    const handleRatingInputChange = (order_id) => (event: React.ChangeEvent<HTMLTextAreaElement>) => {
        const updatedRatingFields = {
            ...ratingFields,
            [order_id]: event.target.value,
        };
        setRatingfields(updatedRatingFields);
        //console.log(updatedRatingFields);
        //console.log(event.target.value);
    };

    const receiveOrders = (e: any) => {
        e.preventDefault();
        if (sessionStorage.getItem("session_token") == null) {
            navigate('/');
            return;
        }
        setError(null);
        try {
            is_valid_public_key_pem(public_key);
        } catch (er: any) {
            setError(er);
            return;
        }
        send_pem_get_orders(public_key).then((data) => {
            setGotOrders(true);
            setEncryptedData(data.encrypted_data);
            setTextData(data.encrypted_data);
            setEncryptedKey(data.encrypted_key);
        }).catch((er) => {
            setError(er);
        });
    }

    const showEncryptedOrders = (e: any) => {
        e.preventDefault();
        setError(null);
        try {
            is_valid_text_data(text_data, encrypted_data);
        } catch (er: any) {
            setError(er);
            return;
        }
        try {
            decrypt_products(encrypted_key, private_key, text_data).then((decrypted_data) => {
                const dataArray = JSON.parse(decrypted_data);
                setDecryptedData(dataArray);

            }).catch((er) => {
                setError(er);
            })
        } catch (er: any) {
            setError(er);
            return;
        }
    }

    const getReviewFromJson = (id: string): string => {
        let review = "";
        decrypted_data.forEach((order) => {
            order.products.forEach((product) => {
                if (product.product.id == id) {
                    if (product.product.reviews.length === 0) {
                        review = "";
                    } else {
                        review = product.product.reviews[0].review;
                    }

                }
            })
        })
        return review;
    }
    const getRatingFromJson = (id: string): string => {
        let rating = "";
        decrypted_data.forEach((order) => {
            order.products.forEach((product) => {
                if (product.product.id == id) {
                    if (product.product.reviews.length === 0) {
                        rating = "";
                    } else {
                        rating = product.product.reviews[0].rating;
                    }

                }
            })
        })
        return rating;
    }

    const sendReview = (e: any) => {
        setLoading(true);
        setError(null);
        setReviewSuccess(false);
        e.preventDefault();
        const id = e.target.id;
        let review = textfields[id];
        let rating = ratingFields[id];
        if (review === undefined) {
            review = getReviewFromJson(id)
        }
        if (rating === undefined) {
            rating = getRatingFromJson(id);
        }
        const json_review = {
            review_data: {
                product_id: parseInt(id),
                rating: parseInt(rating),
                review: review
            }
        }
        let type: Encryption_types = Encryption_types.Review;
        encrypt_data(json_review, sessionStorage.getItem('server_public_key'), type).then((encoded_data) => {
            fetch('http://localhost:8000/api/review', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'Authorization': `Token ${sessionStorage.getItem("session_token")}`,
                },
                body: JSON.stringify(encoded_data),
            }).then((response) => {
                if (!response.ok) {
                    return response.json().then(response => {
                        if (response.hasOwnProperty('detail')) {
                            if (response.hasOwnProperty('rating')) {
                                throw new Error(response.rating[0])
                            }
                            if (response.hasOwnProperty('review')) {
                                throw new Error(response.review[0])
                            }
                            if (response.hasOwnProperty('user')) {
                                throw new Error(response.user[0])
                            }
                            if (response.hasOwnProperty('product')) {
                                throw new Error(response.product[0])
                            }
                            if (response.hasOwnProperty('__all__')) {
                                throw new Error(response.__all__[0])
                            }
                        }
                        throw new Error()
                    });
                }
                return response.json();
            }).then(() => {
                setLoading(false);
                setReviewSuccess(true);
                //handleShow();
            })
                .catch((error) => {
                    setError(error);
                    setLoading(false);
                    setReviewSuccess(false);
                });
        });


    }

    const formatDate = (dateString) => {
        const date = new Date(dateString);
        return `${date.toLocaleDateString()} ${date.toLocaleTimeString()}`;
    };

    const createPDF = (e: any) => {
        setLoading(true);
        setError(null);
        setReviewSuccess(false);
        e.preventDefault();
        const orderId = parseInt(e.target.id);
        let order;
        for (var i = 0; i < decrypted_data.length; i++) {
            if (decrypted_data[i].order_id === orderId) {
                order = decrypted_data[i];
                break;
            }
        }
        console.log(order);

        const pdf = new jsPDF();

        pdf.setFont('helvetica', 'bold');
        pdf.setFontSize(20);
        pdf.text('Invoice', 10, 20);

        pdf.setFont('helvetica', 'normal');
        pdf.setFontSize(12);
        pdf.text(`Order ID: ${order.order_id}`, 10, 30);
        pdf.text(`Order Date: ${formatDate(order.created_at)}`, 10, 40);

        pdf.text(`Bill To:`, 10, 60);
        pdf.text(`${order.user_information.first_name} ${order.user_information.last_name}`, 10, 70);
        pdf.text(`${order.user_information.address}`, 10, 80);
        pdf.text(`${order.user_information.email}`, 10, 90);
        pdf.text(`${order.user_information.phone}`, 10, 100);

        pdf.text('Products:', 10, 120);
        let currentY = 130;

        order.products.forEach((item, index) => {
            pdf.text(`- ${item.product.name}`, 20, currentY);
            currentY += 10;
            pdf.text(`  Description: ${item.product.description}`, 20, currentY);
            currentY += 10;
            pdf.text(`  Quantity: ${item.quantity}`, 20, currentY);
            currentY += 10;
            pdf.text(`  Total Product Price: ${item.total_price}€`, 20, currentY);
            currentY += 15;

            if (item.product.reviews.length > 0) {
                pdf.setFont('helvetica', 'italic');
                pdf.text(`    Review: ${item.product.reviews[0].review}`, 20, currentY);
                pdf.setFont('helvetica', 'normal');
                currentY += 10;
            }
        });

        pdf.text(`Total Order Price: ${order.total_price}€`, 10, currentY + 10);

        pdf.save(`order_invoice_${order.order_id}.pdf`);

    }


    useEffect(() => {
        if (sessionStorage.getItem("session_token") == null) {
            navigate('/');
        }
        if (decrypted_data !== null) {
            setDecrypted(true);
        }
    }, [decrypted_data]);


    return (
        <>
            {got_orders ? (

                isDecrypted ? (
                    <>
                        <Container>
                            {decrypted_data.map((order) => (
                                <Card border="danger" bg="secondary" className="my-5 text-center"
                                      style={{borderWidth: "3px"}} key={order.order_id}>
                                    <Card.Body>
                                        <Card.Title className="fs-3" style={{color: "white"}}>Order
                                            no. {order.order_id}</Card.Title>
                                    </Card.Body>
                                    <h4>Products:</h4>
                                    <Row xs={1} md={2} lg={3} className="g-3 mx-3">
                                        {order.products.map((product) => (
                                            <Col key={product.product.id}>
                                                <Card key={product.product.id} style={{borderColor: "darkred"}}>
                                                    <Card.Header><Card.Subtitle>{product.product.name}</Card.Subtitle></Card.Header>
                                                    <ListGroup className="list-group-flush small">
                                                        <ListGroup.Item className="text-start">Product
                                                            Quantity: {product.quantity}</ListGroup.Item>
                                                        <ListGroup.Item className="text-start">Total Product
                                                            Price: {product.total_price} €</ListGroup.Item>
                                                    </ListGroup>
                                                    {product.product.reviews.length === 0 ? (
                                                        <Card.Body
                                                            style={{backgroundColor: "darkred", padding: 0}}>
                                                            <ListGroup className="list-group-flush small">
                                                                <ListGroup.Item style={{backgroundColor: "darkred"}}>
                                                                    <Card.Subtitle
                                                                        style={{color: "white"}}
                                                                        className="mb-2"
                                                                    >Rating</Card.Subtitle>
                                                                    <FormControl
                                                                        type="number"
                                                                        placeholder="Insert rating here ..."
                                                                        id={product.product.id}
                                                                        value={ratingFields[product.product.id] || ''}
                                                                        onChange={handleRatingInputChange(product.product.id)}
                                                                        min={0}
                                                                        max={5}
                                                                    />
                                                                </ListGroup.Item>
                                                                <ListGroup.Item style={{backgroundColor: "darkred"}}>
                                                                    <Card.Subtitle
                                                                        style={{color: "white"}}
                                                                        className="mb-2"
                                                                    >Review</Card.Subtitle>
                                                                    <Form.Control
                                                                        as="textarea"
                                                                        id={product.product.id}
                                                                        rows={3}
                                                                        cols={30}
                                                                        placeholder={"Insert your review here ..."}
                                                                        value={textfields[product.product.id] || ''}
                                                                        onChange={handleReviewTextareaChange(product.product.id)}
                                                                    />
                                                                </ListGroup.Item>
                                                            </ListGroup>
                                                            <Button
                                                                id={product.product.id}
                                                                variant="outline-danger"
                                                                onClick={(e) => sendReview(e)}
                                                                className="mb-2"
                                                            >Send review</Button>
                                                        </Card.Body>
                                                    ) : (
                                                        <Card.Body
                                                            style={{backgroundColor: "darkred", padding: 0}}>
                                                            <ListGroup className="list-group-flush small">
                                                                <ListGroup.Item style={{backgroundColor: "darkred"}}>
                                                                    <Card.Subtitle
                                                                        style={{color: "white"}}
                                                                        className="mb-2"
                                                                    >Rating</Card.Subtitle>
                                                                    <FormControl
                                                                        type="number"
                                                                        placeholder="Insert rating here ..."
                                                                        id={product.product.id}
                                                                        value={ratingFields[product.product.id] === undefined ? product.product.reviews[0].rating : (ratingFields[product.product.id] === '' ? '' : ratingFields[product.product.id])}
                                                                        onChange={handleRatingInputChange(product.product.id)}
                                                                    />
                                                                </ListGroup.Item>
                                                                <ListGroup.Item style={{backgroundColor: "darkred"}}>
                                                                    <Card.Subtitle
                                                                        style={{color: "white"}}
                                                                        className="mb-2"
                                                                    >Review</Card.Subtitle>
                                                                    <Form.Control
                                                                        as="textarea"
                                                                        id={product.product.id}
                                                                        rows={3}
                                                                        cols={30}
                                                                        placeholder={"Insert your review here ..."}
                                                                        value={textfields[product.product.id] === undefined ? product.product.reviews[0].review : (textfields[product.product.id] === '' ? '' : textfields[product.product.id])}
                                                                        onChange={handleReviewTextareaChange(product.product.id)}
                                                                    />
                                                                </ListGroup.Item>
                                                            </ListGroup>
                                                        </Card.Body>
                                                    )}
                                                </Card>
                                            </Col>
                                        ))}
                                    </Row>
                                    <h4 className="mt-3">Total Order Price:{' '}
                                        <span
                                            className="fw-bold text-decoration-underline">{parseFloat(order.total_price).toFixed(2)} €</span>
                                    </h4>
                                    <Card.Footer className="text-muted">
                                        <Button id={order.order_id} variant="danger" onClick={(e) => createPDF(e)}>Download
                                            PDF</Button>
                                    </Card.Footer>
                                </Card>
                            ))}
                            {error && <p>Review was not created successfully. {error.message}</p>}
                            {review_success && <p>Review was created successfully.</p>}
                        </Container>
                    </>
                ) : (
                    <Card className="my-5">
                        <Card.Header className="text-center">
                            <ListGroup className="list-group-flush">
                                <ListGroup.Item as="h5" style={{backgroundColor: 'unset'}}>Encrypted
                                    data</ListGroup.Item>
                                <ListGroup.Item style={{backgroundColor: 'unset'}}>
                                    <Form.Control as="textarea" rows={10} value={text_data}
                                                  onChange={handleDataTextareaChange} disabled={true}/>
                                </ListGroup.Item>
                            </ListGroup>
                        </Card.Header>
                        <Card.Body className="text-center">
                            <Card.Title as="h5">Insert your own Private key:</Card.Title>
                            <Card.Text>
                                <Form.Control as="textarea" rows={10} value={private_key}
                                              onChange={handleSKTextareaChange}/>
                            </Card.Text>
                            <Button variant="primary" onClick={(e) => showEncryptedOrders(e)}>Decrypt
                                Orders</Button>
                        </Card.Body>
                        <Card.Text className="ms-2">
                            {error && <p>{error.message}</p>}
                        </Card.Text>
                    </Card>
                )

            ) : (
                <Card className="mt-5">
                    <Card.Header as="h5" className="text-center">Insert your own Public key:</Card.Header>
                    <Card.Body className="text-center">
                        {/*<Card.Title>Special title treatment</Card.Title>*/}
                        <Card.Text>
                            <Form.Control as="textarea" rows={10} value={public_key}
                                          onChange={handlePKTextareaChange}/>
                        </Card.Text>
                        <Button variant="primary" onClick={(e) => receiveOrders(e)}>Receive Orders</Button>

                    </Card.Body>
                    <Card.Text className="ms-2">
                        {error && <p>{error.message}</p>}
                    </Card.Text>
                </Card>
            )}
        </>
    );

}