import Button from 'react-bootstrap/Button';
import Col from 'react-bootstrap/Col';
import Form from 'react-bootstrap/Form';
import Row from 'react-bootstrap/Row';
import Card from 'react-bootstrap/Card';
import {useEffect, useState} from 'react';
import {Fernet} from 'fernet-ts';
import {encrypt_data, get_server_PK, decrypt_by_symmetric_key} from "./utils.ts";
import {useNavigate} from 'react-router-dom';
import { useAuth } from '../context/AuthContext.tsx';
import Encryption_types from "./encryption_types.ts";


export function LoginForm() {

    const [server_public_key, setServerPublicKey] = useState("");
    const [loading, setLoading] = useState(false);
    const [error, setError] = useState(null);
    const [login_success, setLoginSuccess] = useState(false);
    const navigate = useNavigate();
    const [formData, setFormData] = useState({
        email: '',
        password: ''
    });
    const { loginAu, logoutAu } = useAuth();

    const handleInputChange = (e: any) => {
        const {name, value} = e.target;
        setFormData({...formData, [name]: value});
    };

    const signIn = (e: any) => {

        e.preventDefault();
        if (sessionStorage.getItem("session_token") !== null){
            navigate('/');
            return;
        }
        setLoading(true);
        setError(null);
        setLoginSuccess(false);
        let type: Encryption_types=Encryption_types.Login;
        const symmetric_key = Fernet.generateSecret();
        encrypt_data(formData, server_public_key, type, symmetric_key).then((encoded_data) => {
            fetch('http://localhost:8000/api/login', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify(encoded_data),
            }).then((response) => {
                if (!response.ok) {
                    return response.json().then(response => {
                        if (response.hasOwnProperty("detail")){
                            throw new Error(response.detail)
                        }
                        throw new Error("Unexpected error while logging in. Try again please.")
                    });
                }
                return response.json();
            }).then((response) => {
                decrypt_by_symmetric_key(response.data, symmetric_key).then((decrypted_data) =>{
                    const json_data = JSON.parse(decrypted_data);
                    sessionStorage.setItem('session_token', json_data.token);
                    sessionStorage.setItem('id', json_data.user_information.id);
                    sessionStorage.setItem('first_name', json_data.user_information.first_name);
                    sessionStorage.setItem('last_name', json_data.user_information.last_name);
                    sessionStorage.setItem('email', json_data.user_information.email);
                    sessionStorage.setItem('address', json_data.user_information.address);
                    sessionStorage.setItem('phone', json_data.user_information.phone);
                    setLoading(false);
                    setLoginSuccess(true);
                    navigate('/');
                }).catch((error) => {
                    setError(error.message);
                    setLoading(false);
                    setLoginSuccess(false);
                })
            })
            .catch((error) => {
                setError(error);
                setLoading(false);
                setLoginSuccess(false);
            });
        });

    }

    useEffect(() => {
        if (sessionStorage.getItem("session_token") !== null){
            loginAu();
        } else {
            logoutAu();
        }
         get_server_PK().then((data) => {
             setServerPublicKey(data.public_key);
             sessionStorage.setItem('server_public_key', data.public_key);
         }).catch((error) => {
                 setError(error);
         })
     }, []);

    return (
        <Card className='mx-auto' style={{width: '36rem'}}>
            <Card.Body>
                <Form onSubmit={signIn}>
                    <Form.Group as={Row} className="mb-3" controlId="formHorizontalEmail">
                        <Col>
                            <Form.Label className='ms-2 h6'>
                                Email
                            </Form.Label>
                            <Form.Control
                                type="email"
                                name='email'
                                value={formData.email}
                                onChange={handleInputChange}
                                placeholder="example@mail.com"
                                required={true}/>
                        </Col>
                    </Form.Group>

                    <Form.Group as={Row} className="mb-3" controlId="formHorizontalPassword">
                        <Col>
                            <Form.Label className='ms-2 h6'>
                                Password
                            </Form.Label>
                            <Form.Control
                                type="password"
                                name='password'
                                value={formData.password}
                                onChange={handleInputChange}
                                placeholder="***************"
                                required={true}/>
                        </Col>
                    </Form.Group>

                    <Form.Group as={Row} className="mb-3" controlId="formHorizontalCheck">
                        <Col>
                            <Form.Check label="Remember me"/>
                        </Col>
                    </Form.Group>

                    <Form.Group as={Row} className="mb-3 text-center">
                        <Col>
                            <Button type="submit" /*onClick={(e) => signIn(e)}*/>Log in</Button>
                        </Col>
                    </Form.Group>
                </Form>
                {loading && <p>Sending data...</p>}
                {error && <p>{error.message}</p>}
                {!loading && login_success && <p>{sessionStorage.getItem('session_token')}</p>}
            </Card.Body>
        </Card>
    );
}