import {useNavigate} from 'react-router-dom';
import {useEffect, useState} from "react";
import Button from "react-bootstrap/Button";
import Row from "react-bootstrap/Row";
import Card from "react-bootstrap/Card";
// import Form from "react-bootstrap/Form";
import Col from "react-bootstrap/Col";
import {Container} from "react-bootstrap";
import {useShoppingCart} from "../context/ShoppingCartContext.tsx";

export function LogoutForm() {

    const [error, setError] = useState(null);
    const navigate = useNavigate();
    const {emptyCart} = useShoppingCart()

    const logout = (e: any) => {
        e.preventDefault();
        setError(null);
        let session_token = sessionStorage.getItem("session_token");
        if (session_token == null) {
            navigate('/');
            return;
        }
        let user_token = {
            token: session_token,
        };
        fetch('http://localhost:8000/api/logout', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(user_token),
        }).then((response) => {
            if (!response.ok) {
                return response.json().then(response => {
                    if (response.hasOwnProperty("detail")) {
                        throw new Error(response.detail)
                    }
                    throw new Error("Unexpected error while logging out. Try again please.")
                });
            }
            return response.json();
        }).then(() => {
            sessionStorage.removeItem("session_token");
            sessionStorage.removeItem('id');
            sessionStorage.removeItem('first_name');
            sessionStorage.removeItem('last_name');
            sessionStorage.removeItem('email');
            sessionStorage.removeItem('address');
            sessionStorage.removeItem('phone');
            sessionStorage.removeItem('server_public_key');
            //TODO
            //vymazat cart zo session aj local storage
            emptyCart();
            navigate('/');
        })
            .catch((error) => {
                setError(error);
            });
    };

    useEffect(() => {
        if (sessionStorage.getItem("session_token") == null) {
            navigate('/');
        }
    }, []);

    return (
        <>
            <Container>
                <Row>
                    <Col></Col>
                    <Col xs={6}>
                        <Card className='mx-auto'>
                            <Card.Header as="h5" className="text-center">
                                Are you sure you want to log out?
                            </Card.Header>
                            <Card.Body>
                                <Row className='my-5 mx-auto' style={{width: '7rem'}}>
                                    <Button onClick={(e) => logout(e)}>Log out</Button>
                                </Row>
                                {error && <p>{error.message}</p>}
                            </Card.Body>
                        </Card>
                    </Col>
                    <Col></Col>
                </Row>
            </Container>
        </>
    );
}