import {Container, Nav, Navbar as NavbarBs} from "react-bootstrap";
import {NavLink} from "react-router-dom";
import {useEffect, useState} from "react";
import {useAuth} from '../context/AuthContext.tsx';
import {useShoppingCart} from "../context/ShoppingCartContext.tsx";


export function Navbar() {

    // const [logged_in, setLoggedIn] = useState(false);
    const {isLoggedIn, loginAu, logoutAu} = useAuth();
    const {openCart, cartQuantity, isDecrypted} = useShoppingCart();

    useEffect(() => {
        if (sessionStorage.getItem("session_token") !== null) {
            // setLoggedIn(true);
            loginAu();
            // console.log("token nie je null");
        } else {
            logoutAu();
            // console.log("token je null");
        }
    }, []);

    return (
        <>
            <NavbarBs sticky="top" className="bg-white shadow-sm mb-3">
                <Container>
                    <NavbarBs.Brand href="#" style={{ display: "flex", alignItems: "center" }}>
                        <img
                            alt=""
                            src="/img.png"
                            // width="30"
                            // height="30"
                            className="d-inline-block align-top"
                            style={{width: "2.5rem", height: "2.5rem"}}
                        />
                        <span style={{ marginLeft: '10px' }}>E-SHOP</span>
                    </NavbarBs.Brand>
                    <Nav className="me-auto">
                        <Nav.Link to="/" as={NavLink}>
                            Home
                        </Nav.Link>
                        {isLoggedIn && (
                            <>
                            <Nav.Link to="/store" as={NavLink}>
                                Store
                            </Nav.Link>
                            <Nav.Link to="/client-zone" as={NavLink}>
                                Client zone
                            </Nav.Link>
                            </>
                        )}
                    </Nav>
                    {cartQuantity > 0 && isDecrypted && <NavbarBs.Brand
                        // href=""
                        onClick={openCart}
                        style={{position: "relative", cursor: 'pointer'}}
                        className="mx-4">
                        <img
                            src="/cart.png"
                            alt=""
                            // width="30"
                            // height="30"
                            className="d-inline-block align-top"
                            style={{width: "2.5rem", height: "2.5rem"}}
                        />
                        <div
                            className="rounded-circle bg-danger d-flex justify-content-center align-items-center small"
                            style={{
                                color: "white",
                                width: "1.5rem",
                                height: "1.5rem",
                                position: "absolute",
                                bottom: 0,
                                right: 0,
                                transform: "translate(25%, -115%)"
                        }}>
                            {cartQuantity}
                        </div>
                    </NavbarBs.Brand>}
                    {isLoggedIn ? (
                        <NavbarBs.Brand href="/logout">
                            <img
                                alt=""
                                src="/login.png"
                                // width="30"
                                // height="30"
                                className="d-inline-block align-top"
                                style={{width: "2.5rem", height: "2.5rem"}}
                            />{' '}
                        </NavbarBs.Brand>
                    ) : (
                        <NavbarBs.Brand href="/login">
                            <img
                                alt=""
                                src="/login.png"
                                // width="30"
                                // height="30"
                                className="d-inline-block align-top"
                                style={{width: "2.5rem", height: "2.5rem"}}
                            />{' '}
                        </NavbarBs.Brand>
                    )}

                </Container>
            </NavbarBs>
        </>
    );
}