import {Modal, Offcanvas, Stack} from "react-bootstrap";
import {useShoppingCart} from "../context/ShoppingCartContext.tsx";
import {CartItem} from "./CartItem.tsx";
import Button from "react-bootstrap/Button";
import {useState} from "react";
import Encryption_types from "./encryption_types.ts";
import {encrypt_data} from "./utils.ts";
import {useNavigate} from 'react-router-dom';

type ShoppingCartProps = {
    isOpen: boolean
}

export function ShoppingCart({isOpen}: ShoppingCartProps) {

    const [show, setShow] = useState(false);
    const [loading, setLoading] = useState(false);
    const [error, setError] = useState(null);
    const [order_success, setOrderSuccess] = useState(false);
    const navigate = useNavigate();

    const {jsonData} = useShoppingCart()
    const {closeCart, cartItems, emptyCart} = useShoppingCart()

    // const user_id = sessionStorage.getItem('id');
    const user_first_name = sessionStorage.getItem('first_name');
    const user_last_name = sessionStorage.getItem('last_name');
    const user_email = sessionStorage.getItem('email');
    const user_address = sessionStorage.getItem('address');
    const user_phone = sessionStorage.getItem('phone');

    const handleClose = () => setShow(false);
    const handleShow = () => {
        setShow(true);
        setLoading(false);
        setError(null);
        setOrderSuccess(false);
        closeCart();
    };

    const handleRedirect = () => {
        setLoading(false);
        setError(null);
        setOrderSuccess(false);
        setShow(false);
        navigate('/');
    }

    function renameKey ( obj, oldKey, newKey ) {
      obj[newKey] = obj[oldKey];
      delete obj[oldKey];
    }

    const placeOrder = () => {
        setLoading(true);
        setError(null);
        setOrderSuccess(false);
        {cartItems.map((item) => {
            const cartItem = jsonData.find(i => i.id === item.id);
        })}
        let json_orderData = {
            order_data: cartItems
        }
        let order_string = JSON.stringify(json_orderData)
        let order = JSON.parse(order_string)
        order.order_data.forEach( obj => renameKey( obj, 'id', 'product_id' ) );
        let type: Encryption_types=Encryption_types.Register;

        encrypt_data(order, sessionStorage.getItem('server_public_key'), type).then((encoded_data) => {
            fetch('http://localhost:8000/api/order', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'Authorization': `Token ${sessionStorage.getItem("session_token")}`,
                },
                body: JSON.stringify(encoded_data),
            }).then((response) => {
                if (!response.ok) {
                    return response.json().then(response => {
                        if (response.hasOwnProperty("detail")){
                            throw new Error(response.detail);
                        }
                        throw new Error("Order was not placed")
                    });
                }
                return response.json();
            }).then(() => {
                setLoading(false);
                setOrderSuccess(true);
                //TODO
                //vymazat z kosika vsetky produkty (objednavka bola odoslana)

                emptyCart();
            })
            .catch((error) => {
                setError(error);
                setLoading(false);
                setOrderSuccess(false);
            });
        });
    };

    return (
        !order_success ? (
            <>
            <Offcanvas show={isOpen} onHide={closeCart} placement="end">
                <Offcanvas.Header closeButton>
                    <Offcanvas.Title>Cart</Offcanvas.Title>
                </Offcanvas.Header>
                <Offcanvas.Body>
                    <Stack>
                        {cartItems.map(item => {
                            console.log(cartItems);
                            return (<CartItem key={item.id} {...item} />)
                        })}
                        <div className="ms-auto fw-bold fs-5">
                            Total: {cartItems.reduce((total, cartItem) => {
                            const item = jsonData.find(i => i.id === cartItem.id)
                            return total + (item?.price || 0) * cartItem.quantity
                        }, 0).toFixed(2)} €
                        </div>
                        {cartItems.length > 0 && <Button variant="primary" onClick={handleShow}>Checkout</Button>}
                    </Stack>
                </Offcanvas.Body>
            </Offcanvas>
            <Modal
                show={show}
                onHide={handleClose}
                backdrop="static"
                keyboard={false}
            >
                <Modal.Header closeButton>
                    <Modal.Title>CHECKOUT</Modal.Title>
                </Modal.Header>
                <Modal.Body>
                    <h5>Product Summary</h5>
                    <ul>
                        {cartItems.map((item) => {
                            const cartItem = jsonData.find(i => i.id === item.id)
                            return cartItem ? (
                                <li key={item.id}>
                                    {cartItem.name} x{item.quantity} = {(cartItem.price * item.quantity).toFixed(2)} €
                                </li>
                            ) : null
                        })}
                    </ul>
                    <hr/>
                    <div className="ms-auto fw-bold fs-6 text-end">
                        Total: {cartItems.reduce((total, cartItem) => {
                        const item = jsonData.find(i => i.id === cartItem.id)
                        return total + (item?.price || 0) * cartItem.quantity
                    }, 0).toFixed(2)} €
                    </div>
                    <hr/>
                    <h5>Personal Information</h5>
                    <ul>
                        {/*{ user_id ? (<li>UID : {user_id}</li>) : null}*/}
                        { user_first_name ? (<li>Name : {user_first_name}</li>) : null}
                        { user_last_name ? (<li>Surname : {user_last_name}</li>) : null}
                        { user_address ? (<li>Address : {user_address}</li>) : null}
                        { user_email ? (<li>Email : {user_email}</li>) : null}
                        { user_phone ? (<li>Phone : {user_phone}</li>) : null}
                    </ul>
                </Modal.Body>
                <Modal.Footer>
                    <Button variant="outline-secondary" onClick={handleClose}>
                        Close
                    </Button>
                    <Button variant="outline-success" onClick={placeOrder}>Place order</Button>
                    {loading && <p>Sending data...</p>}
                    {error && <p>{error.message}</p>}

                </Modal.Footer>
            </Modal>
        </>
            ):(
                <>
            <Modal
                show={show}
                backdrop="static"
                keyboard={false}
            >
                <Modal.Header closeButton>
                    <Modal.Title>Order Done</Modal.Title>
                </Modal.Header>
                <Modal.Body>
                    <h5>Your order was placed</h5>
                </Modal.Body>
                <Modal.Footer>
                    <Button variant="outline-secondary" onClick={handleRedirect}>
                        Close
                    </Button>
                </Modal.Footer>
            </Modal>
        </>
        )

    )
}