import React, {useEffect, useState} from "react";
import Button from "react-bootstrap/Button";
import {useNavigate} from 'react-router-dom';

import {
    is_valid_public_key_pem,
    send_pem_get_products,
    is_valid_text_data,
    decrypt_products,
} from "./utils.ts";
import Card from "react-bootstrap/Card";
import ListGroup from 'react-bootstrap/ListGroup';
import Form from 'react-bootstrap/Form';
import Table from 'react-bootstrap/Table';
import {StoreItem} from "./StoreItem.tsx";
import {useShoppingCart} from "../context/ShoppingCartContext.tsx";
import {FormControl} from "react-bootstrap";

export function StoreForm() {

    const [public_key, setPublicKey] = useState<string>('');
    const [private_key, setPrivateKey] = useState<string>('');
    const [text_data, setTextData] = useState<string>('');
    const [error, setError] = useState(null);
    const [got_products, setGotProducts] = useState(false);
    // const [isDecrypt, setDecrypt] = useState(false);
    const [encrypted_data, setEncryptedData] = useState<string>('');
    const [encrypted_key, setEncryptedKey] = useState<string>('');
    // const [decrypted_data, setDecryptedData] = useState([]);
    const navigate = useNavigate();
    const {jsonData, setDecryptedData, setfilteredData, filteredData, isDecrypted, setDecrypted, isFiltered, setFiltered} = useShoppingCart()
    const [searchString, setSearchString] = useState("");
    const [emptySearchBar, setEmptySearchBar] = useState(true);

    const handleKeyDown = (event) => {
        if (event.key === 'Enter') {
            event.preventDefault();
        }
    };

    const handlePKTextareaChange = (event: React.ChangeEvent<HTMLTextAreaElement>) => {
        setPublicKey(event.target.value);
    };
    const handleSKTextareaChange = (event: React.ChangeEvent<HTMLTextAreaElement>) => {
        setPrivateKey(event.target.value);
    };
    const handleDataTextareaChange = (event: React.ChangeEvent<HTMLTextAreaElement>) => {
        setTextData(event.target.value);
    };

    const receiveProducts = (e: any) => {
        e.preventDefault();
        if (sessionStorage.getItem("session_token") == null) {
            navigate('/');
            return;
        }
        setError(null);
        try {
            is_valid_public_key_pem(public_key);
        } catch (er: any) {
            setError(er);
            return;
        }
        send_pem_get_products(public_key, searchString).then((data) => {
            setGotProducts(true);
            setEncryptedData(data.encrypted_data);
            setTextData(data.encrypted_data);
            setEncryptedKey(data.encrypted_key);
        }).catch((er) => {
            setError(er);
        });
    }

    const showEncryptedProducts = (e: any) => {
        e.preventDefault();
        setError(null);
        try {
            is_valid_text_data(text_data, encrypted_data);
        } catch (er: any) {
            setError(er);
            return;
        }
        try {
            decrypt_products(encrypted_key, private_key, text_data).then((decrypted_data) => {
                setDecrypted(true);
                const dataArray = JSON.parse(decrypted_data);
                setDecryptedData(dataArray);

            }).catch((er) => {
                setError(er);
            })
        } catch (er: any) {
            setError(er);
            return;
        }
    }

    const showFilteredProducts = (searchStr: string) => {
        setError(null);
        try {
            is_valid_public_key_pem(public_key);
        } catch (er: any) {
            setError(er);
            return;
        }
        console.log('data sent to server: ' + searchStr)
        send_pem_get_products(public_key, searchStr).then((data) => {

            setEncryptedData(data.encrypted_data);
            //setTextData(data.encrypted_data);
            setEncryptedKey(data.encrypted_key);
            try {
                decrypt_products(data.encrypted_key, private_key, data.encrypted_data).then((decrypted_data) => {
                    const dataArray = JSON.parse(decrypted_data);
                    console.log(dataArray);
                    setFiltered(true);
                    setfilteredData(dataArray);

                }).catch((er) => {
                    setError(er);
                })
            } catch (er: any) {
                setError(er);
                return;
            }
        }).catch((er) => {
                setError(er);
            });
    }

    const handleSearchChange = (newSearchString: string) => {
        // Update the search string in the parent component state
        setSearchString(newSearchString);
        if ((newSearchString === "") && emptySearchBar) {
            return;
        }
        if ((newSearchString === "") && (!emptySearchBar)) {
            showFilteredProducts(newSearchString);
            setEmptySearchBar(true);
            return;
        }
        if (newSearchString !== "") {
            showFilteredProducts(newSearchString);
            setEmptySearchBar(false);
            return;
        }
    };

    const columns = ['Name', 'Description', 'Price', 'Stock', '', ''];

    useEffect(() => {
        if (sessionStorage.getItem("session_token") == null) {
            navigate('/');
        }
    }, []);


    return (
        <>
            {got_products ? (

                isDecrypted ? (
                    <>
                        <FormControl
                            type="text"
                            placeholder="Search ..."
                            className="mr-sm-2 w-50 mx-auto mt-5"
                            value={searchString}
                            onKeyDown={handleKeyDown}
                            onChange={(e) => handleSearchChange(e.target.value)}
                        />
                        <Table responsive className="my-5" variant="dark" bordered>
                            <thead>
                            <tr>
                                {columns.map((item, index) => (
                                    <th key={index} style={{backgroundColor: '#00fff6', color: 'black'}}>{item}</th>
                                ))}
                            </tr>
                            </thead>
                            <tbody id='encryptedProductsList'>
                            {isFiltered ? (
                                filteredData.map((item, index) =>
                                <StoreItem key={index} {...item} />
                            )
                            ) : (jsonData.map((item, index) =>
                                <StoreItem key={index} {...item} />
                            ))}
                            </tbody>
                        </Table>
                    </>

                ) : (
                    <Card className="my-5">
                        <Card.Header className="text-center">
                            <ListGroup className="list-group-flush">
                                <ListGroup.Item as="h5" style={{backgroundColor: 'unset'}}>Encrypted
                                    data</ListGroup.Item>
                                <ListGroup.Item style={{backgroundColor: 'unset'}}>
                                    <Form.Control as="textarea" rows={10} value={text_data}
                                                  onChange={handleDataTextareaChange} disabled={true}/>
                                </ListGroup.Item>
                            </ListGroup>
                        </Card.Header>
                        <Card.Body className="text-center">
                            <Card.Title as="h5">Insert your own Private key:</Card.Title>
                            <Card.Text>
                                <Form.Control as="textarea" rows={10} value={private_key}
                                              onChange={handleSKTextareaChange}/>
                            </Card.Text>
                            <Button variant="primary" onClick={(e) => showEncryptedProducts(e)}>Decrypt
                                Products</Button>
                        </Card.Body>
                        <Card.Text className="ms-2">
                            {error && <p>{error.message}</p>}
                        </Card.Text>
                    </Card>
                )

            ) : (
                <Card className="mt-5">
                    <Card.Header as="h5" className="text-center">Insert your own Public key:</Card.Header>
                    <Card.Body className="text-center">
                        {/*<Card.Title>Special title treatment</Card.Title>*/}
                        <Card.Text>
                            <Form.Control as="textarea" rows={10} value={public_key}
                                          onChange={handlePKTextareaChange}/>
                        </Card.Text>
                        <Button variant="primary" onClick={(e) => receiveProducts(e)}>Receive Products</Button>

                    </Card.Body>
                    <Card.Text className="ms-2">
                        {error && <p>{error.message}</p>}
                    </Card.Text>
                </Card>
            )}
        </>
    );
}