import {Fernet} from 'fernet-ts';
// @ts-ignore
import * as Forge from 'node-forge'
import Encryption_types from "./encryption_types.ts";


export function is_valid_public_key_pem(pem_public_key: string): any {
  try {
    const public_key = Forge.pki.publicKeyFromPem(pem_public_key);
    if (public_key != null){
        return true;
    }
    return true;
  } catch (error) {
    throw new Error('Invalid PEM format');
  }
}

export function valid_private_key_from_pem(pem_private_key: string): any {
  try {
      const private_key = Forge.pki.privateKeyFromPem(pem_private_key);
      return private_key;
  } catch (error) {
      throw new Error('Invalid PEM format');
  }
}

export function is_valid_text_data(text_area_data: string, received_data: string): any {
  if (text_area_data === received_data){
      return true;
  }else {
      throw new Error('Data in text area do not match');
  }
}

export function get_server_PK(): Promise<any> {
  return fetch('http://localhost:8000/api/public-key') // Replace with your API endpoint
      .then((response) => {
        if (!response.ok) {
          throw new Error('Error getting servers public key');
        }
        return response.json()
        })
      .catch(() => {
        throw new Error('Error getting servers public key');
      });
}


export function encrypt_data(data: object, server_public_key: string, type:Encryption_types, s_key: any=null): Promise<{ data: string; symmetric_key: string }> {
    if ((type === "Register") || (type === "Review")){
        s_key = Fernet.generateSecret();
    }
    //s_key = Fernet.generateSecret();
      let body = JSON.stringify(data);
      const encrypt = async () => {
          const f = await Fernet.getInstance(s_key);
          return await f.encrypt(body);
      }
      return encrypt().then((encrypted_data) => {
        //console.log("key: " + s_key);
        //console.log(token);

        const publicKey = Forge.pki.publicKeyFromPem(server_public_key);
        let encrypted_symmetric_key = publicKey.encrypt(s_key, 'RSA-OAEP', {
          md: Forge.md.sha256.create(),
        });
        encrypted_data = Forge.util.encode64(encrypted_data);
        encrypted_symmetric_key = Forge.util.encode64(encrypted_symmetric_key);
        let json_to_server = {
          data: encrypted_data,
          symmetric_key: encrypted_symmetric_key,
        };
        return json_to_server;
      });
}

export function send_pem_get_products(pem_public_key: string, search_string: string = ""){
    let json;
    if (search_string !== ""){
        json = {
            public_key: pem_public_key,
            product_name_substring: search_string
        };
    }else {
        json = {
            public_key: pem_public_key
        };
    }
    return fetch('http://localhost:8000/api/encrypted/products', {
                method: 'POST',
                headers: {
                    'Authorization': `Token ${sessionStorage.getItem("session_token")}`,
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify(json),
            })
      .then((response) => {
        if (!response.ok) {
          return response.json().then(response => {
                if (response.hasOwnProperty("detail")){
                    throw new Error(response.detail);
                }
                throw new Error("Unexpected error while receiving products. Try again please.")
            });
        }
        return response.json()
        })
      .catch((e) => {
        throw new Error(e.message);
      });
}

export function send_pem_get_orders(pem_public_key: string){
    let json = {
        public_key: pem_public_key
    };
    return fetch('http://localhost:8000/api/orders', {
                method: 'POST',
                headers: {
                    'Authorization': `Token ${sessionStorage.getItem("session_token")}`,
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify(json),
            })
      .then((response) => {
        if (!response.ok) {
          return response.json().then(response => {
                if (response.hasOwnProperty("detail")){
                    throw new Error(response.detail);
                }
                throw new Error("Unexpected error while receiving orders. Try again please.")
            });
        }
        return response.json()
        })
      .catch((e) => {
        throw new Error(e.message);
      });
}

export function decrypt_products(encrypted_key: string, private_key_pem: string, data: string){
    let private_key;
    const decrypt_data = async (symmetric_key: string) => {
        const f = await Fernet.getInstance(symmetric_key);
        return await f.decrypt(data);
    }
    try {
        private_key = valid_private_key_from_pem(private_key_pem);
    } catch (er: any){
        throw new Error(er.message);
    }
    let symmetric_key;
    try {
        symmetric_key = private_key.decrypt(Forge.util.decode64(encrypted_key), 'RSA-OAEP', {
            md: Forge.md.sha256.create(),
            mgf1: {
                md: Forge.md.sha256.create(),
            },
        });
    } catch (er){
        throw new Error("Incorrect private key");
    }

    return decrypt_data(symmetric_key).then((decrypted_data) => {
        return decrypted_data;
    }).catch((er) => {
        throw new Error(er.message);
    })

}

export function decrypt_by_symmetric_key(data: string, symmetric_key: string){
    const decrypt_data = async () => {
        const f = await Fernet.getInstance(symmetric_key);
        return await f.decrypt(data);
    }
    return decrypt_data().then((decrypted_data) => {
        return decrypted_data;
    }).catch((er) => {
        throw new Error(er.message);
    })


}