import {createContext, ReactNode, useContext, useState} from 'react';

type AuthProviderProps = {
    children: ReactNode
}

type AuthContext = {
    isLoggedIn: boolean,
    loginAu: () => void,
    logoutAu: () => void
}

const AuthContext = createContext({} as AuthContext);

export function useAuth() {
  return useContext(AuthContext);
}

export function AuthProvider({ children }: AuthProviderProps) {
  const [isLoggedIn, setIsLoggedIn] = useState(false);

  const loginAu = () => {
    setIsLoggedIn(true);
  };

  const logoutAu = () => {
    setIsLoggedIn(false);
  };

  return (
    <AuthContext.Provider value={{ isLoggedIn, loginAu, logoutAu }}>
      {children}
    </AuthContext.Provider>
  );
}
