from datetime import datetime, timedelta
from typing import Annotated, Union

from fastapi import Form, HTTPException, Request
from fastapi.security import OAuth2PasswordBearer
from jose import jwt

SECRET = "18cb5e02e437cd4853c8a37154920aabdc761f902abd79bc8fe8cc813a7aab86"
ALGORITHM = "HS256"
ACCESS_TOKEN_EXPIRE_MINUTES = 30


class OAuth2PasswordCookie(OAuth2PasswordBearer):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    async def __call__(self, request: Request) -> str:
        token = request.cookies.get("access_token")

        if not token:
            raise HTTPException(status_code=403, detail="Not authenticated")

        return token


class OAuth2MailPasswordRequestForm:
    def __init__(
        self,
        *,
        grant_type: Annotated[
            Union[str, None],
            Form(pattern="password"),
        ] = None,
        mail: Annotated[
            str,
            Form(),
        ],
        password: Annotated[
            str,
            Form(),
        ],
        scope: Annotated[
            str,
            Form(),
        ] = "",
        client_id: Annotated[
            Union[str, None],
            Form(),
        ] = None,
        client_secret: Annotated[
            Union[str, None],
            Form(),
        ] = None,
    ):
        self.grant_type = grant_type
        self.mail = mail
        self.password = password
        self.scopes = scope.split()
        self.client_id = client_id
        self.client_secret = client_secret


def create_access_token(data: dict, expires_delta: timedelta | None = None) -> str:
    data = data.copy()

    if expires_delta:
        expire = datetime.utcnow() + expires_delta
    else:
        expire = datetime.utcnow() + timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES)

    data.update({"exp": expire})

    token = jwt.encode(data, SECRET, algorithm=ALGORITHM)

    return token

