import { Component, OnInit, ViewChild } from '@angular/core';
import { FormGroup, FormControl } from '@angular/forms';
import { HttpClient } from '@angular/common/http';
import { Router } from '@angular/router';
import { DataSharingService } from '../data-sharing.service';
import { map, startWith } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { AuthService } from '../auth.service';
import { CustomModalComponent } from 'src/app/custom-modal/custom-modal.component';
import { ChangeDetectorRef } from '@angular/core';

@Component({
  selector: 'app-landing-page',
  templateUrl: './landing-page.component.html',
  styleUrls: ['./landing-page.component.css']
})
export class LandingPageComponent implements OnInit {

  public type: string;
  public arrival_city: string;
  public departure_date: string;
  public arrival_date: string;
  public price: string;
  public city: string;

  public isLoggedIn: boolean = false;
  

  constructor(private http: HttpClient, private router: Router, private dataSharingService: DataSharingService, private authService: AuthService, private changeDetector: ChangeDetectorRef) {
    this.type = '';
    this.arrival_city = '';
    this.departure_date = '';
    this.arrival_date = '';
    this.price = '';
    this.city = '';
    this.authService.isLoggedInObservable.subscribe(
      (loginStatus) => {
        this.isLoggedIn = loginStatus;
      }
    );
  }

  onSuggestionClick(suggestion: string): void {
    this.arrival_city = suggestion;
    this.showDropdown1 = false;
  }

  filterForm: FormGroup | undefined;

  europeanCities = [
    'Paris', 'London', 'Berlin', 'Madrid', 'Rome',
    'Vienna', 'Budapest', 'Prague', 'Warsaw', 'Oslo',
    'Athens', 'Lisbon', 'Dublin', 'Brussels', 'Amsterdam'
  ];

  travelTypes = ['Bus', 'Plane', 'Train', 'MHD'];

  filteredDestinations!: Observable<string[]>;
  filteredCities!: Observable<string[]>;
  filteredTypes!: Observable<string[]>;

  @ViewChild(CustomModalComponent)
  customModal: CustomModalComponent = new CustomModalComponent;


  ngOnInit(): void {
    this.filteredDestinations = this.createFilterObservable(this.arrival_city, this.europeanCities);
    this.filteredCities = this.createFilterObservable(this.city, this.europeanCities);
    this.filteredTypes = this.createFilterObservable(this.type, this.travelTypes);
    this.filterForm = new FormGroup({
      type: new FormControl(''),
      arrival_city: new FormControl(''),
      departure_date: new FormControl(''),
      arrival_date: new FormControl(''),
      price: new FormControl(''),
      city: new FormControl(''),
    });
  }

  showDropdown1: boolean = false;
  showDropdown2: boolean = false;
  showDropdown3: boolean = false;

  selectSuggestion(suggestion: string) {
    this.arrival_city = suggestion;
  }

  createFilterObservable(value: string, list: string[]): Observable<string[]> {
    return new Observable<string>((subscriber) => {
      subscriber.next(value);
    }).pipe(
      startWith(''),
      map(input => this.filterList(input, list))
    );
  }

  filterList(value: string, list: string[]): string[] {
    const filterValue = value.toLowerCase();
    return list.filter(item => item.toLowerCase().includes(filterValue));
  }

  onSubmit() {

    if (this.isLoggedIn) {
      const formData: FormData = new FormData();

    formData.append('type', this.type);
    formData.append('arrival_city', this.arrival_city);
    formData.append('departure_date', this.departure_date);
    formData.append('arrival_date', this.arrival_date);
    formData.append('price', this.price);
    formData.append('city', this.city);


    this.http.post<any>('api/route', formData)
      .subscribe({
        next: (response) => {
          this.dataSharingService.setFilteredRoutes(response);
          this.router.navigate(['/filtered-routes']);
        },
        error: (error) => {
          //console.error('Error fetching filtered routes', error);
          this.customModal.open("Chyba", "Nepodarilo sa načítať vyhľadané trasy.");
        }
      });
    } else {
      this.customModal.open("Chyba", "Pre vyhľadávanie musíte byť prihlásený.");
    }
  }
}
