import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { HttpClient } from '@angular/common/http';
import { Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { CustomModalComponent } from 'src/app/custom-modal/custom-modal.component';

@Component({
  selector: 'app-payment',
  templateUrl: './payment.component.html',
  styleUrls: ['./payment.component.css']
})
export class PaymentComponent implements OnInit {

  invoiceId: number | undefined;
  paymentForm: FormGroup;

  constructor(private route: ActivatedRoute, private http: HttpClient, private router: Router, private formBuilder: FormBuilder) {
    this.paymentForm = this.formBuilder.group({
      cardNumber: ['', [Validators.required, Validators.pattern(/^\d{4} \d{4} \d{4} \d{4}$/)]],
      cardExpiration: ['', [Validators.required, Validators.pattern(/^(0[1-9]|1[0-2])\/\d{2}$/)]],
      cardCVV: ['', [Validators.required, Validators.pattern(/^\d{3}$/)]]
    });
  }

  @ViewChild(CustomModalComponent)
  customModal: CustomModalComponent = new CustomModalComponent;

  ngOnInit(): void {
    this.invoiceId = this.route.snapshot.params['invoiceId'];
    
  }

  submitPayment() {
    if (this.paymentForm.valid){
      this.http.post(`api/pay/${this.invoiceId}`, {})
      .subscribe({
        next: () => {
          this.router.navigate(['/profile']);
          this.customModal.open("Platba", "Platba bola úspešná. \n Ďakujeme za nákup.");
        },
        error: (error) => {
          this.customModal.open("Chyba", "Platba sa nepodarila.");
        }
      });
    } else {
      this.customModal.open("Chyba", "Formulár nie je správne vyplnený. <br> Správny formát je: <br> Číslo karty: 1234 1234 1234 1234 <br> Dátum expirácie: 01/23 <br> CVV: 123.");
    }
    
  }

}
