import json
from django.contrib.auth import authenticate
from django_ratelimit.decorators import ratelimit

from rest_framework import status
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import AllowAny
from rest_framework.permissions import IsAuthenticated
from rest_framework_simplejwt.tokens import RefreshToken

from volby_backend.models import User
from volby_backend.serializers import UserRegistrationSerializer
from volby_backend.utils.responses import ErrorResponse, SuccessResponse
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework import status
from rest_framework_simplejwt.tokens import RefreshToken, AccessToken


@api_view(["POST"])
@permission_classes([AllowAny])
def register(request):
    user = User.objects.filter(id_card_number=request.data["id_card_number"]).first()
    if not user or not user.voting_rights:
        return ErrorResponse(
            status.HTTP_401_UNAUTHORIZED,
            "Bad Request",
            "User with this ID card number and birth number can't vote online!!!",
        )

    if user.birth_number != request.data["birth_number"]:
        return ErrorResponse(
            status.HTTP_400_BAD_REQUEST,
            "Bad Request",
            "User with this ID card number and birth number can't vote online!!!",
        )

    if user.email:
        return ErrorResponse(
            status.HTTP_403_FORBIDDEN,
            "Bad Request",
            "User with this ID card number is already registered!!!",
        )

    serializer = UserRegistrationSerializer(user, data=request.data, partial=True)
    serializer.is_valid(raise_exception=True)
    serializer.save()
    return SuccessResponse(
        "User registered successfully!", status_code=status.HTTP_201_CREATED
    )


@api_view(["POST"])
@permission_classes([AllowAny])  # Allow any user to access this endpoint
@ratelimit(key="ip", rate="100/h", method="POST", block=True)
def login(request):
    req = json.loads(request.body)
    email = req["email"]
    password = req["password"]

    try:
        user = authenticate(email=email, password=password)

        if user is None or not user.voting_rights:
            raise Exception("Invalid credentials")

        refresh = RefreshToken.for_user(user)
        access = str(refresh.access_token)

        res_body = {
            "access_token": access,
            "refresh_token": str(refresh),
        }
        res = SuccessResponse(res_body, status_code=status.HTTP_200_OK)
        return res
    except Exception as e:
        return ErrorResponse(status.HTTP_401_UNAUTHORIZED, "Unauthorized", str(e))


"""
@api_view(["POST"])
@permission_classes([IsAuthenticated])
def logout(request):
    refresh_token = request.data.get("refresh_token")

    if not refresh_token:
        return ErrorResponse(
            status.HTTP_401_UNAUTHORIZED, "Unauthorized", "Refresh token is required"
        )

    try:
        refresh_token = RefreshToken(refresh_token)
        refresh_token.blacklist()
        return SuccessResponse("Successfully logged out.")
    except Exception as e:
        return ErrorResponse(status.HTTP_400_BAD_REQUEST, "Bad Request", str(e))
"""


@api_view(["POST"])
@permission_classes([IsAuthenticated])
def logout(request):
    refresh_token = request.data.get("refresh_token")

    if not refresh_token:
        return Response(
            {"error": "Refresh token is required"}, status=status.HTTP_401_UNAUTHORIZED
        )

    try:
        refresh_token = RefreshToken(refresh_token)
        refresh_token.check_blacklisted()

        # Additionally, you might want to set a short expiration time for access tokens
        access_token = AccessToken.for_user(request.user)
        access_token.set_exp(1)  # Set a short expiration time, e.g., 1 second

        return Response(
            {"message": "Successfully logged out."}, status=status.HTTP_200_OK
        )
    except Exception as e:
        return Response({"error": str(e)}, status=status.HTTP_400_BAD_REQUEST)


# api for verifying token
@api_view(["GET"])
@permission_classes([IsAuthenticated])
def verify_token(request):
    return SuccessResponse("Token is valid")
