from django.contrib.auth.base_user import BaseUserManager
from django.contrib.auth.hashers import make_password


class UserManager(BaseUserManager):
    def create_user(self, id_card_number, **extra_fields):
        if not id_card_number:
            raise ValueError("The id card number must be set")

        # verify id card number with external service

        user = self.model(id_card_number=id_card_number)

        extra_fields.setdefault("voting_rights", True)
        extra_fields.setdefault("is_superuser", False)

        user.save()
        return user

    def update_user(self, email, password, **extra_fields):
        if not email:
            raise ValueError("The email must be set")

        user = self.model(email=email, **extra_fields)

        hashed_password = make_password(password)
        user.password = hashed_password

        user.save()
        return user

    def create_superuser(self, email, password, **extra_fields):
        extra_fields.setdefault("is_superuser", True)

        if extra_fields.get("is_superuser") is not True:
            raise ValueError("Superuser must have is_superuser=True.")

        return self.create_user(email, password, **extra_fields)
