from django.db import models
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.contrib.postgres.fields import ArrayField
from django.utils import timezone

from volby_backend.managers import UserManager
from django.utils import timezone

# models docs: https://docs.djangoproject.com/en/4.2/topics/db/models/


class Document(models.Model):
    email = models.TextField(default='"email', unique=True)
    password = models.TextField(default='"password')
    # salt = models.TextField()


class User(AbstractBaseUser, PermissionsMixin):
    id = models.AutoField(primary_key=True, editable=False)
    email = models.EmailField(unique=True, null=True)
    id_card_number = models.CharField(default="", max_length=10, unique=True)
    birth_number = models.CharField(default="", max_length=10, unique=True)
    public_key = models.CharField(null=True)

    voting_rights = models.BooleanField(default=True)
    is_superuser = models.BooleanField(default=False)

    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = []

    objects = UserManager()

    def __str__(self):
        return self.email


class PoliticalParty(models.Model):
    id = models.AutoField(primary_key=True, editable=False)
    name = models.CharField(max_length=255, unique=True)
    description = models.TextField()
    attributes = ArrayField(models.CharField(max_length=50), blank=True, null=True)

    def __str__(self):
        return self.name


class Vote(models.Model):
    id = models.AutoField(primary_key=True, editable=False)
    party_name = models.TextField(editable=False, null=True)
    timestamp = models.DateTimeField(default=timezone.now, editable=False)

    def __str__(self):
        return str(self.id)


class EncryptedVote(models.Model):
    id = models.AutoField(primary_key=True, editable=False)
    data = models.BinaryField(editable=False)
    key = models.BinaryField(editable=False)
    timestamp = models.DateTimeField(default=timezone.now, editable=False)

    def __str__(self):
        return str(self.id)

class EncryptedVoteServer(models.Model):
    id = models.AutoField(primary_key=True, editable=False)
    data = models.BinaryField(editable=False)
    key = models.BinaryField(editable=False)
    timestamp = models.DateTimeField(default=timezone.now, editable=False)

    def __str__(self):
        return str(self.id)