import re

from django.contrib.auth.models import User
from rest_framework import serializers
from django.http import HttpResponse
from django.contrib.auth import get_user_model
from django.contrib.auth.password_validation import validate_password
from .models import PoliticalParty


User = get_user_model()

class PoliticalPartySerializer(serializers.ModelSerializer):
    class Meta:
        model = PoliticalParty
        fields = ['id', 'name', 'description', 'attributes']

class UserRegistrationSerializer(serializers.ModelSerializer):
    password2 = serializers.CharField(
        write_only=True, required=True, style={"input_type": "password"}
    )

    class Meta:
        model = User
        fields = ("email", "password", "password2")

    def validate(self, attrs):
        if attrs["password"] != attrs["password2"]:
            raise serializers.ValidationError(
                {"password": "Password fields didn't match."}
            )

        try:
            validate_password(attrs["password"])
        except Exception as e:
            raise serializers.ValidationError(str(e), code="password_invalid")

        if not re.match(
            r"^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!#%*?&])[A-Za-z\d@$!#%*?&]{8}",
            attrs["password"],
        ):
            raise serializers.ValidationError(
                {
                    "password": "Password must contain at least 8 characters, 1 uppercase letter, 1 lowercase letter, 1 number and 1 special character."
                }
            )

        return attrs

    def update(self, instance, validated_data):
        instance.email = validated_data["email"]
        instance.set_password(validated_data["password"])
        instance.save()
        return instance
