import base64
from Crypto.Cipher import AES, PKCS1_OAEP
from Crypto.Random import get_random_bytes
from .key_management import load_public_key, load_private_key
from Crypto.PublicKey import RSA
from Crypto.Hash import SHA256


# Symmetric encryption
def symmetric_encrypt(data):
    key = get_random_bytes(16)  # AES-128
    cipher = AES.new(key, AES.MODE_GCM)
    nonce = cipher.nonce
    encrypted_key = asymmetric_encrypt_key(key)
    ciphertext, tag = cipher.encrypt_and_digest(data.encode())
    encrypted_data = nonce + ciphertext + tag
    return encrypted_data, encrypted_key


class InvalidPublicKeyFormatError(Exception):
    pass


def symmetric_encrypt_user(data, user_public_key_der):
    key = get_random_bytes(16)  # AES-128
    cipher = AES.new(key, AES.MODE_GCM)
    nonce = cipher.nonce
    try:
        pem_public_key = user_public_key_der
        pem_public_key = pem_public_key.replace(
            "b'-----BEGIN PUBLIC KEY-----", ""
        ).replace("-----END PUBLIC KEY-----'", "")
        pem_public_key = pem_public_key.replace(
            "-----BEGIN PUBLIC KEY-----", ""
        ).replace("-----END PUBLIC KEY-----", "")
        user_public_key_der = pem_public_key.replace(r"\n", "")
        der_public_key = base64.b64decode(user_public_key_der)
        user_public_key = RSA.import_key(der_public_key)
        cipher_rsa = PKCS1_OAEP.new(user_public_key, hashAlgo=SHA256)
        enc_key = cipher_rsa.encrypt(key)
        ciphertext, tag = cipher.encrypt_and_digest(data.encode())
        encrypted_data = nonce + ciphertext + tag

        return encrypted_data, enc_key
    except Exception:
        raise InvalidPublicKeyFormatError("Invalid public key format.")


# Asymmetric encryption
def asymmetric_encrypt_key(key):
    public_key = load_public_key()

    cipher_rsa = PKCS1_OAEP.new(public_key, hashAlgo=SHA256)
    enc_key = cipher_rsa.encrypt(key)
    return enc_key


# Asymmetric decryption
def decrypt_data(encrypted_data, encrypted_key):
    private_key = load_private_key()

    cipher_rsa = PKCS1_OAEP.new(private_key, hashAlgo=SHA256)
    decrypted_key = cipher_rsa.decrypt(encrypted_key)

    nonce = encrypted_data[:16]
    ciphertext = encrypted_data[16:]
    cipher = AES.new(decrypted_key, AES.MODE_GCM, nonce=nonce)
    data = cipher.decrypt(ciphertext).decode()

    return data
