import base64
import json
from django.middleware.csrf import get_token

from rest_framework import status
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import AllowAny

from volby_backend.services.checksum_service import generate_checksum
from volby_backend.services.encryption_service import decrypt_data, symmetric_encrypt
from volby_backend.models import User
from volby_backend.utils.responses import SuccessResponse


@api_view(["GET"])
@permission_classes([AllowAny])
def encrypt_data_view(request):
    # Retrieve all records from the Document table
    users = User.objects.all()

    # Create a list to store the data for each document
    document_data = []

    for document in users:
        document_data.append(
            {
                "id": document.id,
                "id_card_number": document.id_card_number,
                "birth_number": document.birth_number,
                "email": document.email,
                "password": document.password,
                "voting_rights": document.voting_rights,
                "public_key": document.public_key,
            }
        )

    # Create a dictionary that includes the list of document data
    data_to_encrypt = {"users": document_data}

    # Convert the data to a JSON string
    json_data = json.dumps(data_to_encrypt)

    # Encrypt the JSON data
    encrypted_data, encrypted_key = symmetric_encrypt(json_data)

    res_body = {
        "encrypted_data": base64.b64encode(encrypted_data).decode(),
        "encrypted_key": base64.b64encode(encrypted_key).decode(),
    }

    checksum = generate_checksum(json.dumps(res_body))

    res = SuccessResponse(res_body, status_code=status.HTTP_200_OK)
    res["Checksum"] = checksum
    res["csrftoken"] = get_token(request)

    return res


@api_view(["POST"])
@permission_classes([AllowAny])
def decrypt_data_view(request):
    data = base64.b64decode(request.data["data"].encode())
    key = base64.b64decode(request.data["key"].encode())

    decrypted_json_data = decrypt_data(data, key)

    # Parse the decrypted JSON data
    decrypted_data = json.loads(decrypted_json_data)

    # Now you can access individual records from the "Document" table
    decrypted_documents = decrypted_data.get("users", [])

    checksum = generate_checksum(json.dumps(decrypted_documents))

    res = SuccessResponse(decrypted_documents, status_code=status.HTTP_200_OK)
    res["Checksum"] = checksum

    return res
