import { useState } from "react";
import { authenticatedFetch } from "../utils/authenticatedFetch";

const UpdateUserEmailForm = () => {
  const [email, setEmail] = useState('');
  const [success, setSuccess] = useState(false);
  const [error, setError] = useState('');


  const handleUpdate = async (e: React.FormEvent<HTMLFormElement>) => {
    e.preventDefault();
    setSuccess(false);
    setError('');

    try {
      const response = await authenticatedFetch('/user/update-user-email', {
        method: 'PUT',
        body: JSON.stringify({ "new_email": email }),
      });

      if (!response.ok) {
        throw new Error('Nepodarilo sa upraviť email.');
      }

      setError('');
      setSuccess(true);
      setEmail('');
    } catch (error) {
      setError("Nepodarilo sa upraviť email.")
    }

  };

  return (
    <form onSubmit={handleUpdate} className="flex flex-col items-center justify-center gap-4">
      <div className="flex flex-row gap-4">
        <input
          type="email"
          className="border rounded-full px-5 py-2"
          id="email"
          value={email}
          placeholder="Nový email"
          onChange={(e) => setEmail(e.target.value)}
        />
        <button type="submit" className="group relative flex justify-center py-2 px-4 border border-transparent text-sm font-medium rounded-full text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500 transition-all duration-200">
          Upraviť
        </button>
      </div>
      {error && <p className="text-red-500">{error}</p>}
      {success && <p className="text-green-500">Email bol úspešne upravený.</p>}
    </form>
  );
}

export default UpdateUserEmailForm;