import React, { useContext, useState } from 'react';
import { AuthContext } from '../context/AuthContext';
import { importPrivateKey } from '../utils/cryptoHelpers';

const UploadPrivateKeyForm = () => {
  const { user } = useContext(AuthContext);
  const [success, setSuccess] = useState<boolean>(false);
  const [error, setError] = useState<string>('');
  const [privateKeyFile, setPrivateKeyFile] = useState<File | null>(null);
  // const [passphrase, setPassphrase] = useState<string>('');
  const [inputKey, setInputKey] = useState(Math.random().toString());

  const handleChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    const { name, files } = e.target;

    // if (name === 'passphrase') {
    //   setPassphrase(value);
    // } else if (name === 'private_key') {
    //   setPrivateKeyFile(files ? files[0] : null);
    // }
    if (name === 'private_key') {
      setPrivateKeyFile(files ? files[0] : null);
    }
  }

  const handleSubmit = async (e: React.FormEvent<HTMLFormElement>) => {
    e.preventDefault();
    setError('');
    setSuccess(false);

    if (!privateKeyFile) {
      setError('Prosím vyberte súbor s privátnym kľúčom');
      return;
    }

    const privateKey = await privateKeyFile.text();

    // encrypt private key with user's passphrase
    try {
      const importedPrivateKey = await importPrivateKey(privateKey, true);
      if (!importedPrivateKey) {
        setError('Nesprávne heslo');
        return;
      }
  
      // const encryptedPrivateKey = await encryptPrivateKey(importedPrivateKey, passphrase);
  
      // save encrypted private key to session storage
      // sessionStorage.setItem('privateKey', JSON.stringify(encryptedPrivateKey));
  
      // save private key to user from auth context
      if (user) {
        user.private_key = importedPrivateKey;
      }
  
      // setPassphrase('');
      setPrivateKeyFile(null);
      setSuccess(true);
      setInputKey(Math.random().toString()); // Reset the input
    } catch (err) {
      setError('Nesprávny formát');
    }
  };

  return (
    <div className='p-4 space-y-5 flex flex-col items-center justify-center border border-slate-200 rounded-md'>
      <form className="flex flex-row items-center justify-center space-x-5" onSubmit={handleSubmit}>
        {/* <div>
          <label htmlFor="passphrase" className="sr-only"></label>
          <input value={passphrase} onChange={handleChange} required autoComplete='off' id="passphrase" name="passphrase" type="password" className="appearance-none rounded-md relative block w-full px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" placeholder="Heslo" />
        </div> */}
        <div>
          <label htmlFor="private_key" className="sr-only">Private Key</label>
          <input key={inputKey} onChange={handleChange} required id="private_key" name="private_key" type="file" accept=".pem" className="appearance-none rounded-md relative block w-full px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" />
        </div>
        <div>
          <button type="submit" className="group relative w-full flex justify-center py-2 px-4 border border-transparent text-sm font-medium rounded-md text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500">
            Nahrať privátny kľúč
          </button>
        </div>
      </form>
      {success && <p className="text-green-500">Privátny kľúč bol úspešne nahratý</p>}
      {error && <p className="text-red-500">{error}</p>}
    </div>
  );
};

export default UploadPrivateKeyForm;