const API_URL = import.meta.env.REACT_APP_API_URL;

export function getCsrfTokenFromCookies(): string | undefined {
    return document.cookie
        .split("; ")
        .find((row) => row.startsWith("csrftoken"))
        ?.split("=")[1];
}

export async function getCsrfToken(): Promise<string | undefined> {
    let csrfToken = getCsrfTokenFromCookies();

    if (csrfToken === undefined) {
        const response = await fetch(`${API_URL}/csrf/`, {
            headers: {
                "Content-Type": "application/json",
                Referer: "http://localhost:3000/",
            },
        });
        const data = await response.json();
        csrfToken = data.csrfToken;
    }

    return csrfToken;
}
