import { FC, useContext, useEffect, } from "react";
import { AuthContext } from "../context/AuthContext";
import { useNavigate } from "react-router-dom";
import UploadPublicKeyForm from "../components/UploadPublicKeyForm";
import UploadPrivateKeyForm from "../components/UploadPrivateKeyForm";
import GenerateKeysButton from "../components/GenerateKeysButton";
import TestDecryptionButton from "../components/TestDecryptionButton";
import UpdateUserEmailForm from "../components/UpdateUserEmailForm";

const AccountPage: FC = () => {
  const { user } = useContext(AuthContext);
  const navigate = useNavigate();

  useEffect(() => {
    if (!user) {
      navigate('/auth/login');
    }
  }, [user, navigate]);

  return (
    <section className="flex flex-col items-center justify-start gap-4 mt-24 pb-20">
      <h2 className=" font-bold text-2xl text-center">Osobné údaje</h2>
      <p>Tu je možné zmeniť emailovú adresu.</p>
      <UpdateUserEmailForm />

      <h2 className=" font-bold text-2xl text-center mt-20">Šifrovanie</h2>
      <p>V tejto sekcii je možné vložiť alebo vygenerovať šifrovacie kľúče.</p>
      <UploadPublicKeyForm />
      <UploadPrivateKeyForm />
      <GenerateKeysButton />
      <TestDecryptionButton />
    </section>
  );
};

export default AccountPage;