import { FC, useContext } from "react"
import { AuthContext } from "../context/AuthContext"
import { BrowserRouter, Navigate, Route, Routes } from "react-router-dom"
import Register from "../components/Register"
import Login from "../components/Login"
import { PrivateRoutes } from "./PrivateRoutes"
import Layout from "../layout/Layout"
import VotedPage from "../pages/VotedPage"
import DecryptPage from "../pages/DecryptPage"

const { PUBLIC_URL } = import.meta.env;

const AppRoutes: FC = () => {
  const { user } = useContext(AuthContext);

  const isAuthenticated = user && user.token_type === 'access';

  return (
    <BrowserRouter basename={PUBLIC_URL}>
      <Layout>
        <Routes>
          {isAuthenticated ? (
            <>
              <Route path='/*' element={<PrivateRoutes />} />
              <Route path="/auth/*" element={<Navigate to="/" />} />
            </>
          ) : (
            <>
              <Route path="/auth/register" element={<Register />} />
              <Route path="/auth/login" element={<Login />} />
              <Route path="/voted" element={<VotedPage />} />
              <Route path="/decrypt" element={<DecryptPage />} />
              <Route path="/*" element={<Navigate to="/auth/login" />} />
            </>
          )}
        </Routes>
      </Layout>
    </BrowserRouter>
  )
}

export { AppRoutes }