import { getCsrfToken, getCsrfTokenFromCookies } from "../lib/csrfTokenHandler";

type FetchOptions = {
  method?: string;
  headers?: any;
  body?: any;
  signal?: AbortSignal;
};

export const authenticatedFetch = async (
  path: string,
  options: FetchOptions
) => {
  const token = localStorage.getItem("jwtToken");

  // check options and remove duplicate headers authorization, content-type, referer
  if (options.headers) {
    delete options.headers.Authorization;
    delete options.headers["Content-Type"];
    delete options.headers.Referer;
  }

  let headers = {
    Authorization: `Bearer ${token}`,
    "Content-Type": "application/json",
    Referer: import.meta.env.BASE_URL || "default_referer",
    ...options.headers,
  };

  if (options.method && options.method.toLowerCase() === "post") {
    let csrfToken = getCsrfTokenFromCookies();

    if (csrfToken === undefined) {
      csrfToken = await getCsrfToken();
    }

    headers = {
      ...headers,
      "X-CSRFToken": csrfToken || "",
    };
  }

  if (path.startsWith("/")) {
    path = path.substring(1);
  }

  const response = await fetch(`${import.meta.env.VITE_API_URL}/${path}`, {
    headers,
    ...options,
  });

  return response;
};
