import { getCsrfToken, getCsrfTokenFromCookies } from "../lib/csrfTokenHandler";

async function customFetch(path: string, options: RequestInit = {}) {
  if (options.method?.toUpperCase() === "POST") {
    let csrfToken = getCsrfTokenFromCookies();

    if (csrfToken === undefined) {
      csrfToken = await getCsrfToken();
    }

    options.headers = {
      ...options.headers,
      "Content-Type": "application/json",
      "X-CSRFToken": csrfToken || "",
      Referer: import.meta.env.BASE_URL || "default_referer",
    };
  }

  if (path.startsWith("/")) {
    path = path.substring(1);
  }

  const response = await fetch(`${import.meta.env.VITE_API_URL}/${path}`, options);

  if (response.status === 403) {
    const csrfToken = await getCsrfToken();
    options.headers = {
      ...options.headers,
      "X-CSRFToken": csrfToken || "",
    };
    return fetch(`${import.meta.env.VITE_API_URL}/${path}`, options);
  }

  return response;
}

export default customFetch;
