import calendar
import random
import time
from datetime import datetime, timedelta
import psycopg2
import psycopg2.extras
from entities.record_type import RecordType
from entities.user import User, get_user_by_username


def create_records_table(conn):
    cursor = conn.cursor()
    cursor.execute("CREATE TABLE IF NOT EXISTS records ("
                   "id SERIAL PRIMARY KEY,"
                   "user_id INTEGER REFERENCES users(id),"
                   "type VARCHAR(50) NOT NULL,"
                   "name VARCHAR(50) NOT NULL,"
                   "value VARCHAR(50) NOT NULL,"
                   "datetime TIMESTAMP NOT NULL,"
                   "author VARCHAR(50))")
    conn.commit()
    cursor.close()


def generate_records(conn, users, months, years):
    user_ids = []
    for user in users:
        try:
            user = get_user_by_username(conn, user)
            if user:
                user_ids.append(user.id)
        except:
            pass

    for user_id in user_ids:
        time.sleep(2)
        if get_records_by_user_id(conn, user_id):
            continue
        for year in years:
            for month in months:
                # Get the number of days in the specified month and year
                days_in_month = calendar.monthrange(year, month)[1]

                for day in range(1, days_in_month + 1):
                    # Generate random arrival and departure times
                    arrival_time = datetime(year, month, day, random.randint(0, 10), random.randint(0, 59))
                    departure_time = arrival_time + timedelta(hours=random.uniform(7, 9))

                    # Randomly select author and type
                    author = str(random.randint(1, 3))

                    # Set name and value based on author
                    record_name = 'Príchod'
                    record_value = ''
                    if author == '1':
                        record_value = ''
                    elif author == '2':
                        record_value = 'Zadané používateľom'
                    elif author == '3':
                        record_value = 'Zadané vo virtuálnom termináli'

                    # Create Record object and save to the database
                    record = Record(user_id, RecordType.PRICHOD.value, arrival_time,
                                    author, record_name, record_value)
                    record.save_record(conn)
                    author = str(random.randint(1, 3))
                    record_value = ''
                    if author == '1':
                        record_value = ''
                    elif author == '2':
                        record_value = 'Zadané používateľom'
                    elif author == '3':
                        record_value = 'Zadané vo virtuálnom termináli'
                    # Set name and value based on author
                    record_name = 'Odchod'
                    # Create Record object and save to the database
                    record = Record(user_id, RecordType.ODCHOD.value, departure_time,
                                    author, record_name, record_value)
                    record.save_record(conn)


def get_records_by_user_id(conn, user_id):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM records WHERE user_id = %s", (user_id,))
    records = cursor.fetchall()
    cursor.close()

    return records


# get records by user_id and date (day) which is not datetime but date (16.12.2020)
def get_records_by_user_id_and_day(conn, user_id, day):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM records WHERE user_id = %s AND datetime::date = %s", (user_id, day))
    records = cursor.fetchall()
    cursor.close()

    return records


def get_records_by_user_id_and_month(conn, user_id, month):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM records WHERE user_id = %s AND EXTRACT(MONTH FROM datetime) = %s ORDER BY datetime asc", (user_id, month))
    records = cursor.fetchall()
    cursor.close()

    return records


def get_records_by_user_id_and_types(conn, user_id, types):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM records WHERE user_id = %s AND type IN %s ORDER BY datetime asc", (user_id, tuple(map(str, types))))
    records = cursor.fetchall()
    cursor.close()

    return records


def get_records_by_user_id_month_type(conn, user_id, month, record_type):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM records WHERE user_id = %s AND EXTRACT(MONTH FROM datetime) = %s AND type = %s",
                   (user_id, month, record_type))
    records = cursor.fetchall()
    cursor.close()

    return records


# function to get record with record_type, which has the nearest higher record_time to the given record_time
def get_nearest_higher_record(conn, user_id, record_type, record_datetime):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute(
        "SELECT * FROM records WHERE user_id = %s AND type = %s AND datetime > %s ORDER BY datetime ASC LIMIT 1",
        (user_id, record_type, record_datetime))
    record = cursor.fetchone()
    cursor.close()

    return record


# function to get record with record_type, which has the nearest lower record_time to the given record_time
def get_nearest_lower_record(conn, user_id, record_type, record_datetime):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute(
        "SELECT * FROM records WHERE user_id = %s AND type = %s AND datetime < %s ORDER BY datetime DESC LIMIT 1",
        (user_id, record_type, record_datetime))
    record = cursor.fetchone()
    cursor.close()

    return record


def get_record_by_user_datetime(conn, user_id, record_datetime):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM records WHERE user_id = %s AND datetime = %s", (user_id, record_datetime))
    record = cursor.fetchone()
    cursor.close()

    return record


def get_record_by_user_datetime_type(conn, user_id, record_datetime, record_type):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM records WHERE user_id = %s AND datetime = %s AND type = %s",
                   (user_id, record_datetime, record_type))
    record = cursor.fetchone()
    cursor.close()

    return record


# function to get all distinct month year combinations for a given user_id
# month and year must be strings
def get_distinct_month_year_combinations_attend(conn, user_id):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT DISTINCT EXTRACT(MONTH FROM datetime) AS month, EXTRACT(YEAR FROM datetime) AS year "
                   "FROM records WHERE user_id = %s AND type IN('1','2') ORDER BY year DESC, month DESC", (user_id,))
    records = cursor.fetchall()
    cursor.close()

    return records


def get_worktime(conn, user_id, month, year):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM records WHERE user_id = %s AND EXTRACT(MONTH FROM datetime) = %s "
                   "AND EXTRACT(YEAR FROM datetime) = %s AND type IN ('1','2') ORDER BY datetime ASC",
                   (user_id, month, year))
    records = cursor.fetchall()
    cursor.close()

    result = []
    current_day = None
    current_records = []
    all_records = []
    total_worktime = timedelta()

    for i, record in enumerate(records):
        record_type = record['type']
        record_date = record['datetime'].date()

        if current_day is None:
            current_day = record_date

        if record_date != current_day:
            result.append({'date': current_day, 'records': current_records, 'worktime': total_worktime})
            current_records = []
            all_records = []
            total_worktime = timedelta()
            current_day = record_date

        if record_type == '1':
            current_records.append(record)
            all_records.append(record)
        elif record_type == '2':
            if current_records:
                all_records.append(record)
                last_arrive = current_records[-1]['datetime']
                if last_arrive > record['datetime']:
                    # Overnight shift, calculate worktime for the previous day
                    work_hours = last_arrive - current_records[0]['datetime']
                else:
                    # Same day shift
                    work_hours = record['datetime'] - last_arrive

                total_worktime += work_hours

    # Handle the last day
    if current_records:
        result.append({'date': current_day, 'records': all_records, 'worktime': total_worktime})

    return result


def add_missing_days(result, month, year):
    # Create a set of existing dates
    existing_dates = {entry['date'] for entry in result}

    # Get the first and last day of the month
    first_day = datetime(year, month, 1).date()
    last_day = datetime(year, month + 1, 1).date() - timedelta(days=1)

    # Add missing days with empty records and 0 worktime
    current_day = first_day
    while current_day <= last_day:
        if current_day not in existing_dates:
            result.append({'date': current_day, 'records': [], 'worktime': timedelta()})
        current_day += timedelta(days=1)

    # Sort the result array by date
    result.sort(key=lambda x: x['date'])

    return result


def get_records_by_user_id_month_year_types(conn, user_id, month, year, types):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM records WHERE user_id = %s AND EXTRACT(MONTH FROM datetime) = %s "
                   "AND EXTRACT(YEAR FROM datetime) = %s AND type IN %s ORDER BY datetime ASC",
                   (user_id, month, year, tuple(map(str, types))))
    records = cursor.fetchall()
    cursor.close()

    return records


def get_records_by_user_id_day_month_year_types(conn, user_id, day, month, year, types):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM records WHERE user_id = %s AND EXTRACT(DAY FROM datetime) = %s "
                   "AND EXTRACT(MONTH FROM datetime) = %s AND EXTRACT(YEAR FROM datetime) = %s "
                   "AND type IN %s ORDER BY datetime ASC",
                   (user_id, day, month, year, tuple(map(str, types))))
    records = cursor.fetchall()
    cursor.close()

    return records


def get_worktime_and_records_from_day_attend(conn, user_id, day, month, year):
    records = get_records_by_user_id_day_month_year_types(conn, user_id, day, month, year, [1, 2])

    out_records = []
    total_worktime = timedelta()
    last_arrive = None

    for record in records:
        record_type = record['type']

        if last_arrive is None and record_type == '1':
            last_arrive = record
        elif last_arrive is not None and record_type == '2':
            work_hours = record['datetime'] - last_arrive['datetime']
            total_worktime += work_hours
            out_records.append(last_arrive)
            out_records.append(record)
            last_arrive = None
    # convert total_worktime to seconds and cast to int
    total_worktime = int(total_worktime.total_seconds())
    return total_worktime, out_records


def get_worktime_for_month_year(conn, user_id, month, year):
    days_in_month = calendar.monthrange(year, month)[1]
    result = []
    for day in range(1, days_in_month + 1):
        worktime, records = get_worktime_and_records_from_day_attend(conn, user_id, day, month, year)
        result.append({'date': datetime(year, month, day).date(), 'worktime': worktime, 'records': records})
    return result



def delete_records_by_ids(conn, record_ids):
    cursor = conn.cursor()
    cursor.execute("DELETE FROM records WHERE id IN %s", (tuple(map(int, record_ids)),))
    cursor.close()
    conn.commit()


class Record:
    def __init__(self, user_id, record_type, record_datetime, record_author, record_name=None, record_value=None,
                 record_id=None):
        self.record_id = record_id
        self.user_id = user_id
        self.record_type = record_type
        self.record_name = record_name
        self.record_value = record_value
        self.record_datetime = record_datetime
        self.record_author = record_author

    def save_record(self, conn):
        cursor = conn.cursor()
        cursor.execute(
            "INSERT INTO records (user_id, type, name, value, datetime, author) VALUES (%s, %s, %s, %s, %s, %s)",
            (self.user_id, self.record_type, self.record_name, self.record_value, self.record_datetime,
             self.record_author))
        cursor.close()
        conn.commit()

    def delete_record(self, conn):
        cursor = conn.cursor()
        cursor.execute("DELETE FROM records WHERE id = %s", (self.record_id,))
        cursor.close()
        conn.commit()

    def update_record(self, conn):
        cursor = conn.cursor()
        cursor.execute("UPDATE records SET type = %s, name = %s, value = %s, datetime = %s, author = %s WHERE id = %s",
                       (
                           self.record_type, self.record_name, self.record_value, self.record_datetime,
                           self.record_author))
        cursor.close()
        conn.commit()

    def get_record(self, conn):
        cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
        cursor.execute("SELECT * FROM records WHERE id = %s", (self.record_id,))
        record_data = cursor.fetchone()
        cursor.close()

        if record_data:
            return Record(record_data['id'], record_data['user_id'], record_data['type'], record_data['name'],
                          record_data['value'], record_data['datetime'], record_data['author'])
        else:
            return None
