function loadPublicKeyFromFile(file) {
    return new Promise((resolve, reject) => {
        const reader = new FileReader();
        reader.onload = (event) => {
            const publicKeyPem = event.target.result;
            const publicKey = forge.pki.publicKeyFromPem(publicKeyPem);
            resolve(publicKey);
        };
        reader.onerror = (event) => {
            reject(event.target.error);
        };
        reader.readAsText(file);
    });
}

function loadPrivateKeyFromFile(file) {
    return new Promise((resolve, reject) => {
        const reader = new FileReader();
        reader.onload = (event) => {
            const privateKeyPem = event.target.result;
            const privateKey = forge.pki.privateKeyFromPem(privateKeyPem);
            resolve(privateKey);
        };
        reader.onerror = (event) => {
            reject(event.target.error);
        };
        reader.readAsText(file);
    });
}

function decrypt(privateKey, message) {
    try {
        const decodedMessage = forge.util.decode64(message); // Ensure the message is properly Base64-decoded
        return privateKey.decrypt(decodedMessage, "RSA-OAEP", {
            md: forge.md.sha256.create(),
            mgf1: {
                md: forge.md.sha256.create()
            }
        });
    } catch (error) {
        console.error("Decryption Error:", error.message);
        return null; // Return null or handle the error as needed
    }
}

function decryptAesCbc(encryptedBytes, keyBytes, ivBytes) {
    const decipher = forge.cipher.createDecipher('AES-CBC', keyBytes);
    decipher.start({ iv: ivBytes });
    decipher.update(forge.util.createBuffer(encryptedBytes));
    decipher.finish();

    // Get the decrypted bytes
    let decryptedBytes = decipher.output.getBytes();

    // Remove PKCS#7 padding
    const paddingLength = decryptedBytes.charCodeAt(decryptedBytes.length - 1);
    decryptedBytes = decryptedBytes.slice(0, -paddingLength);

    // Convert the raw binary string to UTF-8
    return forge.util.decodeUtf8(decryptedBytes);
}

function generateAesKey() {
    return forge.random.getBytesSync(32);
}

function generateAesIv() {
    return forge.random.getBytesSync(16);
}

function encryptMessageWithSymmetricKey(symmetricKey, iv, message) {

    const aesKey = forge.util.createBuffer(symmetricKey, 'raw');
    const messageBytes = forge.util.createBuffer(message, 'utf-8');
    const cipher = forge.cipher.createCipher('AES-CBC', aesKey);
    cipher.start({ iv: iv });
    cipher.update(messageBytes);
    cipher.finish();

    const ciphertext = cipher.output.getBytes();
    return forge.util.bytesToHex(ciphertext);
}

function decryptMessageWithSymmetricKey(encryptedMessage, key, iv) {
    // Convert the IV and key from hexadecimal strings to bytes
  const ivBytes = iv
  const keyBytes = key

  // Convert the encrypted message from Base64 to bytes
  const encryptedBytes = forge.util.decode64(encryptedMessage);

  // Create a cipher object for decryption
  const decipher = forge.cipher.createDecipher('AES-CBC', keyBytes);
  decipher.start({ iv: ivBytes });

  // Update the decipher with the encrypted message
  decipher.update(forge.util.createBuffer(encryptedBytes));

  // Finish the decryption
  decipher.finish();

  // Get the decrypted bytes as a buffer
  const decryptedBuffer = decipher.output;

  // Convert the decrypted bytes to a UTF-8 string
    return decryptedBuffer.toString('utf-8');
}

