import base64
from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import padding as padding2

def encrypt_with_public_key(public_key, data):
    return public_key.encrypt(
        data,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )


def encryptAES(key, iv, data):
    data = bytes(data, 'utf-8')
    # Create a padding object with the same block size as AES
    padder = padding2.PKCS7(algorithms.AES.block_size).padder()

    # Apply padding to the data
    padded_data = padder.update(data) + padder.finalize()

    # Create a cipher object for AES encryption in CBC mode
    cipher = Cipher(algorithms.AES(key), modes.CBC(iv))

    # Create an encryptor object and encrypt the data
    encryptor = cipher.encryptor()
    encrypted_data = encryptor.update(padded_data) + encryptor.finalize()

    # Return the encrypted data
    return encrypted_data


def decryptAES(key, iv, data):
    data2 = bytes.fromhex(data)
    cipher = Cipher(algorithms.AES(key), modes.CBC(iv))
    decryptor = cipher.decryptor()
    decrypted_data = decryptor.update(data2) + decryptor.finalize()
    # Create a padding object with the same block size as AES
    unpadder = padding2.PKCS7(algorithms.AES.block_size).unpadder()

    # Remove padding and return the original plaintext
    unpadded_data = unpadder.update(decrypted_data) + unpadder.finalize()
    return unpadded_data.decode('utf-8')


class CryptographyUtils:
    def __init__(self):
        self.private_key = None
        self.public_key = None
        self.symmetric_key = None

    def generate_asymmetric_keys(self):
        # Generate a new RSA private key
        self.private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=2048,
        )

        # Get the corresponding public key
        self.public_key = self.private_key.public_key()
        self.save_private_key("./keys/private_key.pem")
        self.save_public_key("./keys/public_key.pem")

    def save_private_key(self, filename):
        # Serialize and save the private key to a file
        private_pem = self.private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        )
        with open(filename, "wb") as private_file:
            private_file.write(private_pem)
            private_file.close()

    def save_public_key(self, filename):
        # Serialize and save the public key to a file
        public_pem = self.public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo
        )
        with open(filename, "wb") as public_file:
            public_file.write(public_pem)
            public_file.close()

    def load_public_key(self, filename):
        # Load a public key from a file
        with open(filename, "rb") as public_file:
            public_pem = public_file.read()
        self.public_key = serialization.load_pem_public_key(public_pem)
        public_file.close()

    def load_private_key(self, filename):
        # Load a private key from a file
        with open(filename, "rb") as private_file:
            private_pem = private_file.read()
        self.private_key = serialization.load_pem_private_key(private_pem, password=None)
        private_file.close()

    def load_keys(self, public_key_filename, private_key_filename):
        self.load_public_key(public_key_filename)
        self.load_private_key(private_key_filename)

    def decryptMessage(self, encryptedMessage):
        message = self.private_key.decrypt(
            base64.b64decode(encryptedMessage),
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None
            )
        )
        message = str(message, 'utf-8')
        return message

    def decryptMessage2(self, encryptedMessage):
        message = self.private_key.decrypt(
            base64.b64decode(encryptedMessage),
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None
            )
        )

        return message
