from flask import Flask
from flask_bcrypt import Bcrypt
import re


app = Flask(__name__)


def hash_password(password):
    bcrypt = Bcrypt(app)
    return bcrypt.generate_password_hash(password).decode('utf-8')


def check_password(password, hashed_password):
    bcrypt = Bcrypt(app)
    return bcrypt.check_password_hash(hashed_password, password)


def is_password_valid(password):
    # Check if the password has at least 8 characters
    if len(password) < 8 or len(password) > 50:
        return False

    # Check if the password has at least one capital letter
    if not any(char.isupper() for char in password):
        return False

    # Check if the password has at least one regular letter (lowercase)
    if not any(char.islower() for char in password):
        return False

    # Check if the password has at least one digit
    if not any(char.isdigit() for char in password):
        return False

    # Check if the password has at least one special character
    if not re.search(r'[.!@#$%^&*()_+{}":;\']', password):
        return False

    # Check if the password is safe from SQL injection
    if re.search(r'[\'";]', password):
        return False

    return True


class PasswordUtils:
    def __init__(self):
        pass
