<?php

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);
error_reporting(E_WARNING);
require_once('config.php');

session_start();

try {
    $db = new PDO("mysql:host=$hostname;dbname=$dbname", $username, $password);
    $db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
}catch (PDOException $e){
    echo $e->getMessage();
}

if (!isset($_SESSION["email"]) || $_SESSION["loggedin"] !== true) {
    header("Location: login.php");
    exit;
}

$sql = "SELECT * FROM pouzivatel WHERE email = :email";
$stmt = $db->prepare($sql);
$stmt->bindParam(":email", $_SESSION["email"], PDO::PARAM_STR);
$stmt->execute();
$row = $stmt->fetch(PDO::FETCH_ASSOC);

if ($_SERVER["REQUEST_METHOD"] == "POST") {
    $meno = $_POST['firstname'] . ' ' . $_POST['lastname'];
    $query = "UPDATE pouzivatel SET meno = ?, cislo_op = ?, email=? WHERE email = ?";
    $stmt = $db->prepare($query);
    $success = $stmt->execute([$meno, $_POST['cislo_op'],$_POST['email'],$_SESSION["email"] ]);
    $result = $stmt->fetch(PDO::FETCH_ASSOC);

    if($row['meno'] != $meno){
        $sql = "SELECT u.iban FROM pouzivatel pouz JOIN ucet u ON pouz.id=u.pouzivatel_id WHERE pouz.email=:email";
        $stmt = $db->prepare($sql);
        $stmt->bindParam(":email", $_POST['email'], PDO::PARAM_STR);
        $stmt->execute();
        $row = $stmt->fetch(PDO::FETCH_ASSOC);

        $query = "UPDATE platba SET nazov_prijemcu = ? WHERE na_ucet = ?";
        $stmt = $db->prepare($query);
        $success = $stmt->execute([$meno, $row['iban']]);
        $result = $stmt->fetch(PDO::FETCH_ASSOC);
    }

    if($_POST['email'] != $_SESSION['email']){
        header("Location: logout.php");
        exit();
    }else{
        header("Location: index.php");
        exit();
    }
}



?>
<!doctype html>
<html lang="sk">
<head>
    <meta charset="UTF-8">
    <meta name="viewport"
          content="width=device-width, user-scalable=no, initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0">
    <meta http-equiv="X-UA-Compatible" content="ie=edge">
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-GLhlTQ8iRABdZLl6O3oVMWSktQOp6b7In1Zl3/Jr59b6EGGoI1aFkw7cmDA6j6gD" crossorigin="anonymous">
    <link rel="stylesheet" href="https://cdn.datatables.net/1.13.2/css/dataTables.bootstrap5.min.css">
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.6.3/jquery.min.js" integrity="sha512-STof4xm1wgkfm7heWqFJVn58Hm3EtS31XFaagaa8VMReCXAkQnJZ+jEy8PCC/iT18dFy95WcExNHFTqLyp72eQ==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/js/bootstrap.bundle.min.js" integrity="sha384-w76AqPfDkMBDXo30jS1Sgez6pr3x5MlQ1ZAGC+nuZB+EYdgRZgiwxhTBTkF7CXvN" crossorigin="anonymous"></script>

    <title>Zmena údajov</title>
</head>
<body class=" d-flex flex-column min-vh-100">
<header>
    <nav class=" navbar navbar-expand-lg bg-body-tertiary" data-bs-theme="dark">
        <div class="container-fluid">
            <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarNav" aria-controls="navbarNav" aria-expanded="false" aria-label="Toggle navigation">
                <span class="navbar-toggler-icon"></span>
            </button>
            <div class="collapse navbar-collapse justify-content-between" id="navbarNav">
                <ul class="navbar-nav">
                    <li class="nav-item">
                        <a class="nav-link active" aria-current="page" href="index.php">Domov</a>
                    </li>
                </ul>
                <ul class="navbar-nav ">
                    <li class="nav-item ">
                        <a class="nav-link" href="logout.php">Odhlásenie</a>
                    </li>
                </ul>
            </div>
        </div>
    </nav>
</header>
<div class="d-flex pt-2 justify-content-center"><h1>Zmena osobných údajov</h1></div>
<div class="container border rounded bg-secondary">
    <div class="width-50 p-5">
        <div class="border p-4 bg-light">
            <h2 class="justify-content-center center-text">Zadajte Vaše údaje</h2>
            <hr class="mb-4 mt-3">
            <form action="<?php echo htmlspecialchars($_SERVER['PHP_SELF']);?>" method="post">
                <div class="mb-3 row">
                    <input type="hidden" name="id" value=" ">
                    <div class="col form-group">
                        <label for="firstname" class="form-label">Meno*</label><input value="<?php echo explode(" ", $row['meno'])[0] ?>" type="text" name="firstname" id="firstname" class="form-control" required>
                    </div>
                    <div class="col form-group">
                        <label for="lastname" class="form-label">Priezvisko*</label><input value="<?php echo explode(" ", $row['meno'])[1] ?>" type="text" name="lastname" id="lastname" class="form-control" required>
                    </div>
                </div>
                <div class="mb-3 row">
                    <div class="col form-group">
                        <label for="email" class="form-label">Email*</label><input type="email" name="email" id="email" class="form-control" value="<?php echo $row['email'] ?>">
                    </div>
                    <div class="col form-group">
                        <label for="cislo_op" class="form-label">Cislo OP*</label><input value="<?php echo $row['cislo_op'] ?>" type="text" name="cislo_op" id="cislo_op" class="form-control" required>
                    </div>
                </div>
                <div class="mb-3 row">
                    <div class="col form-group">
                        <button type="submit" class="btn btn-primary" name="btn-edit-person" id="btn-edit-person">Zmeniť</button>
                    </div>
                </div>
                <?php
                if (!empty($errmsg)) {
                    echo $errmsg;
                }?>
            </form>
        </div>
    </div>
</div>

<footer class="d-flex justify-content-center py-3 my-4 mt-auto border-top border-dark-subtle">
    <div><span class="text-muted">© 2023 Úvod do počítačovej bezpečnosti</span></div>
</footer>

<script src="https://cdn.datatables.net/1.13.2/js/jquery.dataTables.min.js"></script>
<script src="https://cdn.datatables.net/1.13.2/js/dataTables.bootstrap5.min.js"></script>

</body>
</html>
