<?php
require_once('config.php');
require_once 'insert_edit.php';

try {
    $db = new PDO("mysql:host=$hostname;dbname=$dbname", $username, $password);
    $db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    echo $e->getMessage();
}
session_start();

$login_attempts_max = 3; // Maximálny pocet neuspesnych pokusov
$lockout_time = 100; // časovy rozostup (10 sekúnd)


$error_msg = "";
if ($_SERVER["REQUEST_METHOD"] == "POST") {

    $sql = "SELECT meno, email, heslo FROM pouzivatel WHERE email = :email";

    $stmt = $db->prepare($sql);

    $stmt->bindParam(":email", $_POST["email"], PDO::PARAM_STR);


    if ($stmt->execute()) {
        if (isset($_SESSION['login_attempts']) && $_SESSION['login_attempts'] >= $login_attempts_max) {
            // požívateľ má príliš veľa neúspešných pokusov
            $time_since_last_attempt = time() - $_SESSION['last_login_attempt'];

            if ($time_since_last_attempt < $lockout_time) {
                // zablokovany casový úsek ešte neuplynul
                $remaining_time = $lockout_time - $time_since_last_attempt;
                $error_msg .= "<p class='text-danger'>Máte zablokovaný účet. Skúste to znovu za $remaining_time sekúnd.</p>";
                exit;
            } else {
                // resetujte počet pokusov a povoľte porihlasenie
                $_SESSION['login_attempts'] = 2;
            }
        } else {
            if ($stmt->rowCount() == 1) {
                $row = $stmt->fetch();
                $hashed_password = $row["heslo"];


                if (password_verify($_POST["heslo"], $hashed_password)) {
                    $_SESSION['login_attempts'] = 0;
                    $row = $stmt->fetch();
                    $_SESSION['loggedin'] = true;
                    $_SESSION['email'] = $_POST["email"];
                    header("Location: index.php");
                    //echo "<p class='text-success'>Ste úspešne prihlásený, pokračujte na hlavnú kartu kliknutím <a href='index_old.php'>tu</a></p>";
                    exit;
                } else {
                    $_SESSION['login_attempts']++;
                    $_SESSION['last_login_attempt'] = time();
                    $error_msg .= "<p class='text-danger'>Nesprávny login alebo heslo.</p>";
                }
            } else {
                $_SESSION['login_attempts']++;
                $_SESSION['last_login_attempt'] = time();
                $error_msg .= "<p class='text-danger'>Nesprávny login alebo heslo.</p>";
            }
        }
    } else {
        $error_msg .= "<p class='text-danger'>Niečo sa pokazilo!</p>";
    }

    unset($stmt);
    unset($db);
}


?>
<!doctype html>
<html lang="sk">
<head>
    <meta charset="UTF-8">
    <meta name="viewport"
          content="width=device-width, user-scalable=no, initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0">
    <meta http-equiv="X-UA-Compatible" content="ie=edge">
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/css/bootstrap.min.css" rel="stylesheet"
          integrity="sha384-GLhlTQ8iRABdZLl6O3oVMWSktQOp6b7In1Zl3/Jr59b6EGGoI1aFkw7cmDA6j6gD" crossorigin="anonymous">
    <link rel="stylesheet" href="https://cdn.datatables.net/1.13.2/css/dataTables.bootstrap5.min.css">
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.6.3/jquery.min.js"
            integrity="sha512-STof4xm1wgkfm7heWqFJVn58Hm3EtS31XFaagaa8VMReCXAkQnJZ+jEy8PCC/iT18dFy95WcExNHFTqLyp72eQ=="
            crossorigin="anonymous" referrerpolicy="no-referrer"></script>
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/js/bootstrap.bundle.min.js"
            integrity="sha384-w76AqPfDkMBDXo30jS1Sgez6pr3x5MlQ1ZAGC+nuZB+EYdgRZgiwxhTBTkF7CXvN"
            crossorigin="anonymous"></script>
    <!--    <link rel="stylesheet" href="css/utility.css">-->

    <title>Prihlásenie</title>
</head>
<body>

<div class="d-flex pt-2 justify-content-center"><h1>Prihlásenie</h1></div>

<header class="mb-3 mt-2">
    <nav class="navbar navbar-expand bg-body-tertiary" data-bs-theme="dark">
        <div class="collapse navbar-collapse justify-content-center" id="navbarNavDropdown">
            <ul class="navbar-nav">
                <li class="nav-item">
                    <a class="nav-link" aria-current="page" href="register.php">Registrácia</a>
                </li>
        </div>
    </nav>
</header>

<div class="container border rounded bg-secondary">
    <div class="width-50 p-5">
        <div class="border p-4 bg-light">
            <h2 class="justify-content-center center-text">Zadajte Vaše údaje</h2>
            <hr class="mb-4 mt-3">
            <form action="<?php echo htmlspecialchars($_SERVER['PHP_SELF']); ?>" method="post">

                <input type="hidden" name="id" value=" ">

                <div class="mb-3 row">
                    <div class="col form-group">
                        <label for="email" class="form-label">Email*</label><input type="email" name="email" id="email"
                                                                                   class="form-control"
                                                                                   placeholder="name@example.com"
                                                                                   required>
                    </div>
                    <div class="col form-group">
                        <label for="heslo" class="form-label">Heslo*</label><input type="password" name="heslo"
                                                                                   id="heslo" class="form-control"
                                                                                   required>
                    </div>
                </div>
                <div class="mb-3 row">
                    <div class="col form-group">
                        <button type="submit" class="btn btn-primary" name="btn-edit-person" id="btn-edit-person">
                            Prihlásiť
                        </button>
                    </div>
                </div>
                <?php
                if (!empty($error_msg)) {
                    echo $error_msg;
                }?>
            </form>
        </div>
    </div>
</div>

<footer class="d-flex justify-content-center py-3 my-4 mt-4 border-top border-dark-subtle">
    <span class="text-muted">© 2023 Úvod do počítačovej bezpečnosti</span>
</footer>

<script src="https://cdn.datatables.net/1.13.2/js/jquery.dataTables.min.js"></script>
<script src="https://cdn.datatables.net/1.13.2/js/dataTables.bootstrap5.min.js"></script>

</body>
</html>