<?php

require_once('config.php');

session_start();

try {
    $db = new PDO("mysql:host=$hostname;dbname=$dbname", $username, $password);
    $db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
}catch (PDOException $e){
    echo $e->getMessage();
}

if (!isset($_SESSION["email"]) || $_SESSION["loggedin"] !== true) {
    header("Location: login.php");
    exit;
}
$query = "SELECT ucet.iban FROM pouzivatel JOIN ucet ON pouzivatel.id = ucet.pouzivatel_id where pouzivatel.email=?";
$stmt = $db->prepare($query);
$stmt->execute([$_SESSION["email"]]);
$iban = $stmt->fetch(PDO::FETCH_ASSOC);




function checkValidIBAN($iban){
    if (preg_match('/^[A-Z]{2}\d+$/', $iban)) {
        return true;
    }
    return false;
}
function checkSum($suma){
    if ($suma > 0) {
        return true;
    }
    return false;
}


if ($_SERVER["REQUEST_METHOD"] == "POST") {
    $errmsg = "";


    if (!checkValidIBAN($_POST['naucet'])) {
        $errmsg .= "<p class='text-danger'>Číslo účtu príjemcu nie je zadané v správnom formáte.</p>";
    }
    if (!checkSum($_POST['suma'])) {
        $errmsg .= "<p class='text-danger'>Zadaná suma musí byť kladná.</p>";
    }
    if (empty($errmsg)) {
        $sql = "INSERT INTO platba (z_uctu,na_ucet, nazov_prijemcu, variabilny_symbol, suma) VALUES (:z_uctu, :na_ucet, :nazov_prijemcu, :variabilny_symbol, :suma)";

        $z_uctu = $iban['iban'];
        $na_ucet = $_POST['naucet'];
        $nazov_prijemcu = $_POST['menoprijemcu'];
        $variabilny_symbol = $_POST['variabilnysymbol'];
        $suma = $_POST['suma'];

        $stmt = $db->prepare($sql);

        $stmt->bindParam(":z_uctu", $z_uctu, PDO::PARAM_STR);
        $stmt->bindParam(":na_ucet", $na_ucet, PDO::PARAM_STR);
        $stmt->bindParam(":nazov_prijemcu", $nazov_prijemcu, PDO::PARAM_STR);
        $stmt->bindParam(":variabilny_symbol", $variabilny_symbol , PDO::PARAM_STR);
        $stmt->bindParam(":suma", $suma, PDO::PARAM_STR);


        $stmt->execute();

        $query = "SELECT ucet.zostatok FROM pouzivatel JOIN ucet ON pouzivatel.id = ucet.pouzivatel_id where pouzivatel.email=?";
        $stmt = $db->prepare($query);
        $stmt->execute([$_SESSION["email"]]);
        $zostatok = $stmt->fetch(PDO::FETCH_ASSOC);

        $zostatok = $zostatok['zostatok'] - $_POST['suma'];

        $sql = "UPDATE ucet SET zostatok=? WHERE iban=?";
        $stmt = $db->prepare($sql);
        $success = $stmt->execute([$zostatok, $iban['iban']]);

        $query = "SELECT ucet.zostatok FROM ucet where ucet.iban=?";
        $stmt = $db->prepare($query);
        $stmt->execute([$na_ucet]);
        $zostatok_ucet_prijemcu = $stmt->fetch(PDO::FETCH_ASSOC);


        if(!empty($zostatok_ucet_prijemcu)){
            $zostatok_ucet_prijemcu = $zostatok_ucet_prijemcu['zostatok'] + $_POST['suma'];


            $sql = "UPDATE ucet SET zostatok=? WHERE iban=?";
            $stmt = $db->prepare($sql);
            $success = $stmt->execute([$zostatok_ucet_prijemcu, $_POST['naucet']]);

        }

        header("Location: index.php");


    }
    unset($stmt);
    unset($pdo);
}


?>
<!doctype html>
<html lang="sk">
<head>
    <meta charset="UTF-8">
    <meta name="viewport"
          content="width=device-width, user-scalable=no, initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0">
    <meta http-equiv="X-UA-Compatible" content="ie=edge">
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-GLhlTQ8iRABdZLl6O3oVMWSktQOp6b7In1Zl3/Jr59b6EGGoI1aFkw7cmDA6j6gD" crossorigin="anonymous">
    <link rel="stylesheet" href="https://cdn.datatables.net/1.13.2/css/dataTables.bootstrap5.min.css">
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.6.3/jquery.min.js" integrity="sha512-STof4xm1wgkfm7heWqFJVn58Hm3EtS31XFaagaa8VMReCXAkQnJZ+jEy8PCC/iT18dFy95WcExNHFTqLyp72eQ==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/js/bootstrap.bundle.min.js" integrity="sha384-w76AqPfDkMBDXo30jS1Sgez6pr3x5MlQ1ZAGC+nuZB+EYdgRZgiwxhTBTkF7CXvN" crossorigin="anonymous"></script>

    <title>Platba</title>
</head>
<body class=" d-flex flex-column min-vh-100">
<header>
<nav class=" navbar navbar-expand-lg bg-body-tertiary" data-bs-theme="dark">
    <div class="container-fluid">
        <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarNav" aria-controls="navbarNav" aria-expanded="false" aria-label="Toggle navigation">
            <span class="navbar-toggler-icon"></span>
        </button>
        <div class="collapse navbar-collapse justify-content-between" id="navbarNav">
            <ul class="navbar-nav">
                <li class="nav-item">
                    <a class="nav-link active" aria-current="page" href="index.php">Domov</a>
                </li>
            </ul>
            <ul class="navbar-nav ">
                <li class="nav-item ">
                    <a class="nav-link" href="logout.php">Odhlásenie</a>
                </li>
            </ul>
        </div>
    </div>
</nav>
</header>
<div class="d-flex pt-2 justify-content-center"><h1>Prevod na účet</h1></div>
<div class="container border rounded bg-secondary">
    <div class="width-50 p-5">
        <div class="border p-4 bg-light">
            <h2 class="justify-content-center center-text">Zadajte platobné údaje</h2>
            <hr class="mb-4 mt-3">
            <form action="<?php echo htmlspecialchars($_SERVER['PHP_SELF']);?>" method="post">
                <div class="mb-3 row">
                    <input type="hidden" name="id" value=" ">
                    <div class="col form-group">
                        <label for="zuctu" class="form-label">Z účtu*</label><input type="text" name="zuctu" id="zuctu" class="form-control"value="<?php echo $iban["iban"]; ?>" required>
                    </div>
                    <div class="col form-group">
                        <label for="naucet" class="form-label">IBAN - číslo účtu príjemcu*</label><input type="text" name="naucet" id="naucet" class="form-control" required>
                    </div>
                </div>
                <div class="mb-3 row">
                    <div class="col form-group">
                        <label for="menoprijemcu" class="form-label">Meno príjemcu*</label><input type="text" name="menoprijemcu" id="menoprijemcu" class="form-control" required>
                    </div>
                    <div class="col form-group">
                        <label for="suma" class="form-label">Suma*</label><input type="number" name="suma" id="suma" class="form-control" step="any" required>
                    </div>
                    <div class="col form-group">
                        <label for="variabilnysymbol" class="form-label">Variabilný symbol*</label><input type="text" name="variabilnysymbol" id="variabilnysymbol" class="form-control" required>
                    </div>
                </div>
                <div class="mb-3 row">
                    <div class="col form-group">
                        <button type="submit" class="btn btn-primary" name="btn-vykonaj-platbu" id="btn-vykonaj-platbu">Vykonať prevod</button>
                    </div>
                </div>
                <?php
                if (!empty($errmsg)) {
                    echo $errmsg;
                }?>
            </form>
        </div>
    </div>
</div>

<footer class="d-flex justify-content-center py-3 my-4 mt-auto border-top border-dark-subtle">
    <div><span class="text-muted">© 2023 Úvod do počítačovej bezpečnosti</span></div>
</footer>

<script src="https://cdn.datatables.net/1.13.2/js/jquery.dataTables.min.js"></script>
<script src="https://cdn.datatables.net/1.13.2/js/dataTables.bootstrap5.min.js"></script>

</body>
</html>
