<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);
error_reporting(E_WARNING);
require_once('config.php');
require_once 'insert_edit.php';
session_start();

try {
    $db = new PDO("mysql:host=$hostname;dbname=$dbname", $username, $password);
    $db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
}catch (PDOException $e){
    echo $e->getMessage();
}

function checkEmpty($field) {
    if (empty(trim($field))) {
        return true;
    }
    return false;
}
function checkLength($field, $min, $max) {
    $string = trim($field);
    $length = strlen($string);
    if ($length < $min || $length > $max) {
        return false;
    }
    return true;
}

function userExist($db, $email) {
    $exist = false;

    $param_email = trim($email);

    $sql = "SELECT id FROM pouzivatel WHERE email = :email";
    $stmt = $db->prepare($sql);
    $stmt->bindParam(":email", $param_email, PDO::PARAM_STR);

    $stmt->execute();

    if ($stmt->rowCount() == 1) {
        $exist = true;
    }
    unset($stmt);
    return $exist;
}

function checkValidPassword($password){
    if (preg_match('/^(?=.*[A-Z])(?=.*[0-9])(?=.*[^A-Za-z0-9]).*$/', $password)) {
        return true;
    }
    return false;
}


if ($_SERVER["REQUEST_METHOD"] == "POST") {
    $errmsg = "";

    $password = $_POST['password'];
    $compromised_passwords = file('passwords.txt', FILE_IGNORE_NEW_LINES);

    if (in_array($_POST['password'], $compromised_passwords)) {

        $errmsg .= "<p class='text-danger'>Heslo patrí medzi najčastejšie používané heslá.</p>";
    }
    if (checkLength($_POST['password'], 6, 32) === false) {
        $errmsg .= "<p class='text-danger'>Heslo musí mať 6 až 32 znakov.</p>";
    } elseif (!checkValidPassword($_POST['password'])) {
        $errmsg .= "<p class='text-danger'>Heslo musí obsahovať aspoň 1 veľké písmeno, aspoň 1 malé písmeno, aspoň 1 číslicu a aspoň 1 špeciálny znak.</p>";
    } elseif (checkFullname($_POST['firstname'], $_POST['lastname']) == false) {
        $errmsg .= "<p class='text-danger'>Meno alebo priezvisko je zadané nesprávne.</p>";
    }

    if (userExist($db, $_POST['email']) === true) {
        $errmsg .= "<p class='text-danger'>Používateľ s týmto e-mailom už existuje.</p>";

    }

    if (empty($errmsg)) {
        $sql = "INSERT INTO pouzivatel (meno, email, heslo, cislo_op) VALUES (:meno, :email, :password, :cislo_op)";

        $meno = $_POST['firstname'] . ' ' . $_POST['lastname'];
        $email = $_POST['email'];
        $hashed_password = password_hash($_POST['password'], PASSWORD_BCRYPT); // pouzity PASSWORD_BCRYPT, salt sa generuje automaticky - docs
        $cislo_op = $_POST['cislo_op'];

        $stmt = $db->prepare($sql);

        $stmt->bindParam(":meno", $meno, PDO::PARAM_STR);
        $stmt->bindParam(":email", $email, PDO::PARAM_STR);
        $stmt->bindParam(":password", $hashed_password, PDO::PARAM_STR);
        $stmt->bindParam(":cislo_op", $cislo_op, PDO::PARAM_STR);

        $stmt->execute();


        $sql = "SELECT id FROM pouzivatel WHERE email = :email";
        $stmt = $db->prepare($sql);
        $stmt->bindParam(":email", $_POST['email'], PDO::PARAM_STR);

        $stmt->execute();
        $result = $stmt->fetch(PDO::FETCH_ASSOC);

        $sql = "INSERT INTO ucet (pouzivatel_id, iban, zostatok) VALUES (:pouzivatel_id, :iban, :zostatok)";

        $pouzivatel_id = $result['id'];
        $iban = "SK8975000000000000000000" . $pouzivatel_id;
        $zostatok = 450000.0;

        $stmt = $db->prepare($sql);

        $stmt->bindParam(":pouzivatel_id", $pouzivatel_id, PDO::PARAM_STR);
        $stmt->bindParam(":iban", $iban, PDO::PARAM_STR);
        $stmt->bindParam(":zostatok", $zostatok, PDO::PARAM_STR);

        $stmt->execute();


        header("Location: login.php");

    }
    unset($stmt);
    unset($pdo);
}


?>

<!doctype html>
<html lang="sk">
<head>
    <meta charset="UTF-8">
    <meta name="viewport"
          content="width=device-width, user-scalable=no, initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0">
    <meta http-equiv="X-UA-Compatible" content="ie=edge">
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-GLhlTQ8iRABdZLl6O3oVMWSktQOp6b7In1Zl3/Jr59b6EGGoI1aFkw7cmDA6j6gD" crossorigin="anonymous">
    <link rel="stylesheet" href="https://cdn.datatables.net/1.13.2/css/dataTables.bootstrap5.min.css">
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.6.3/jquery.min.js" integrity="sha512-STof4xm1wgkfm7heWqFJVn58Hm3EtS31XFaagaa8VMReCXAkQnJZ+jEy8PCC/iT18dFy95WcExNHFTqLyp72eQ==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/js/bootstrap.bundle.min.js" integrity="sha384-w76AqPfDkMBDXo30jS1Sgez6pr3x5MlQ1ZAGC+nuZB+EYdgRZgiwxhTBTkF7CXvN" crossorigin="anonymous"></script>
<!--    <link rel="stylesheet" href="css/utility.css">-->

    <title>Registrácia</title>
</head>
<body>

<div class="d-flex pt-2 justify-content-center"><h1>Registrácia</h1></div>

<header class="mb-3 mt-2" >
    <nav class="navbar navbar-expand bg-body-tertiary" data-bs-theme="dark">
        <div class="collapse navbar-collapse justify-content-center" id="navbarNavDropdown">
            <ul class="navbar-nav">
                <li class="nav-item">
                    <a class="nav-link" aria-current="page" href="login.php">Prihlásenie</a>
                </li>
        </div>
    </nav>
</header>

<div class="container border rounded bg-secondary">
    <div class="width-50 p-5">
        <div class="border p-4 bg-light">
            <h2 class="justify-content-center center-text">Zadajte Vaše údaje</h2>
            <hr class="mb-4 mt-3">
            <form action="<?php echo htmlspecialchars($_SERVER['PHP_SELF']);?>" method="post">
                <div class="mb-3 row">
                    <input type="hidden" name="id" value=" ">
                    <div class="col form-group">
                        <label for="firstname" class="form-label">Meno*</label><input type="text" name="firstname" id="firstname" class="form-control" required>
                    </div>
                    <div class="col form-group">
                        <label for="lastname" class="form-label">Priezvisko*</label><input type="text" name="lastname" id="lastname" class="form-control" required>
                    </div>
                </div>
                <div class="mb-3 row">
                    <div class="col form-group">
                        <label for="email" class="form-label">Email*</label><input type="email" name="email" id="email" class="form-control" placeholder="name@example.com" required>
                    </div>
                    <div class="col form-group">
                        <label for="cislo_op" class="form-label">Cislo OP*</label><input type="text" name="cislo_op" id="cislo_op" class="form-control" required>
                    </div>
                    <div class="col form-group">
                        <label for="password" class="form-label">Heslo*</label><input type="password" name="password" id="password" class="form-control" required>
                    </div>
                </div>
                <div class="mb-3 row">
                    <div class="col form-group">
                        <button type="submit" class="btn btn-primary" name="btn-edit-person" id="btn-edit-person">Registrovať</button>
                    </div>
                </div>
                <?php
                if (!empty($errmsg)) {
                    echo $errmsg;
                }?>
            </form>
        </div>
    </div>
</div>

<footer class="d-flex justify-content-center py-3 my-4 mt-4 border-top border-dark-subtle">
    <span class="text-muted">© 2023 Úvod do počítačovej bezpečnosti</span>
</footer>

<script src="https://cdn.datatables.net/1.13.2/js/jquery.dataTables.min.js"></script>
<script src="https://cdn.datatables.net/1.13.2/js/dataTables.bootstrap5.min.js"></script>

</body>
</html>
