from fastapi import Header, HTTPException
from passlib.context import CryptContext
from datetime import datetime, timedelta
from jose import jwt
from passlib.context import CryptContext
from typing import Union
import re

from services.user_service import get_user_by_email

from models.models import User

SECRET_KEY = "09d25e094faa6ca2556c818166b7a9563b93f7099f6f0f4caa6cf63b88e8d3e7"
ALGORITHM = "HS256"
ACCESS_TOKEN_EXPIRE_MINUTES = 30

pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")


def hash_password(password: str):
    return pwd_context.hash(password)


def verify_password(stored_password: str, plain_password: str):
    return pwd_context.verify(plain_password, stored_password)


def create_access_token(data: dict, expires_delta: Union[timedelta, None] = None):
    to_encode = data.copy()
    if expires_delta:
        expire = datetime.utcnow() + expires_delta
    else:
        expire = datetime.utcnow() + timedelta(minutes=15)
    to_encode.update({"exp": expire})
    encoded_jwt = jwt.encode(to_encode, SECRET_KEY, algorithm=ALGORITHM)
    return encoded_jwt


def get_user_email_from_token(token: str):
    return jwt.decode(token, SECRET_KEY, algorithms=[ALGORITHM])


async def get_user_from_token(authorization: str = Header(None, title="Authorization")) -> User:
    try:
        token = authorization.split(" ")[1]
        email = get_user_email_from_token(token)
        user = get_user_by_email(email["sub"])
    except Exception as e:
        raise HTTPException(
            status_code=401, detail="Invalid token"
        )
    # get user by email from db, give me code

    return user


def validate_password_complexity(password):
    pattern = re.compile(r'^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)[A-Za-z\d]{8,}$')
    return pattern.match(password)
