from flask import Flask, render_template, url_for, flash, redirect, request, jsonify, session, make_response
from flask_sqlalchemy import SQLAlchemy
from sqlalchemy.sql import text
from Crypto.Cipher import AES, PKCS1_OAEP
from Crypto.Random import get_random_bytes
from Crypto.PublicKey import RSA
import os
from time import gmtime
from datetime import datetime, timezone, date
from calendar import timegm
from hashlib import sha512
from uuid import uuid4
from sqlalchemy import create_engine, MetaData, Table, Column, Integer, String, LargeBinary
from codecs import decode
from forms import RegistrationForm, LoginForm, CreatePatient, CreateMedicalRecord, MedicalInvestigationForm
from flask_login import LoginManager, UserMixin, login_user, current_user, logout_user
from sqlalchemy.orm import declarative_base
from sqlalchemy import Date, or_
from datetime import datetime
import pdfkit


"""SQL for init database:

CREATE TABLE message (ID INT NOT NULL, messageText VARCHAR(255), PRIMARY KEY (ID));
CREATE TABLE users (ID INT NOT NULL, username VARCHAR(255), password VARCHAR(255), PRIMARY KEY (ID));

INSERT INTO message VALUES (0, 'Toto je veta ktoru by som mal v1d1et len ja a potom ty!');
INSERT INTO users VALUES (0, 'petko','heslo');
INSERT INTO users VALUES (1, 'matko','pass');

"""
app = Flask(__name__)
app.config['SECRET_KEY'] = '311a923103e0c411ebfa4f40f70b1040'
app.config['SQLALCHEMY_DATABASE_URI'] = 'postgresql://postgres:postgres@flask_pgdb:5432/postgres'
db = SQLAlchemy(app)
engine = create_engine("postgresql://postgres:postgres@flask_pgdb:5432/postgres")
login_manager = LoginManager(app)
Base = declarative_base()

MAX_LOGIN_ATTEMPTS = 5
TIMEOUT_PERIOD_SECONDS = 300

with open('common_passwords.txt', 'r') as file:
    common_passwords = [line.strip() for line in file.readlines()]

@login_manager.user_loader
def load_user(user_id):
    return User.query.get(int(user_id))


# Model
class User(db.Model, UserMixin):
    __tablename__ = "users"
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(64), unique=True, nullable=False)
    email = db.Column(db.String(128), unique=True, nullable=False)
    password = db.Column(db.String(246), nullable=False)
    salt = db.Column(db.String(128), nullable=False)
    dateRegistered = db.Column(db.Integer, nullable=False)
    patients = db.relationship('Patients', backref='users', lazy=True)

    def __init__(self, username, email, password, salt, date_registered):
        self.username = username
        self.email = email
        self.password = password
        self.salt = salt
        self.dateRegistered = date_registered


class Patients(db.Model):
    __tablename__ = "patients"
    id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey('users.id'), nullable=False)
    firstName = db.Column(db.String(246), nullable=False)
    lastName = db.Column(db.String(246), nullable=False)
    birthDate = db.Column(db.Date, nullable=False)
    address = db.Column(db.String(246), nullable=False)
    phoneNumber = db.Column(db.String(20), nullable=False)
    email = db.Column(db.String(246), nullable=False)
    gender = db.Column(db.String(246), nullable=False)
    height = db.Column(db.Float, nullable=False)
    weight = db.Column(db.Float, nullable=False)
    bloodType = db.Column(db.String(246), nullable=False) 
    allergies = db.Column(db.String(246), nullable=False)
    healthInsurance = db.Column(db.String(246), nullable=False)
    medicalRecords = db.relationship('MedicalRecord', backref='patient')

    def __init__(self, user_id, firstName, lastName, birthDate, address, phoneNumber, email, gender, height, weight, bloodType, allergies, healthInsurance):
        self.user_id = user_id
        self.firstName = firstName
        self.lastName = lastName
        self.birthDate = birthDate
        self.address = address
        self.phoneNumber = phoneNumber
        self.email = email
        self.gender = gender
        self.height = height
        self.weight = weight
        self.bloodType = bloodType
        self.allergies = allergies
        self.healthInsurance = healthInsurance

class MedicalRecord(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    patient_id = db.Column(db.Integer, db.ForeignKey('patients.id'))
    medical_record = db.Column(db.Text, nullable=False)
    timestamp = db.Column(db.DateTime, default=datetime.utcnow)

class Message(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    messageText = db.Column(db.String(255), nullable=False)


# DB adapter
with app.app_context():
    db.create_all()

# ----------------------------------------------------------------------------
# Default behavior for app
@app.route('/')
def home():
    return render_template('index.html')

@app.route('/view_patients', methods=['GET'])
def view_patients():
    if current_user.is_authenticated:
        patients = Patients.query.filter_by(user_id=current_user.id).all()
        return render_template('view_patients.html', patients=patients)

@app.route('/add_patient', methods=['POST', 'GET'])
def add_patient():
    if current_user.is_authenticated:
        form = CreatePatient()
        if form.validate_on_submit():
            create_patient(form)
            return redirect(url_for('home'))
        return render_template('add_patient.html', title='Add Patient', form=form)
    else:
        return redirect(url_for('login'))

@app.route('/search_patients', methods=['GET'])
def search_patients():
    search_query = request.args.get('search_query', '')

    if current_user.is_authenticated:
        patients = Patients.query.filter(
            or_(Patients.firstName.ilike(f'%{search_query}%'), Patients.lastName.ilike(f'%{search_query}%')),
            Patients.user_id == current_user.id
        ).all()

        return render_template('view_patients.html', patients=patients)
    else:
        return redirect(url_for('login'))

@app.route('/clear_search', methods=['GET'])
def clear_search():
    return redirect(url_for('view_patients'))

@app.route('/delete_patient/<int:patient_id>', methods=['POST'])
def delete_patient(patient_id):
    if current_user.is_authenticated:
        patient = Patients.query.get_or_404(patient_id)
        db.session.delete(patient)
        db.session.commit()
        flash('Patient deleted successfully!', 'success')
    return redirect(url_for('view_patients'))

@app.route('/create_request/<int:patient_id>', methods=['GET', 'POST'])
def create_request(patient_id):
    if current_user.is_authenticated:
        patient = Patients.query.filter_by(id=patient_id, user_id=current_user.id).first()
        if patient:
            form = MedicalInvestigationForm()

            if form.validate_on_submit():
                request_text = form.request_text.data
                diagnosis = form.diagnosis.data
                doctor_type = form.doctor_type.data

                pdf_options = {
                    'page-size': 'A5',
                    'margin-top': '0mm',
                    'margin-right': '0mm',
                    'margin-bottom': '0mm',
                    'margin-left': '0mm',
                }

                rendered_html = render_template(
                    'request_pdf.html',
                    patient=patient,
                    diagnosis=diagnosis,
                    doctor_type=doctor_type,
                    request_text=request_text,
                    current_date=date.today().strftime('%Y-%m-%d'),
                    user_name=current_user.username
                )
                pdf = pdfkit.from_string(rendered_html, False)


                response = make_response(pdf)
                response.headers['Content-Type'] = 'application/pdf'
                response.headers['Content-Disposition'] = f'inline; filename=request_for_{patient.firstName}_{patient.lastName}.pdf'

                flash('Request created successfully!', 'success')
                return response

            return render_template('create_request.html', patient=patient, form=form)
        else:
            flash('Patient not found!', 'danger')
            return redirect(url_for('view_patients'))
    else:
        return redirect(url_for('login'))
    
    
@app.route('/add_record/<user_id>', methods=['POST', 'GET'])
def add_record(user_id):
    if current_user.is_authenticated:
        form = CreateMedicalRecord()
        if form.validate_on_submit():
            record = MedicalRecord(patient_id=user_id, medical_record=form.record.data)

            db.session.add(record)
            db.session.commit()

            flash('Medical Record created successfully!', 'success')
            return redirect(url_for('view_patients'))
        return render_template('add_record.html', title='add record', form=form)
    else:
        return redirect(url_for('login'))


@app.route('/show_record/<user_id>', methods=['POST', 'GET'])
def show_record(user_id):
    if current_user.is_authenticated:
        patient = Patients.query.get_or_404(user_id)
        medical_records = MedicalRecord.query.filter_by(patient_id=user_id).all()
        rendered_html = render_template('medical_records_pdf.html', patient=patient, medical_records=medical_records)

        pdf_options = {
            'page-size': 'A4',
            'margin-top': '0mm',
            'margin-right': '0mm',
            'margin-bottom': '0mm',
            'margin-left': '0mm',
        }

        pdf = pdfkit.from_string(rendered_html, False)

        response = make_response(pdf)
        response.headers['Content-Type'] = 'application/pdf'
        response.headers['Content-Disposition'] = f'inline; filename=user_medical_records_{patient.id}.pdf'

        return response
    else:
        return redirect(url_for('login'))


@app.route('/register', methods=['GET', 'POST'])
def register():
    if current_user.is_authenticated:
        return redirect(url_for('home'))

    form = RegistrationForm()
    if form.validate_on_submit():
        register_user(form)
        if not is_common_password(form.password.data):
            flash(f'Account created for {form.username.data}!', 'success')
            return redirect(url_for('login'))
    return render_template('register.html', title='Register', form=form)


@app.route('/login', methods=['GET', 'POST'])
def login():
    if current_user.is_authenticated:
        return redirect(url_for('home'))
    form = LoginForm()
    if form.validate_on_submit():
        user = User.query.filter_by(email=form.email.data).first()
        if user and check_password(form.email.data, form.password.data):
            login_user(user)
            return redirect(url_for('home'))
        else:
            flash('Login unsuccessful', 'danger')
    return render_template('login.html', title='Login', form=form)


@app.route('/logout')
def logout():
    logout_user()
    return redirect(url_for('home'))


@app.route('/users', methods=['GET'])
def get_all_users():
    users = User.query.all()
    user_list = []
    for user in users:
        user_data = {
            'id': user.id,
            'username': user.username,
            'password': user.password
        }
        user_list.append(user_data)
    return jsonify({'users': user_list})


@app.route('/users', methods=['POST'])
def create_user():
    data = request.get_json()
    username = data.get('username')
    password = data.get('password')

    if not username or not password:
        return jsonify({'error': 'Both name and password are required'}), 400

    new_user = User(username=username, password=password)
    db.session.add(new_user)
    db.session.commit()

    return jsonify({'message': 'User created successfully'}), 201

#odkomentovat ak by neboli ziadne data v tabulkach, respektive by tabulka neexistovala
@app.route('/initDB', methods=['GET'])
def creqate_db():
    with engine.connect() as con:

        meta = MetaData()

        users = Table(
            'users', meta,
            Column('id', Integer, primary_key=True),
            Column('username', String, unique=True),
            Column('password', String),
            Column('email', String, unique=True),
            Column('salt', String),
            Column('dateRegistered', Integer),
        )
        meta.create_all(engine)

        message = Table(
            'message', meta,
            Column('id', Integer, primary_key=True),
            Column('messageText', String),
        )
        meta.create_all(engine)

        new_user = RegistrationForm()

        new_user.username = "test2"
        new_user.password = "jednoducheee"
        new_user.email = "nejaky@mail.com"

        register_user(new_user)

        #print(check_password("nejaky@mail.com", "jednoducheee"))

        return 'Was working'


def our_AES_cypher(bits, data):
    # upravene z: https://pycryptodome.readthedocs.io/en/latest/src/cipher/aes.html
    key = os.urandom(16)
    print(key)
    cipher = AES.new(key, AES.MODE_CCM)

    nonce = cipher.nonce
    ciphertext, tag = cipher.encrypt_and_digest(str.encode(data))
    return ciphertext, tag, cipher


def our_AES_cypher(plaintext):
    # upravene z: https://gist.github.com/jbdatko/7425443
    key = b'Sixteen byte key'
    nonce = get_random_bytes(11)
    cipher = AES.new(key, AES.MODE_CCM, nonce)
    msg = nonce, cipher.encrypt(plaintext), cipher.digest()
    return msg, key
    # We assume that the tuple ``msg`` is transmitted to the receiver:

# upravene z: https://pycryptodome.readthedocs.io/en/latest/src/examples.html
def our_rsa_keypair():
    keyPair = RSA.generate(4096)
    return keyPair

# upravene z : https://cryptobook.nakov.com/digital-signatures/rsa-sign-verify-examples
def our_sign(msg, keyPair):
    hash = int.from_bytes(sha512(msg).digest(), byteorder='big')
    signature = pow(hash, keyPair.d, keyPair.n)
    print("Signature:", hex(signature))
    return signature

# prevzane z : https://stackoverflow.com/questions/48240551/how-to-generate-a-digital-signature-of-a-string-in-python
def our_verify(msg, signature, keyPair):
    hash = int.from_bytes(sha512(msg).digest(), byteorder='big')
    hashFromSignature = pow(signature, keyPair.e, keyPair.n)
    print("Signature valid:", hash == hashFromSignature)
    return (hash, hashFromSignature)


def our_AES_decypher(msg, key):
    # upravene z : https://gist.github.com/jbdatko/7425443
    nonce, ciphertext, mac = msg
    cipher = AES.new(key, AES.MODE_CCM, nonce)
    plaintext = cipher.decrypt(ciphertext)
    try:
        cipher.verify(mac)
        print
        "The message is authentic: pt=%s" % (plaintext)
    except ValueError:
        print
        "Key incorrect or message corrupted"
    return plaintext

def our_cypher_key(publicKey, symmetricKey):
    publicKey = RSA.import_key(publicKey)
    cipherRsa = PKCS1_OAEP.new(publicKey)
    encSessionKey = cipherRsa.encrypt(symmetricKey)
    return encSessionKey

def our_decypher_key(privateKey, encSessionKey):
    privateKey = RSA.import_key(privateKey)
    cipherRsa = PKCS1_OAEP.new(privateKey)
    sessionKey = cipherRsa.decrypt(encSessionKey)
    return sessionKey

@app.route('/message', methods=['GET'])
def get_message():
    theMessage = ""
    with engine.connect() as con:
        messages = con.execute(text('SELECT * FROM message;'))
        messageList = []
        for message in messages:
            message_data = {
                'id': message[0],
                'text': message[1],
            }
            messageList.append(message_data)
            theMessage = message[1]

    theMessage = bytes(theMessage, "utf-8")

    keyPair = our_rsa_keypair()
    keyPair2 = our_rsa_keypair()

    signature = our_sign(theMessage, keyPair2)

    msg, symmetricKey = our_AES_cypher(theMessage)
    nonce, ciphertext, mac = msg

    encSessionKey = our_cypher_key(keyPair.publickey().export_key(), symmetricKey)

    symmetricKey = our_decypher_key(keyPair.export_key(), encSessionKey)

    msgDecyphered = our_AES_decypher(msg, symmetricKey)

    hash1, hash2 = our_verify(msgDecyphered, signature, keyPair2)

    print('symetricky kluc: ' + str(symmetricKey))
    print('verejny kluc (kluc): '+ str(keyPair.publickey().export_key()))
    print( 'privatny kluc (kluc): '+ str(keyPair.export_key()))
    print( 'verejny kluc (podpis): '+ str(keyPair2.publickey().export_key()))
    print( 'privatny kluc (podpis): '+ str(keyPair2.export_key()))
    print( 'podpis: '+ str(signature))
    print( 'hash 1: '+ str(hash1))
    print( 'hash 2: '+ str(hash2))
    print( 'dekryptovana sprava: ' + msgDecyphered.decode(encoding='utf-8'))


    return jsonify({'symetricky kluc': str(symmetricKey),
                    'verejny kluc (kluc)': str(keyPair.publickey().export_key()),
                    'privatny kluc (kluc)': str(keyPair.export_key()),
                    'verejny kluc (podpis)': str(keyPair2.publickey().export_key()),
                    'privatny kluc (podpis)': str(keyPair2.export_key()),
                    'podpis': str(signature),
                    'hash 1': str(hash1),
                    'hash 2': str(hash2),
                    'dekryptovana sprava': msgDecyphered.decode(encoding='utf-8')
                    })


def current_time():
    return timegm(gmtime())


def hash_salt_pswd(password, my_salt):
    if my_salt == "":
        salt = uuid4().hex.encode('utf-8')
    else:
        my_salt = my_salt.replace("\\x", "")
        salt = decode(my_salt, 'hex_codec')
    hashPswd = sha512(password.encode('utf-8') + salt).hexdigest()
    return hashPswd, salt

def create_patient(form: CreatePatient):
    try:    
        patient = Patients(
            user_id=current_user.id,
            firstName=form.firstName.data,
            lastName=form.lastName.data,
            birthDate=date(int(form.birthDate.data.strftime('%Y%m%d')[:4]),int(form.birthDate.data.strftime('%Y%m%d')[4:6]),int(form.birthDate.data.strftime('%Y%m%d')[6:])), 
            address=form.address.data,
            phoneNumber=form.phoneNumber.data,
            email=form.email.data,
            gender=form.gender.data,
            height=form.height.data,
            weight=form.weight.data,
            bloodType=form.bloodType.data,
            allergies=form.allergies.data,
            healthInsurance=form.healthInsurance.data,
        )

        db.session.add(patient)
        db.session.commit()

        flash('Patient succesfully added to system!', 'success')
    except Exception as e:
        flash(f'Error creating user: {str(e)}', 'danger')


def register_user(form: RegistrationForm):
    hashed_password, salt = hash_salt_pswd(form.password.data, "")

    if is_common_password(form.password.data):
        return flash('Registration unsuccessful! Weak password!', 'danger')

    user = User(username=form.username.data, email=form.email.data, password=hashed_password, salt=salt,
                date_registered=current_time())

    db.session.add(user)
    db.session.commit()

    # with engine.connect() as con:
    #     sqlQuery = "INSERT INTO public.users (name, password, salt, dateRegistered) VALUES (%s, %s, %s, %s);"
    #     data = (name, hash_pswd, salt, current_time(), )
    #     try:
    #         con.execute(sqlQuery, data)
    #     except Exception as e:  # unique violation
    #         if e == 23505:
    #             return e  # write something to user
    #
    #     users = con.execute(text('SELECT * FROM users;'))
    #     for user in users:
    #         print(user)


def check_password(email, password):
    with engine.connect() as con:
        sqlQuery = "SELECT salt, password FROM public.users WHERE email=%s;"
        answer = con.execute(sqlQuery, email)

        login_attempts = session.get('login_attempts', 0)
        last_failed_attempt_time = session.get('last_failed_attempt_time')

        if login_attempts >= MAX_LOGIN_ATTEMPTS and last_failed_attempt_time:
            time_since_last_attempt = datetime.utcnow().replace(tzinfo=timezone.utc) - last_failed_attempt_time
            if time_since_last_attempt.total_seconds() < TIMEOUT_PERIOD_SECONDS:
                remaining_timeout = int(TIMEOUT_PERIOD_SECONDS - time_since_last_attempt.total_seconds())
                flash(f'Too many login attempts! Please wait {remaining_timeout} seconds before trying again.', 'danger')
                return False

        for ans in answer:
            hash_pswd_given, nothing = hash_salt_pswd(password, ans[0])
            if hash_pswd_given == ans[1]:
                session.pop('login_attempts', None)
                return True
            else:
                session['login_attempts'] = login_attempts + 1
                session['last_failed_attempt_time'] = datetime.utcnow().replace(tzinfo=timezone.utc)
                flash('Invalid email or password. Please try again.', 'danger')
                return False


## Not used function at ALL! :D funny stuff (there is no real logic in it)
def can_sign(username):
    with engine.connect() as con:
        sqlQuery = "SELECT dateRegistered FROM public.users WHERE username=%s;"
        data = (username)
        answer = con.execute(sqlQuery, data)

        print(answer)
        for ans in answer:
            if ans[0]+5 <= current_time():
                return True
            else:
                return False

def is_common_password(password):
    return password in common_passwords


if __name__ == '__main__':
    app.run(host = "0.0.0.0")