from flask_wtf import FlaskForm
from wtforms import DecimalField, StringField, PasswordField, SubmitField, DateField, IntegerField, SelectField, TextAreaField, FileField
from wtforms.validators import DataRequired, Length, Email, Regexp, EqualTo, ValidationError


class RegistrationForm(FlaskForm):
    username = StringField('Username', validators=[DataRequired(), Length(min=2, max=25)])
    email = StringField('Email', validators=[DataRequired(), Email()])
    password = PasswordField('Password', validators=[
                                         DataRequired(),
                                         Length(min=8),
                                         Regexp(r'^(?=.*[A-Z])(?=.*\d)', message='Password must contains at least one capital letter and one number')
                                         ])
    confirm_password = PasswordField('Confirm passowrd', validators=[DataRequired(), EqualTo('password')])
    submit = SubmitField('Sign Up')

    # def validate_email(self, email):
    #     user = User.query.filter_by(username=email.data).first()
    #     if user:
    #         raise ValidationError('This email is already taken')




class LoginForm(FlaskForm):
    email = StringField('Email', validators=[DataRequired(), Email()])
    password = PasswordField('Password', validators=[DataRequired()])
    submit = SubmitField('Login')


class CreatePatient(FlaskForm):
    gender_choices = [('male', 'Male'), ('female', 'Female'), ('other', 'Other')]

    firstName = StringField('First Name', validators=[DataRequired()])
    lastName = StringField('Last Name', validators=[DataRequired()])
    birthDate = DateField('Date of Birth', validators=[DataRequired()])
    address = StringField('Address', validators=[DataRequired()])
    phoneNumber = StringField('Phone Number', validators=[DataRequired()])
    email = StringField('Email', validators=[DataRequired(), Email()])
    gender = SelectField('Gender',choices=gender_choices, validators=[DataRequired()])
    height = DecimalField('Height (kg)', validators=[DataRequired()])
    weight = DecimalField('Weight (cm)', validators=[DataRequired()])
    bloodType = StringField('Blood Type', validators=[DataRequired()])
    allergies = TextAreaField('Allergies', validators=[DataRequired()])
    healthInsurance = StringField('Health Insurance', validators=[DataRequired()])
    saveData = SubmitField('Save Patient Data')


class CreateMedicalRecord(FlaskForm):
    record = TextAreaField('Medical Record', validators=[DataRequired()])
    submit = SubmitField('Save')

class MedicalInvestigationForm(FlaskForm):
    doctor_choices = [('doctor_type_1', 'Doctor Type 1'), ('doctor_type_2', 'Doctor Type 2'), ('doctor_type_3', 'Doctor Type 3')]

    diagnosis = TextAreaField('Diagnosis', validators=[DataRequired()])
    doctor_type = SelectField('Doctor type', choices=doctor_choices, validators=[DataRequired()])
    request_text = TextAreaField('Request Details', validators=[DataRequired()])
