const express = require("express");
const fs = require("fs");
const authController = express.Router();
const authService = require("../services/authService");
const requestLimiter = require("../middleware/requestLimiter");
const db = require("../db/connect");
const { requireProps } = require("../services/utils");
const jwt = require("jsonwebtoken");
const crypto = require("node:crypto");

authController.post("/register", requestLimiter, async (req, res) => {
  if (!requireProps(["firstname", "lastname", "email", "password"], req.body)) {
    return res.status(400).send({ error: "Required field is missing!" });
  }
  const result = await authService.registerUser(db, req.body);
  const jwtPrivKey = fs.readFileSync("./src/runtime/jwt.key");
  const symmKey = crypto.randomBytes(32).toString("hex");
  const vector = crypto.randomBytes(16).toString("hex");
  if (result.ok) {
    const token = jwt.sign(
      { userSystemId: result.data.system_id, symmKey: symmKey, vector: vector },
      jwtPrivKey,
      { algorithm: "RS256", expiresIn: "10000s" }
    );
    return res.status(200).send({ token: token });
  } else {
    return res.status(400).send({ error: result.result });
  }
});

authController.post("/login", requestLimiter, async (req, res) => {
  if (!requireProps(["email", "password"], req.body)) {
    return res.status(400).send({ error: "Required field is missing!" });
  }
  const result = await authService.loginUser(db, req.body);
  const jwtPrivKey = fs.readFileSync("./src/runtime/jwt.key");
  const symmKey = crypto.randomBytes(32).toString("hex");
  const vector = crypto.randomBytes(16).toString("hex");
  if (result.ok) {
    const token = jwt.sign(
      { userSystemId: result.data.system_id, symmKey: symmKey, vector: vector },
      jwtPrivKey,
      { algorithm: "RS256", expiresIn: "10000s" }
    );
    return res.status(200).send({ token: token });
  } else {
    return res.status(401).send();
  }
});

module.exports = authController;
