const express = require("express");
const db = require("../db/connect");
const verifyJWT = require("../middleware/verifyJWT");
const { objFilter } = require("../services/utils");
const encrypt = require("../middleware/encrypt");
const { requireProps } = require("../services/utils");

const userController = express.Router();

userController.get("/users", verifyJWT, async (req, res) => {
  const result = await db.query(
    "SELECT id, name, surname, system_id, email FROM users"
  );
  res.status(200).send({ users: result.rows });
});

userController.get("/loggedInUser", verifyJWT, async (req, res) => {
  let user = res.locals.user;
  user = objFilter(user, ["id", "name", "surname", "email"]);
  return res.status(200).send(user);
});

userController.put("/updateUserPubKey", verifyJWT, async (req, res) => {
  const user = res.locals.user;
  const publicKey = req.body.publicKey;
  try {
    await db.query("UPDATE users SET public_key = $1 WHERE system_id = $2", [
      publicKey,
      user.system_id,
    ]);
  } catch (err) {
    return res.sendStatus(500);
  }
  return res.sendStatus(200);
});

userController.put("/updateUser", verifyJWT, async (req, res) => {
  if (!requireProps(["firstname", "lastname"], req.body)) {
    return res.status(400).send({ error: "Required field is missing!" });
  }
  const userSystemId = res.locals.user.system_id;
  try {
    await db.query("UPDATE users SET (name, surname) = ($1, $2) WHERE system_id = $3",
    [req.body.firstname, req.body.lastname, userSystemId]);
    res.status(200).send();
  } catch(err) {
    return res.status(500).send();
  }

});

module.exports = userController;
