const { Pool } = require('pg')
const pgMigrations = require("postgres-migrations")

const dbConfig = {
  user: process.env.DB_USER,
  host: process.env.DB_HOST,
  database: process.env.DB_NAME,
  password: process.env.DB_PASS,
  port: Number(process.env.DB_PORT),
  ensureDatabaseExists: true,
  defaultDatabase: process.env.DB_NAME
}

const pool = new Pool(dbConfig)

const tx = async (callback) => {
  const client = await pool.connect()
  try {
    await client.query('BEGIN')
    await client.query('SET TRANSACTION ISOLATION LEVEL SERIALIZABLE')
    try {
      await callback(client)
      client.query('COMMIT')
    } catch (e) {
      client.query('ROLLBACK')
      throw e
    }
  } finally {
    client.release()
  }
}

async function initDb() {
  try {
    await pgMigrations.migrate(dbConfig, "src/db/migrations")
  } catch (err) {
    console.error(err)
    throw err;
  }
}

module.exports = {
  query: async (text, params) => await pool.query(text, params),
  tx: tx,
  init: initDb,
}
