const { encryptData } = require("../services/utils");
const { consoleLogger } = require("../services/utils");
const interceptor = require("express-interceptor");

// Encrypt response, this middleware is call after controller
// return format { secret: {}, data: {} }
// where secret are encrypted data and data everything else
const encrypt = interceptor((req, res) => ({
  isInterceptable: () => true,
  intercept: () => {
    const publicKey = res.locals.user.public_key;
    if (!publicKey) {
      consoleLogger(["Missing public key", req.method, req.path]);
    }
    let result = encryptData(req.body.secret, publicKey);
    if (result.ok) {
      let response = {
        secret: result.data,
        data: req.body.data,
      };
      return res.status(200).send(response);
    }
    consoleLogger(["Encryption error", req.method, req.path]);
    return res.status(400);
  },
}));

module.exports = encrypt;
