const axios = require('axios');
const crypto = require('crypto');

async function isCommonPassword(password) {
    try {
        // approx response latency
        // top 1 000     ... ~90ms
        // top 10 000    ... ~110ms
        // top 100 000   ... ~250ms
        // top 1 000 000 ... ~ 800ms
        const COMMON_PASSWORDS_URL = 'https://raw.githubusercontent.com/danielmiessler/SecLists/master/Passwords/Common-Credentials/10-million-password-list-top-100000.txt';
        
        const response = await axios.get(COMMON_PASSWORDS_URL);
        const commonPasswords = response.data.split('\n');
        
        if (commonPasswords.includes(password)) {
            console.log('[INFO]:\t common password');
            return true;
        } else {
            console.log('[INFO]:\t not common password');
            return false;
        }
    } catch (error) {
        console.error('[ERROR]:', error);
    }
}

async function isPasswordExposed(password) {
    try {
        const sha1Hash = crypto.createHash('sha1').update(password).digest('hex').toUpperCase();
        const prefix = sha1Hash.slice(0, 5);
        const suffix = sha1Hash.slice(5);
        
        const HIBP_API_URL = `https://api.pwnedpasswords.com/range/${prefix}`;
        const response = await axios.get(HIBP_API_URL);
        const hashes = response.data.split('\r\n');
        console.l
        
        for (const hash of hashes) {
            if (hash.startsWith(suffix)) {
                console.log('[INFO]:\t Password has been exposed.');
                return true;
            }
        }
        
        console.log('[INFO]:\t Password hasn\'t been exposed (yet...).');
        return false;
    } catch (error) {
        console.error('[ERROR]:', error);
    }
}

function hashSaltPassword(password) {
    const salt = crypto.randomBytes(16).toString('hex');

    const hash = crypto.pbkdf2Sync(password, salt, 1000, 64, 'sha256').toString('hex');

    return {
        salt: salt,
        hash: hash
    };
}

function verifySaltyPassword(storedHash, storedSalt, passwordToVerify) {
    const hashVerify = crypto.pbkdf2Sync(passwordToVerify, storedSalt, 1000, 64, 'sha256').toString('hex');
    return storedHash === hashVerify;
}

module.exports = {
    isCommonPassword,
    isPasswordExposed,
    hashSaltPassword,
    verifySaltyPassword,
}