const cryptoService = require("../services/cryptoService");
const pdfService = require("../services/pdfService");
const fs = require("fs");
const path = require("path");
const { consoleLogger } = require("./utils");

async function checkAllKeyDependecies() {
  consoleLogger(["Keys check started:"]);

  const KEYS_TARGET_PATH = cryptoService.getKeysPath();
  const GENEREATED_DEST_PATH = pdfService.getDestPath();

  if (!fs.existsSync(KEYS_TARGET_PATH)) {
    consoleLogger(["Created src/runtime dir"]);
    fs.mkdirSync(KEYS_TARGET_PATH, { recursive: true });
  }
  
  if (!fs.existsSync(GENEREATED_DEST_PATH)) {
    consoleLogger(["Created generated pdf dir"]);
    fs.mkdirSync(GENEREATED_DEST_PATH, { recursive: true });
  }

  try {
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "hmac-key.pem"));
    consoleLogger(["\t✔ HMAC key"]);
  } catch (err) {
    consoleLogger(["\t✖ HMAC key"]);
    await cryptoService.generateHmacKey();
  }

  try {
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "jwt.key"));
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "jwt_pub.key"));
    consoleLogger(["\t✔ JWT private key"]);
    consoleLogger(["\t✔ JWT public key"]);
  } catch (err) {
    consoleLogger(["\t✖ JWT private key"]);
    consoleLogger(["\t✖ JWT public key"]);
    await cryptoService.generateJwtKeys();
  }

  try {
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "private-key.pem"));
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "public-key.pem"));
    consoleLogger(["\t✔ RSA private key"]);
    consoleLogger(["\t✔ RSA public key"]);
  } catch (err) {
    consoleLogger(["\t✖ RSA private key"]);
    consoleLogger(["\t✖ RSA public key"]);
    await cryptoService.generateCryptoKey();
  }

  try {
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "sym-key.pem"));
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "sym-key-vector.pem"));
    consoleLogger(["\t✔ SYM private key"]);
    consoleLogger(["\t✔ SYM vector"]);
  } catch (err) {
    consoleLogger(["\t✖ SYM private key"]);
    consoleLogger(["\t✖ SYM vector"]);
    await cryptoService.generateSymKey();
  }

  consoleLogger(["Keys check ended"]);
}

module.exports = {
  checkAllKeyDependecies,
};
