const { format } = require("date-fns");
const forge = require("node-forge");
const fs = require("fs");

function requireProps(requiredList, object) {
  for (let i = 0; i < requiredList.length; i++) {
    if (!object[requiredList[i]]) {
      return false;
    }
  }
  return true;
}

function encryptData(secret, pubKeyPem) {
  try {
    const pubKey = forge.pki.publicKeyFromPem(pubKeyPem);
    const encodedData = forge.util.encodeUtf8(JSON.stringify(secret));
    const encryptedData = pubKey.encrypt(encodedData, "RSA-OAEP", {
      md: forge.md.sha256.create(),
    });
    return { ok: true, data: encryptedData };
  } catch (error) {
    consoleLogger(["Encryption error"]);
    return { ok: false, result: "Encryption error" };
  }
}

function symmetricEncryptData(secret, key, vector) {
  try {
    var cipher = forge.cipher.createCipher("AES-GCM", forge.util.createBuffer(key));
    cipher.start({iv: forge.util.createBuffer(vector)});
    cipher.update(forge.util.createBuffer(secret));
    cipher.finish();
    return { ok: true, data: cipher.output, tag: cipher.mode.tag.toHex() };
  } catch (error) {
    consoleLogger(["Encryption error", error]);
    return { ok: false, result: "Encryption error" };
  }
}

function decryptData(secret) {
  try {
    const privKeyPem = fs.readFileSync("./src/runtime/private-key.pem");
    const privKey = forge.pki.privateKeyFromPem(privKeyPem.toString());
    const decryptedData = privKey.decrypt(secret, "RSA-OAEP", {
      md: forge.md.sha256.create(),
    });
    const decodedData = forge.util.decodeUtf8(decryptedData);
    return { ok: true, data: JSON.parse(decodedData) };
  } catch (error) {
    consoleLogger(["Decryption error"]);
    return { ok: false, result: "Decryption error" };
  }
}

function consoleLogger(array) {
  console.log(`[${format(new Date(), "HH:mm:ss")}]\t${array.join(" ")}`);
}

function objFilter(obj, attributes) {
  const filteredObject = {};

  for (const attribute of attributes) {
    if (obj.hasOwnProperty(attribute)) {
      filteredObject[attribute] = obj[attribute];
    }
  }

  return filteredObject;
}

function createRandomString(pattern) {
  const characters =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
  let randomString = "";

  for (let y = 0; y < pattern.length; y++) {
    for (let i = 0; i < Number(pattern[y]); i++) {
      const randomIndex = Math.floor(Math.random() * characters.length);
      randomString += characters.charAt(randomIndex);
    }
    if (y < pattern.length - 1) {
      randomString += "-";
    }
  }
  return randomString;
}

async function generateUserTag(db) {
  let resultTag;
  do {
    resultTag = createRandomString([4, 3]);
  } while (
    (await db.query("SELECT * FROM users WHERE system_id = $1", [resultTag]))
      .rows.length > 0
  );
  console.log("[INFO]\t tag for user created");
  return resultTag;
}

async function isEmailUnique(db, email) {
  return (
    (await db.query("SELECT * FROM users WHERE email = $1", [email])).rows
      .length === 0
  );
}

module.exports = {
  requireProps,
  generateUserTag,
  createRandomString,
  isEmailUnique,
  objFilter,
  consoleLogger,
  encryptData,
  decryptData,
  symmetricEncryptData,
};
