# BE enpoints DOCS

## auth

### Register

`/api/register` : POST

> **Request**  
> Body:
> ```json
> {
>   "email": "...",
>   "firstname": "...",
>   "lastname": "...",
>   "password": "..."
> }
> ```

> **Response**  
> *Status*; *Message*; (*Note*):
> 
> - 400; {"error": "Required field is missing!"}
> - 400; {"error": "Not valid email!"}
> - 400; {"error": "Password does not meet criteria!"}
> - 400; {"error": "Common password!"}
> - 400; {"error": "Password exposed!"}
> - 400; {"error": "Email already used!"}
> - 204; (Successful creation)

### Login

`/api/login` : POST

> **Request**  
> Body:
> ```json
> {
>   "email": "...",
>   "password": "..."
> }
> ```

> **Response**  
> *Status*; *Message*; (*Note*):
> 
> - 400; {"error": "Required field is missing!"}
> - 401; (Not successful login)
> - 200; (Successful login)

## Flights

### Get flights list

`/api/availableFlights` : GET

> **Response**  
> *Status*; *Message*; (*Note*):
> 
> - 403; (Login required)
> - 200; (List of available flights - today+)
> ```json
> { 
>   [
>     {
>       "id": 21,
>       "price": 9490, // in euro cents
>       "departure": "2023-11-10T06:15:00.000Z",
>       "duration_min": 120, // flight in minutes
>       "arrival": "2023-11-10T08:15:00.000Z", // departure + duration_min
>       "dept_city_name": "Vienna",
>       "dept_city_code": "VIE",
>       "arr_city_name": "Barcelona",
>       "arr_city_code": "BCN",
>       "code": "OS391", // flight code
>     }
>   ]
> }
> ```

### Make a reservation

`/api/makeReservation` : POST

> **Request**  
> Body
> ```json
> {
>   "flight_id": 21 
> }
> ```

### Get user tickets

`/api/myTickets` : GET

> **Response**
> ```json
> {
>   "class": "economy",
>   "code": "OS391", // flight code
>   "seat": "21E",
>   "arr_city_code": "BCN",
>   "dept_city_code": "VIE",
>   "departure": "2023-11-10T08:15:00.000Z"
> }
> ```
> - 200; 
> - 500; (Server error)

### Ticket file

`/ticket/:ticket_code` : GET

> **Response**
> ```json
> {
>   "data": "...", //ecrypted data
>   "tag": "..." //symm encription tag (mandatory for the process)
> }
> ```
> - 200; 
> - 401; (Request not from owner)
> - 404; (Not found)

## User data

### Update personal info

`/api/updateUser` : PUT

> **Request**  
> Body  
> ```json
> {
>   "firstname": "Johny Update",
>   "lastname": "Doe"
> }
> ```

> **Response**  
> *Status*; *Message*; (*Note*):
> - 200; (Updated)
> - 400; (Bad request)
> - 401; (Request not from owner)
> - 403; (Login required)

## Search

### Search available flights

For buying ticket (not limited by JWT)

`/api/searchOffer` : POST

> **Request**  
> Body  
> ```json
> {
>   "phrase": "Bratislava"
> }
> ```
> searchable parameters:
> - departure city
> - arrival city
> - date ('DD.MM.YYYY' format)
> - flight code

> **Response**
> - 200; (List of available flights - today+)
> - 400; {error: "Required field is missing!"}
> - 400; {error: "Too short search phrase!"} ( <= 3 characters )
> - 400; {error: "Too many search parameters!"} ( 1 word allowed )
> - 429; Too many search requests. ( 15 requests / 90 sec allowed )
> 
> Body  
> ```json
> { 
>   "flights": [
>     {
>       "id": 21,
>       "price": 9490, // in euro cents
>       "departure": "2023-11-10T06:15:00.000Z",
>       "duration_min": 120, // flight in minutes
>       "arrival": "2023-11-10T08:15:00.000Z", // departure + duration_min
>       "dept_city_name": "Vienna",
>       "dept_city_code": "VIE",
>       "arr_city_name": "Barcelona",
>       "arr_city_code": "BCN",
>       "code": "OS391", // flight code
>     }
>   ]
> }
> ```

### Search available connections

For buying ticket (not limited by JWT)

`/api/getConnections` : GET

> **Response**
> - 200; (List of all available connections)
> - 500; {Server error}
> 
> Body  
> ```json
> { 
>   [
>     {
>       "id": 21,
>       "dept_city_name": "Vienna",
>       "arr_city_name": "Barcelona",
>     }
>   ]
> }
> ```

### Search available flights

For buying ticket (not limited by JWT)

`/api/searchFlights` : POST

> **Request**  
> Body
> ```json
> {
>   "connectionId": 21,
>   "dateFrom": "2023-11-10T06:15:00.000Z",
>   "dateTo": "2023-11-10T08:15:00.000Z",
> }
> ```

> **Response**  
> - 200; (List of all available flights by searching parameters)
> - 500; (Server error)
>
> Body  
> ```json
> {
>   [
>     {
>       "id": 21,
>       "price": 9490, // in euro cents
>       "departure": "2023-11-10T06:15:00.000Z",
>       "duration_min": 120, // flight in minutes
>       "arrival": "2023-11-10T08:15:00.000Z", // departure + duration_min
>       "dept_city_name": "Vienna",
>       "dept_city_code": "VIE",
>       "arr_city_name": "Barcelona",
>       "arr_city_code": "BCN",
>       "code": "OS391", // flight code
>     }
>   ]
> }
> ```
