import { useState } from 'react';
import { Calendar } from './calendar';
import { Popover, PopoverContent, PopoverTrigger } from './popover';
import { Button } from './button';
import { Calendar as CalendarIcon } from 'lucide-react';
import { cn } from '@/lib/utils';
import { FieldError } from 'react-hook-form';

type ControlledDatePickerProps = {
  disablePast?: boolean;
  defaultValue?: Date;
  value?: Date;
  onValueChange?: (value: Date) => void;
  disabled?: boolean;
  error?: FieldError | null;
};

export function DatePicker({
  defaultValue,
  value,
  onValueChange,
  disablePast = false,
  error = null,
  ...props
}: ControlledDatePickerProps) {
  const [open, setOpen] = useState(false);

  // @ts-ignore
  const [date, setDate] = useState<Date>(defaultValue || null);

  if (defaultValue || !value) value = date;

  function onSelect(value: Date) {
    setDate(value);
    onValueChange?.(value);
  }

  function onOpenChange(open: boolean) {
    if (!props.disabled) setOpen(open);
  }

  function isPastDate(day: Date) {
    const today = new Date();
    return day < today;
  }

  return (
    <Popover open={open} onOpenChange={onOpenChange}>
      <PopoverTrigger asChild>
        <Button
          variant={'outline'}
          className={cn(
            error
              ? `w-full justify-start text-left font-normal border-red-600 dark:border-red-600 ${
                  !value && 'text-muted-foreground'
                }`
              : `w-full justify-start text-left font-normal ${
                  !value && 'text-muted-foreground'
                }`,
          )}
        >
          <CalendarIcon className="mr-2 h-4 w-4" />
          {value
            ? new Date(value).toLocaleDateString('sk-SK', {
                dateStyle: 'short',
              })
            : null}
        </Button>
      </PopoverTrigger>
      <PopoverContent className={'w-auto p-0 flex divide-x divide-muted'}>
        <Calendar
          disabled={disablePast && isPastDate}
          mode="single"
          selected={value}
          // @ts-ignore
          onSelect={onSelect}
          initialFocus
          className="h-fit"
          {...props}
        />
      </PopoverContent>
    </Popover>
  );
}
