import { useQuery, UseQueryOptions } from '@tanstack/react-query';
import apiService from '@/lib/apiService';
import { AxiosError } from 'axios';
import { decryptData } from '@/lib/utils';

type useGetQueryProps<Response> = {
  url: string;
  decrypt: boolean;
} & UseQueryOptions<Response, AxiosError>;

const useGetQuery = <Response>({
  url,
  decrypt,
  ...props
}: useGetQueryProps<Response>) => {
  return useQuery({
    queryFn: async () => {
      const { data } = await apiService.get(url);
      if (decrypt) {
        const decryptedData = decryptData(data.secret);
        return {
          ...decryptedData,
          ...data.data,
        };
      }
      return data;
    },
    gcTime: 0,
    retry: false,
    throwOnError: (error: AxiosError) => error.response?.status === 403,
    ...props,
  });
};

export default useGetQuery;
