import { useMutation, UseMutationOptions } from '@tanstack/react-query';
import apiService from '@/lib/apiService';
import { AxiosError } from 'axios';
import { decryptData, encryptData } from '@/lib/utils';
import { ErrorResponseBody, RequestFormat } from './schema';

type usePutQueryProps<Secret, Data, Response = any> = {
  url: string;
  encrypt: boolean;
  decrypt: boolean;
} & UseMutationOptions<
  Response,
  AxiosError<ErrorResponseBody>,
  RequestFormat<Secret, Data>
>;

const usePutQuery = <Secret, Data, Response = any>({
  url,
  encrypt,
  decrypt,
  ...props
}: usePutQueryProps<Secret, Data, Response>) => {
  return useMutation({
    mutationFn: async (bodyData: RequestFormat<Secret, Data>) => {
      let response: any;
      if (encrypt) {
        const encryptedData = encryptData(bodyData.secret);
        const { data } = await apiService.put(url, {
          secret: encryptedData,
          data: bodyData.data,
        });
        response = data;
      } else {
        const { data } = await apiService.put(url, bodyData.data);
        response = data;
      }
      if (decrypt) {
        const decryptedData = decryptData(response.secret);
        return {
          ...decryptedData,
          ...response.data,
        };
      }
      return response;
    },
    retry: false,
    throwOnError: (error: AxiosError) => error.response?.status === 403,
    ...props,
  });
};

export default usePutQuery;
