import { Outlet, useNavigate } from 'react-router-dom';
import MenuBar from '@/components/MenuBar';
import { useAuthContext } from '@/providers/auth';
import { useEffect } from 'react';
import { ErrorBoundary } from 'react-error-boundary';
import { FallbackProps } from 'react-error-boundary';
import CloseModal from '@/components/CloseModal';
import apiService from '@/lib/apiService';
import { decryptData } from '@/lib/utils';

const ErrorFallback = ({ error, resetErrorBoundary }: FallbackProps) => {
  if (error.response.status === 403) {
    return (
      <CloseModal
        title="Koniec relácie!"
        description="Vaša relácia prihlásenia vypršala. Je nutné sa znova prihlásiť."
        onClose={resetErrorBoundary}
      />
    );
  } else if (error.response.status === 500) {
    return (
      <CloseModal
        title="Chyba na strane servera!"
        description="Za vzniknuté problémy sa ospravedlňujeme."
        onClose={resetErrorBoundary}
      />
    );
  }
  return <></>;
};

const UserLayout = () => {
  const { token, setToken } = useAuthContext();
  const { symmKey, setSymmKey } = useAuthContext();
  const { setVector } = useAuthContext();
  const navigate = useNavigate();

  useEffect(() => {
    if (!token) {
      navigate('/login');
    }
  }, [token]);

  useEffect(() => {
    if (!symmKey) {
      apiService.get('/api/symmetricKey').then(response => {
        const symmKeyData = decryptData(response.data);
        setSymmKey(symmKeyData.symmKey);
        setVector(symmKeyData.vector)
      })
    }
  }, [symmKey])

  return (
    <div className="min-h-screen relative pt-[44px] lg:pl-[210px]">
      <MenuBar />
      <ErrorBoundary
        FallbackComponent={ErrorFallback}
        onReset={() => setToken(null)}
      >
        <main className="flex flex-col items-center p-4 w-full min-h-[calc(100vh-44px)]">
          <Outlet />
        </main>
      </ErrorBoundary>
    </div>
  );
};

export default UserLayout;
