import Axios from 'axios';

const apiClient = Axios.create({
  baseURL:
    import.meta.env?.MODE === 'production'
      ? ''
      : import.meta.env.VITE_BASE_URL,
  headers: {
    Accept: 'application/json',
    'Content-Type': 'application/json',
  },
});

function getAuthHeader() {
  return {
    headers: { Authorization: 'Bearer ' + sessionStorage.getItem('token') },
  };
}

export default {
  async post<T>(url: string, body: T) {
    const credentials = getAuthHeader();
    return apiClient.post(url, body, credentials);
  },
  async get(url: string) {
    return apiClient.get(url, {
      headers: {
        Authorization: 'Bearer ' + sessionStorage.getItem('token'),
        'Cache-Control': 'no-cache',
      },
    });
  },
  async put<T>(url: string, body: T) {
    const credentials = getAuthHeader();
    return apiClient.put(url, body, credentials);
  },
  async delete(url: string) {
    const credentials = getAuthHeader();
    return apiClient.delete(url, credentials);
  },
};
