import * as z from 'zod';

export type TAvailableFlights = {
  id: number;
  price: number;
  departure: string;
  duration_min: number;
  arrival: string;
  dept_city_name: string;
  dept_city_code: string;
  arr_city_name: string;
  arr_city_code: string;
  code: string;
};

export type TConnections = {
  id: number;
  dept_city_name: string;
  arr_city_name: string;
};

export const FlightSearchSchema = z
  .object({
    connectionId: z.string().min(1, 'Povinný atribút').default(''),
    dateFrom: z.date().default(new Date()),
    dateTo: z.date().default(new Date()),
  })
  .refine(data => data.dateFrom <= data.dateTo, {
    message: 'Dátum od musí byť menší alebo rovnaký ako dátum do',
    path: ['dateFrom'],
  })
  .refine(data => data.dateFrom <= data.dateTo, {
    message: 'Dátum do musí byť väčší alebo rovnaký ako dátum od',
    path: ['dateTo'],
  });

export type TFlightSearch = z.infer<typeof FlightSearchSchema>;

export type TBuyTicket = {
  flight_id: string
};
