import { zodResolver } from '@hookform/resolvers/zod';
import { useForm } from 'react-hook-form';
import { Button } from '@/components/ui/button';
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form';
import { Input } from '@/components/ui/input';
import { EncryptionTesterFormSchema, TEncryptionTesterForm } from './schema';
import { Loader2 } from 'lucide-react';
import PageHeader from '@/components/PageHeader';
import usePostQuery from '@/hooks/usePostQuery';
import CloseModal from '@/components/CloseModal';
import { useState } from 'react';

const EncryptionTesterPage = () => {
  const [open, setOpen] = useState(false);
  const [message, setMessage] = useState('');
  const form = useForm<TEncryptionTesterForm>({
    resolver: zodResolver(EncryptionTesterFormSchema),
    defaultValues: {
      message: '',
    },
  });
  const {
    formState: { errors },
  } = form;

  const { mutateAsync, isPending } = usePostQuery<
    TEncryptionTesterForm,
    any,
    TEncryptionTesterForm
  >({
    url: '/api/encryptionTest',
    encrypt: true,
    decrypt: true,
    onSuccess: data => {
      setMessage(data.message);
      setOpen(true);
    },
  });

  const onSubmit = async (data: TEncryptionTesterForm) => {
    await mutateAsync({
      secret: {
        ...data,
      },
      data: {},
    });
  };

  return (
    <>
      <PageHeader title="Tester šifrovania" />
      <CloseModal
        title="Šifrovanie prebehlo úspešne"
        description={'Ozvena: ' + message}
        onClose={() => setOpen(false)}
        open={open}
      />
      <Form {...form}>
        <form
          onSubmit={form.handleSubmit(onSubmit)}
          className="flex flex-col p-6 space-y-4 border w-[min(500px,100%)] rounded-lg"
        >
          <h1 className="self-center text-lg font-bold">
            Zadaj správu na otestovanie obojsmerného asymetrického šifrovania
          </h1>
          <FormField
            control={form.control}
            name="message"
            render={({ field, fieldState: { error } }) => (
              <FormItem>
                <FormLabel>Tajná správa</FormLabel>
                <FormControl>
                  <Input placeholder="Tajná správa" {...field} error={error} />
                </FormControl>
                <FormMessage />
              </FormItem>
            )}
          />
          <p className="text-sm font-medium text-red-600 dark:text-red-600">
            {errors.root && errors.root.message}
          </p>
          <div className="flex items-center w-full justify-center">
            <Button
              disabled={isPending}
              className="w-[200px]"
              variant="default"
              type="submit"
            >
              <Loader2
                className={`w-4 h-4 mr-2 animate-spin ${
                  !isPending && 'hidden'
                }`}
              />
              Odoslať
            </Button>
          </div>
        </form>
      </Form>
    </>
  );
};

export default EncryptionTesterPage;
