import PageHeader from '@/components/PageHeader';
import { TNewPubKey } from './schema';
import { Button } from '@/components/ui/button';
import { Download, KeyRound, Loader2 } from 'lucide-react';
import { useRef, useState } from 'react';
import CloseModal from '@/components/CloseModal';
import { generateRSAKeys } from '@/lib/utils';
import usePutQuery from '@/hooks/usePutQuery';
import { useQueryClient } from '@tanstack/react-query';

const KeysPage = () => {
  const [open, setOpen] = useState(false);
  const queryClient = useQueryClient();
  const privateKeyRef = useRef('');
  const enableKeyGenRef = useRef(true);
  
  const { mutateAsync, isPending } = usePutQuery<any, TNewPubKey>({
    url: '/api/updateUserPubKey',
    encrypt: false,
    decrypt: false,
    onError: () => {
      enableKeyGenRef.current = true;
    },
    onSuccess: () => {
      queryClient.clear();
      localStorage.setItem('client_private_key', privateKeyRef.current);
      setOpen(true);
      enableKeyGenRef.current = true;
    },
  });

  const onGenerateKey = async () => {
    if (enableKeyGenRef.current) {
      enableKeyGenRef.current = false;
      const { pubKeyPem, privKeyPem } = generateRSAKeys();
      privateKeyRef.current = privKeyPem;
      await mutateAsync({
        secret: {},
        data: {
          publicKey: pubKeyPem,
        },
      });
    }
  };

  const handleDownload = () => {
    const privateKeyCurrent = privateKeyRef.current === '' ? localStorage.getItem('client_private_key') : privateKeyRef.current;
    const privateKey = privateKeyCurrent !== null ? privateKeyCurrent : '';
    if (privateKey === '') {
      return
    }
    const blob = new Blob([privateKey], {type: 'text/plain'});
    const link = document.createElement('a');
    link.download = 'private_key.pem';
    link.href = window.URL.createObjectURL(blob);
    link.click();
    if (document.body.contains(link)) {
      document.body.removeChild(link);
    }
  }

  return (
    <>
      <PageHeader title="Generovať klúče" />
      <CloseModal
        title="Kľúče boli úspešne aktualizované!"
        onClose={() => setOpen(false)}
        open={open}
      >
      </CloseModal>
      <div className="flex items-center space-x-2">
        <p className="text-lg">Vygenerovať šifrovacie kľúče</p>
        <Button
          className="w-[80px]"
          variant="outline"
          disabled={isPending}
          onClick={onGenerateKey}
        >
          <Loader2
            className={`w-4 h-4 mr-2 animate-spin ${!isPending && 'hidden'}`}
          />
          <KeyRound className="w-6 h-6" />
        </Button>
      </div>
      <div className="grid items-center space-x-2 mt-8">
        <div className="font-light">Súkromný kľúč si uchovajte pre prípad použitia aplikácie na inom zariadení (alebo po lokálnom premazaní pamäte).</div>
        <div className="mt-2 flex justify-center mt-4 mb-4">
            <Button onClick={handleDownload}><Download className="mr-3 h-4 w-4" />Stiahnuť súkromný kľuč</Button>
        </div>
      </div>
    </>
  );
};

export default KeysPage;
