import * as z from 'zod';

export const RegistrationFormSchema = z
  .object({
    email: z
      .string()
      .min(1, 'Povinný atribút')
      .max(50, 'Maximálna dĺžka atribútu je 50 znakov')
      .email('Neplatný tvar emailovej adresy')
      .default(''),
    firstname: z
      .string()
      .min(1, 'Povinný atribút')
      .max(50, 'Maximálna dĺžka atribútu je 50 znakov')
      .default(''),
    lastname: z
      .string()
      .min(1, 'Povinný atribút')
      .max(50, 'Maximálna dĺžka atribútu je 50 znakov')
      .default(''),
    password: z
      .string()
      .min(8, 'Heslo musí obsahovať najmenej 8 znakov')
      .regex(
        /^.*(?=.{8,})(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[!#@-_$%&?^()|{}:<>*"]).*$/,
        'Heslo musí obsahovať číslicu, špeciálny znak a veľké a malé písmeno',
      )
      .max(100, 'Maximálna dĺžka atribútu je 100 znakov')
      .default(''),
    confirmPassword: z.string().min(1, 'Povinný atribút').default(''),
  })
  .refine(data => data.password === data.confirmPassword, {
    message: 'Heslá sa nezhodujú!',
    path: ['confirmPassword'],
  });

export type TRegistrationForm = z.infer<typeof RegistrationFormSchema>;

export type TPubKeyRequest = {
  publicKey: string;
};

export type TRegistrationFormSecret = {
  email: string,
  password: string
}

export type TRegistrationFormData = {
  email: string,
  password: string,
  firstname: string,
  lastname: string
}
