import React, { createContext, useState, useContext } from 'react';

type AuthProviderProps = {
  children: React.ReactNode;
};

type AuthProviderState = {
  token: string | null;
  setToken: (token: string | null) => void;
  symmKey: string | null;
  setSymmKey: (symmKey: string | null) => void;
  vector: string | null;
  setVector: (vector: string | null) => void;
  isLogin: boolean;
};

const initialState = {
  token: null,
  setToken: () => null,
  symmKey: null,
  setSymmKey: () => null,
  vector: null,
  setVector: () => null,
  isLogin: false,
};

const AuthProviderContext = createContext<AuthProviderState>(initialState);

const AuthProvider = ({ children, ...props }: AuthProviderProps) => {
  const [token, _setToken] = useState<string | null>(
    sessionStorage.getItem('token'),
  );
  const [symmKey, _setSymmKey] = useState<string | null>(
    sessionStorage.getItem('symmKey'),
  );
  const [vector, _setVector] = useState<string | null>(
    sessionStorage.getItem('vector'),
  );
  const [isLogin, setIsLogin] = useState(
    sessionStorage.getItem('token') ? true : false,
  );

  const value = {
    token,
    setToken: (token: string | null) => {
      _setToken(token);
      if (token) {
        sessionStorage.setItem('token', token);
        setIsLogin(true);
      } else {
        localStorage.removeItem('client_private_key');
        sessionStorage.removeItem('token');
        setIsLogin(false);
      }
    },
    symmKey,
    setSymmKey: (symmKey: string | null) => {
      _setSymmKey(symmKey);
      if (symmKey) {
        sessionStorage.setItem('symmKey', symmKey)
      } else {
        sessionStorage.removeItem('symmKey');
      }
    },
    vector,
    setVector: (vector: string | null) => {
      _setVector(vector);
      if (vector) {
        sessionStorage.setItem('vector', vector)
      } else {
        sessionStorage.removeItem('vector');
      }
    },
    isLogin,
  };

  return (
    <AuthProviderContext.Provider {...props} value={value}>
      {children}
    </AuthProviderContext.Provider>
  );
};

export const useAuthContext = () => {
  const context = useContext(AuthProviderContext);

  if (context === undefined)
    throw new Error('useAuthContext must be used within a AuthProvider');

  return context;
};

export default AuthProvider;
