## Technologies

- **React** - A JavaScript library for building user interfaces [docs](https://react.dev/learn).
- **Vite** - A fast, opinionated frontend build tool.
- **TypeScript** - A typed superset of JavaScript that compiles to plain JavaScript.
- **Tailwind CSS** - CSS framework [docs](https://tailwindcss.com/).
- **shadcn/ui** - UI components library [docs](https://ui.shadcn.com/docs).
- **lucide icons** - Icons library [docs](https://lucide.dev/).
- **React router dom** - Routing library for React [docs](https://reactrouter.com/en/main).
- **TanStack Query** - Library for fetching, caching, synchronizing and updating server state [docs](https://tanstack.com/query/v5/docs/react/reference/useQuery).- **Zod** - Schema validator [docs](https://zod.dev).
- **React hook form** - Form state manager [docs](https://react-hook-form.com/ts).

## Local dev run

Create a `.env` file from `.env.example`, set `VITE_BASE_URL` to backend address, port.

Inside frontend/app/ directory:
1. installation
```
npm i
```
2. run
```
npm run dev
```

## 📂 Project Structure
```python
app/
  ├── node_modules/      # Project dependencies
  ├── public/            # Public assets
  ├── src/               # Application source code
  │   ├── components/    # React components
  │   │   └── ui/        # shadc/ui components
  │   ├── layouts/       # App layouts used in router.tsx
  │   ├── lib/           # Utility functions
  │   ├── pages/         # All app subpages
  │   ├── providers/     # Providers for global states sharing
  │   ├── App.tsx        # Application entry point
  │   ├── index.css      # Main CSS stylesheet
  │   ├── main.tsx       # Main rendering file
  │   └── router.tsx     # Main routing file
  ├── .eslintrc.json     # ESLint configuration
  ├── .prettierignore    # Ignore file for prettier
  ├── prettierrc.json    # Prettier config file
  ├── index.html         # HTML entry point
  ├── postcss.config.js  # PostCSS configuration
  ├── tailwind.config.js # Tailwind CSS configuration
  ├── tsconfig.json      # TypeScript configuration
  └── vite.config.ts     # Vite configuration
```
