import {
  BrowserRouter as Router,
  Routes,
  Route,
  useNavigate,
} from "react-router-dom";
import { Feedback } from "./components/Feedback";
import { Invoices } from "./components/Invoices";
import { Login } from "./components/Login";
import { Orders } from "./components/Orders";
import { Profil } from "./components/Profil";
import { Registration } from "./components/Registration";
import { NavBar } from "./components/NavBar";
import "./App.css";
import { Alert, Snackbar } from "@mui/material";
import { createContext } from "react";

import React, { useState, useEffect } from "react";

export const BACKEND_URL = "http://localhost:8080";

export const AlertContext = createContext("");

// TODO: z localstorage abo z kade checkuj ci je prihlaseny podla toho zobrazuje icons v navbare
// export const isAuthenticated = true;

const NotFoundRoute = ({ user }) => {
  const navigate = useNavigate();
  useEffect(() => {
    console.log("no?");
    if (user) {
      navigate("/objednavky");
    } else {
      navigate("/prihlasenie");
    }
  }, [user]);
  return null;
};
function App() {
  const [user, setUser] = useState(null);

  const [alert, setAlert] = useState({
    state: false,
    message: "Alert error message",
    severity: "error",
  });

  useEffect(() => {
    if (!localStorage.getItem("user")) {
      setUser(null);
    } else {
      const storedUser = JSON.parse(localStorage.getItem("user"));

      const getUser = async (id) => {
        const data = await fetch(`${BACKEND_URL}/api/users/${id}`, {
          method: "GET",
          headers: { "Content-Type": "application/json" },
        });
        const fetchedUser = await data.json();

        if (fetchedUser.error === "Používateľ neexistuje.") {
          setUser(null);
          localStorage.removeItem("user");
        } else {
          setUser(fetchedUser);
          localStorage.setItem("user", JSON.stringify(fetchedUser));
        }
      };

      getUser(storedUser.id);
    }
  }, []);

  const handleSetUser = (user) => {
    setUser(user);
  };

  // TODO taketo typy alertov mozete pouzivat
  /*
  {
    state: true,
    message: "Alert success message",
    severity: "success",
  }
  */
  /*
  {
    state: false,
    message: "Alert warning message",
    severity: "warning",
  }
  */
  /*
  {
    state: false,
    message: "Alert info message",
    severity: "info",
  }
  */

  return (
    <AlertContext.Provider value={{ alert, setAlert }}>
      <div className="App">
        <Snackbar
          anchorOrigin={{
            vertical: "top",
            horizontal: "center",
          }}
          open={alert.state}
          autoHideDuration={6000}
          onClose={(e, reason) => {
            if (reason === "clickaway") {
              return;
            }
            setAlert({ ...alert, state: false });
          }}
        >
          <Alert
            onClose={() => {
              setAlert({ ...alert, state: false });
            }}
            severity={alert.severity}
            sx={{ width: "100%" }}
          >
            {alert.message}
          </Alert>
        </Snackbar>

        <Router>
          <div>
            <header className="App-header">
              <NavBar
                user={user}
                isAuthenticated={user != null}
                setUser={(user) => handleSetUser(user)}
              />
            </header>
            <Routes>
              {user != null ? (
                <>
                  <Route
                    exact
                    path="/profil"
                    element={<Profil userId={user.id} />}
                  />
                  <Route exact path="/objednavky" element={<Orders />} />
                  <Route
                    exact
                    path="/faktury"
                    element={<Invoices userId={user.id} />}
                  />
                  <Route exact path="/recenzie" element={<Feedback />} />
                  <Route path="/" element={<NotFoundRoute user={true} />} />
                </>
              ) : (
                <>
                  <Route
                    exact
                    path="/registracia"
                    element={
                      <Registration
                        user={user}
                        setUser={(user) => handleSetUser(user)}
                      />
                    }
                  />
                  <Route
                    exact
                    path="/prihlasenie"
                    element={
                      <Login
                        user={user}
                        setUser={(user) => handleSetUser(user)}
                      />
                    }
                  />
                  <Route path="/" element={<NotFoundRoute user={false} />} />
                </>
              )}
            </Routes>
          </div>
        </Router>
      </div>
    </AlertContext.Provider>
  );
}

export default App;
