import "./App.css";
import React, { useEffect, useState } from "react";
import { Register } from "./components/Register";
import { Login } from "./components/Login";
import { CreateInvoice } from "./components/CreateInvoice";

export const BACKEND_URL = "http://localhost:8080";

function App() {
  const [menu, setMenu] = useState("register");
  const [user, setUser] = useState(null);

  useEffect(() => {
    if (!localStorage.getItem("user")) {
      setUser(null);
    } else {
      const storedUser = JSON.parse(localStorage.getItem("user"));

      const getUser = async (id) => {
        const data = await fetch(`${BACKEND_URL}/api/users/${id}`, {
          method: "GET",
          headers: { "Content-Type": "application/json" },
        });
        const fetchedUser = await data.json();

        if (fetchedUser.error === "User not found") {
          setUser(null);
          localStorage.removeItem("user");
        } else {
          setUser(fetchedUser);
          localStorage.setItem("user", JSON.stringify(fetchedUser));
        }
      };

      getUser(storedUser.id);
    }
  }, []);

  const handleSetUser = (user) => {
    setUser(user);
  };

  const handleMenu = (selection) => {
    setMenu(selection);
  };

  const renderComponent = (menu) => {
    if (menu === "login")
      return <Login user={user} setUser={(user) => handleSetUser(user)} />;
    if (menu === "register")
      return user == null ? (
        <Register user={user} setUser={(user) => handleSetUser(user)} />
      ) : (
        <></>
      );
    if (menu === "invoice")
      return user != null ? <CreateInvoice userId={user.id} /> : <></>;
  };

  return (
    <>
      {user && user.email && (
        <div>
          Logged in as: <b>{user.email}</b>
        </div>
      )}
      <ul>
        <li onClick={() => handleMenu("login")} style={{ cursor: "pointer" }}>
          Login page
        </li>
        {user == null && (
          <li
            onClick={() => handleMenu("register")}
            style={{ cursor: "pointer" }}
          >
            Register page
          </li>
        )}
        {user && (
          <li
            onClick={() => handleMenu("invoice")}
            style={{ cursor: "pointer" }}
          >
            Invoice page
          </li>
        )}
      </ul>

      {renderComponent(menu)}
    </>
  );
}

export default App;
