import React, { useState, useContext, useEffect } from "react";

import {
  Typography,
  Button,
  OutlinedInput,
  Modal,
  Box,
  Divider,
  Container,
} from "@mui/material";

import PictureAsPdfIcon from "@mui/icons-material/PictureAsPdf";
import "./css/general.css";
import "./css/Invoices.css";
import { AlertContext, BACKEND_URL } from "../App";

// TODO: ziskat taketo data z databazy a tie zobrazovat, do zoznamu treba aj nejako reprezentovat ten konkretny subor na stiahnutie

const style = {
  position: "absolute",
  top: "50%",
  left: "50%",
  transform: "translate(-50%, -50%)",
  width: 400,
  bgcolor: "background.paper",
  borderRadius: 5,
  boxShadow: 24,
  p: 4,
};

export const Invoices = ({ userId }) => {
  const { setAlert } = useContext(AlertContext);

  const [chosenInvoice, setChosenInvoice] = useState({ id: null, open: false, name: null });
  const [privateKey, setPrivateKey] = useState("");


  const [invoices, setInvoices] = useState([])


  useEffect(() => {

    const getInvoices = async () => {
      const data = await fetch(`${BACKEND_URL}/api/invoices/${userId}`, {
        method: "GET",
        headers: { "Content-Type": "application/json" },
      });
      const fetchedInvoices = await data.json();

      setInvoices(fetchedInvoices)

    }

    getInvoices()

  }, [userId])


  const handleFileChange = async (file) => {
    try {
      const fileReader = new FileReader();
      fileReader.onloadend = (e) => {
        const content = fileReader.result;
        setPrivateKey(content)
      }
      fileReader.readAsText(file)
    }
    catch (e) {
      // console.log("No file provided")
      setPrivateKey(null)
    }
  }


  const handleSubmit = async () => {

    if (!privateKey) {
      // setError("No private key provided !")
      setAlert({
        state: true,
        severity: "error",
        message: "Nebol zadaný privátny kľúč."
      });
      return
    }

    const body = {
      // dinerId: 1,
      // foodName: 'Suche rozky',
      // price: 5.50,
      userId,
      invoiceId: chosenInvoice.id
    }

    const data = await fetch(`${BACKEND_URL}/api/invoices`, {
      method: "POST",
      headers: { "Content-Type": "application/json" },
      body: JSON.stringify(body)
    })

    const response = await data.json()


    const inp = document.getElementById('importPrivateKeyInput')
    inp.value = ""

    if (response.error) {
      // setError(response.error)
      setAlert({
        state: true,
        severity: "error",
        message: response.error
      });
    }

    if (response.bytes) {
      const bytes = response.bytes

      var nonceCiphertextTag = base64ToArrayBuffer(bytes);
      nonceCiphertextTag = new Uint8Array(nonceCiphertextTag);

      const keyData = privateKey.match(/-----BEGIN [A-Z ]+-----([\s\S]+)-----END [A-Z ]+-----/);
      const out = keyData ? keyData[1].trim() : null;

      const binaryString = atob(out);
      const newBytes = new Uint8Array(binaryString.length);
      for (let i = 0; i < binaryString.length; i++) {
        newBytes[i] = binaryString.charCodeAt(i);
      }

      let parsedPrivateKey = null
      try {
        parsedPrivateKey = await window.crypto.subtle.importKey(
          'pkcs8',
          newBytes,
          { name: 'RSA-OAEP', hash: 'SHA-256' },
          true,
          ['decrypt']
        );
      }
      catch (e) {
        // setError("Unable to import key.")
        setAlert({
          state: true,
          severity: "error",
          message: "Nepodarilo sa importovať kľúč."
        });
        return
      }

      var decrypted = await decrypt(nonceCiphertextTag, parsedPrivateKey);

      if (decrypted == null) {
        // setError("Unable to decrypt, bad private key provided.")
        setAlert({
          state: true,
          severity: "error",
          message: "Nepodarilo sa dešifrovať, bol poskytnutý nesprávny privátny kľúč."
        });
        return;
      }

      createAndDownloadPDFFromBytes(decrypted)
      setChosenInvoice({ ...chosenInvoice, open: false });
    }

  };




  return (
    <Container maxWidth="lg" className="custom-container">
      <div>
        <Typography variant="h3" gutterBottom>
          Vaše faktúry
        </Typography>
        <Divider />
        <div className="invoices-container">
          <div className="invoices-table-titles-container">
            <Typography variant="h6">Názov</Typography>
            <Typography variant="h6">Dátum</Typography>
            <Typography variant="h6">Stiahnutie</Typography>
          </div>
          <Divider />
          <Divider />
          <div className="invoices-data-container">
            {invoices.map((item) => (
              <div className="row-container" key={item.id}>
                <Typography variant="p">{item.name}</Typography>
                <Typography variant="p">{new Date(item.date).toLocaleDateString()} {" "} {new Date(item.date).toLocaleTimeString()}</Typography>
                <Button
                  onClick={() => {
                    setChosenInvoice({ id: item.id, name: item.name, open: true });
                  }}
                >
                  <PictureAsPdfIcon />
                </Button>
              </div>
            ))}
          </div>
          <Divider />
          <Divider />
        </div>
      </div>
      <Modal
        open={chosenInvoice.open}
        onClose={() => {
          setChosenInvoice({ ...chosenInvoice, open: false });
          setPrivateKey("");
        }}
        aria-labelledby="modal-modal-title"
        aria-describedby="modal-modal-description"
      >
        <Box sx={style}>
          <div className="modal-input-field">
            <Typography variant="p">Nahraj súkromný kľúč:</Typography>
            <div className="file-input-container">
              <OutlinedInput
                id="importPrivateKeyInput"
                accept='.pem'
                size="small"
                className="custom-file-input"
                type="file"
                // onChange={(e) => setInputSukromnyKluc(e.target.value)}
                onChange={e => handleFileChange(e.target.files[0])}
              // value={privateKey}
              />
            </div>
          </div>
          {/* TODO: pridaj action na kontrolu a stiahnutie */}
          <Button
            className="dark-button"
            variant="outlined"
            size="medium"
            onClick={handleSubmit}

          // onClick={() => {

          //   console.log(
          //     "skontroluj a stiahni",
          //     chosenInvoice.name,
          //     privateKey
          //   );
          //   setAlert({
          //     state: true,
          //     severity: "success",
          //     message: "stiahnute",
          //   });
          //   setChosenInvoice({ ...chosenInvoice, open: false });
          //   setPrivateKey("");
          // }}
          >
            Skontroluj a stiahni
          </Button>
        </Box>
      </Modal>
    </Container>
  );
};


function createAndDownloadPDFFromBytes(pdfBytes) {
  const blob = new Blob([pdfBytes], { type: 'application/pdf' });
  const url = URL.createObjectURL(blob);

  const a = document.createElement('a');
  a.href = url;
  a.download = 'decryptedInvoice.pdf';
  a.click();

  URL.revokeObjectURL(url);
}

async function decrypt(nonceCiphertextTag, privateKey) {

  var nonce = nonceCiphertextTag.subarray(0, 16);
  const encryptedKey = nonceCiphertextTag.subarray(16, 272)
  var ciphertextTag = nonceCiphertextTag.subarray(272);
  var aesKeyBuffer = null;

  try {
    aesKeyBuffer = await crypto.subtle.decrypt(
      { name: 'RSA-OAEP' },
      privateKey,
      encryptedKey
    );
  } catch (error) {
    return null;
  }

  let aesKey = null;

  try {
    aesKey = await window.crypto.subtle.importKey('raw', aesKeyBuffer, 'AES-GCM', true, ['encrypt', 'decrypt']);
  }
  catch (e) {
    return null;
  }


  try {
    var decrypted = await crypto.subtle.decrypt({ name: 'AES-GCM', iv: nonce }, aesKey, ciphertextTag);
  } catch (error) {
    return null;
  }

  return decrypted
}

function base64ToArrayBuffer(base64) {
  var binary_string = window.atob(base64);
  var len = binary_string.length;
  var bytes = new Uint8Array(len);
  for (var i = 0; i < len; i++) {
    bytes[i] = binary_string.charCodeAt(i);
  }
  return bytes.buffer;
}