import React, { useState, useContext } from "react";

import { hashPassword } from '../components_old/KeyPairs';

import {
  InputLabel,
  OutlinedInput,
  Typography,
  Button,
  Divider,
  Container,
} from "@mui/material";

import "./css/general.css";
import { AlertContext, BACKEND_URL } from "../App";
import { useNavigate } from "react-router-dom";

export const Login = ({ user, setUser }) => {
  const [email, setEmail] = useState("");
  const [password, setPassword] = useState("");

  const navigate = useNavigate();

  const { setAlert } = useContext(AlertContext);


  // const handleEmail = (e) => {
  //   setEmail(e.target.value);
  // };

  // const handlePassword = (e) => {
  //   setPassword(e.target.value);
  // };

  // const handleLogout = async (e) => {
  //   localStorage.removeItem("user")
  //   setUser(null)
  // }

  const handleSubmit = async () => {
    // e.preventDefault();

    // setErrorContent("")

    if (email === '' || password === '') {
      // setErrorContent("You need to fill in this form before submiting.")
      setAlert({
        state: true,
        severity: "error",
        message: "Zlé prihlasovacie údaje."
      });
      return
    }

    const emailRegex = /^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,}$/i;

    if (!emailRegex.test(email)) {
      // setErrorContent("You need to use right email pattern.")
      setAlert({
        state: true,
        severity: "error",
        message: "Zlé prihlasovacie údaje."
      });
      return
    }

    let hashedPassword = null

    try {
      hashedPassword = await hashPassword(password)
    }
    catch (e) {
      // setErrorContent("Unable to hash password.")
      setAlert({
        state: true,
        severity: "error",
        message: "Zlé prihlasovacie údaje."
      });
      return
    }

    const data = await fetch(`${BACKEND_URL}/api/users/login`, {
      method: "POST",
      headers: { "Content-Type": "application/json" },
      body: JSON.stringify({ email, password: hashedPassword })
    })

    const response = await data.json()

    if (response.user) {
      console.log(response.user)
      localStorage.setItem("user", JSON.stringify(response.user))
      setUser(response.user)

      setAlert({
        state: true,
        severity: "success",
        message: "Úspešné prihlásenie.",
      });


      navigate("/objednavky");

      return
    }

    if (response.error) {
      // setErrorContent(response.error)
      setAlert({
        state: true,
        severity: "error",
        message: response.error
      });
      return
    }

    if (response.status === 'Too many login attempts, cooldown.') {
      // setErrorContent(`${response.status} (5 seconds)`)
      setAlert({
        state: true,
        severity: "error",
        message: `Priveľa pokusov o prihlásenie, skúste to znova o 5 sekúnd.`
      });
      return
    }
  };












  return (
    <Container maxWidth="lg" className="custom-container">
      <div>
        <Typography variant="h3" gutterBottom>
          Prihlásenie
        </Typography>
        <Divider />
        <div className="inputs-container">
          <div className="one-input-field">
            <InputLabel htmlFor="email">E-mail:</InputLabel>
            <OutlinedInput
              id="email"
              size="small"
              className="custom-outlined-outlinedInput"
              onChange={(e) => setEmail(e.target.value)}
              value={email}
              placeholder="E-mail"
            />
          </div>
          <div className="one-input-field">
            <InputLabel htmlFor="heslo">Heslo:</InputLabel>
            <OutlinedInput
              id="heslo"
              size="small"
              type="password"
              className="custom-outlined-outlinedInput"
              onChange={(e) => setPassword(e.target.value)}
              value={password}
              placeholder="Heslo"
            />
          </div>
        </div>
        <Button
          className="dark-button"
          variant="outlined"
          size="medium"
          onClick={handleSubmit}
        >
          Prihlásiť
        </Button>
      </div>
    </Container>
  );
};
