import React, { useContext } from "react";
import { Logout as LogoutIcon } from "@mui/icons-material";
import { Link } from "react-router-dom";
import {
  AppBar,
  Box,
  Toolbar,
  IconButton,
  Typography,
  Menu,
  Container,
  Button,
  Tooltip,
  MenuItem,
} from "@mui/material";
import MenuIcon from "@mui/icons-material/Menu";

// import { isAuthenticated } from "../App";
import "./css/NavBar.css";

import { AlertContext } from "../App";

import { useNavigate } from "react-router-dom";

const pages = [
  { name: "Registrácia", path: "/registracia", auth: false },
  { name: "Prihlásenie", path: "/prihlasenie", auth: false },
  { name: "Profil", path: "/profil", auth: true },
  { name: "Objednávky", path: "/objednavky", auth: true },
  { name: "Faktúry", path: "/faktury", auth: true },
  { name: "Recenzie", path: "/recenzie", auth: true },
];

export const NavBar = ({ isAuthenticated, user, setUser }) => {
  const navigate = useNavigate();

  const [anchorElNav, setAnchorElNav] = React.useState(null);

  const { setAlert } = useContext(AlertContext);

  const handleOpenNavMenu = (event) => {
    setAnchorElNav(event.currentTarget);
  };
  const handleCloseNavMenu = () => {
    setAnchorElNav(null);
  };

  const handleLogout = async () => {
    localStorage.removeItem("user");
    setUser(null);
    setAlert({
      state: true,
      severity: "info",
      message: "Úspešné odhlásenie.",
    });
    navigate("/prihlasenie");
  };

  return (
    <AppBar position="static" className="app-bar" color={undefined}>
      <Container maxWidth="xl">
        <Toolbar disableGutters>
          <Typography
            variant="h6"
            noWrap
            component="a"
            sx={{
              mr: 2,
              display: { xs: "none", md: "flex" },
              fontFamily: "monospace",
              fontWeight: 700,
              letterSpacing: ".3rem",
              color: "inherit",
              textDecoration: "none",
            }}
          >
            Jedáleň
          </Typography>

          <Box sx={{ flexGrow: 1, display: { xs: "flex", md: "none" } }}>
            <IconButton
              size="large"
              aria-label="account of current user"
              aria-controls="menu-appbar"
              aria-haspopup="true"
              onClick={handleOpenNavMenu}
              color="inherit"
            >
              <MenuIcon />
            </IconButton>
            <Menu
              id="menu-appbar"
              anchorEl={anchorElNav}
              anchorOrigin={{
                vertical: "bottom",
                horizontal: "left",
              }}
              keepMounted
              transformOrigin={{
                vertical: "top",
                horizontal: "left",
              }}
              open={Boolean(anchorElNav)}
              onClose={handleCloseNavMenu}
              sx={{
                display: { xs: "block", md: "none" },
              }}
            >
              {pages.map((page) =>
                isAuthenticated === page.auth ? (
                  <Link to={page.path} key={page.name}>
                    <MenuItem onClick={handleCloseNavMenu}>
                      <Typography textAlign="center" color="black">
                        {page.name}
                      </Typography>
                    </MenuItem>
                  </Link>
                ) : null
              )}
            </Menu>
          </Box>

          <Box sx={{ flexGrow: 1, display: { xs: "none", md: "flex" } }}>
            {pages.map((page) =>
              isAuthenticated === page.auth ? (
                <Link key={page.name} to={page.path}>
                  <Button
                    onClick={handleCloseNavMenu}
                    sx={{ my: 2, color: "white", display: "block" }}
                  >
                    {page.name}
                  </Button>
                </Link>
              ) : null
            )}
          </Box>
          {isAuthenticated && (
            <Box sx={{ flexGrow: 0 }}>
              <Typography
                variant="h5"
                noWrap
                component="a"
                sx={{
                  mr: 1,
                  flexGrow: 1,
                  fontFamily: "monospace",
                  fontSize: 20,
                  fontWeight: 400,
                  letterSpacing: ".1rem",
                }}
              >
                {/* TODO: tu dajte realne meno */}
                {`${user.firstName} ${user.lastName}`}
              </Typography>
              <Tooltip title="Odhlásenie">
                <IconButton onClick={handleLogout} sx={{ p: 0 }} color="white">
                  <LogoutIcon />
                </IconButton>
              </Tooltip>
            </Box>
          )}
        </Toolbar>
      </Container>
    </AppBar>
  );
};
