import React, { useState, useContext } from "react";

import { hashPassword } from "../components_old/KeyPairs";

import {
  InputLabel,
  OutlinedInput,
  Typography,
  Button,
  Divider,
  Container,
} from "@mui/material";

import "./css/general.css";
import { AlertContext, BACKEND_URL } from "../App";
import { useNavigate } from "react-router-dom";

export const Registration = ({ user, setUser }) => {
  const [firstName, setFirstName] = useState("");
  const [lastName, setLastName] = useState("");
  const [email, setEmail] = useState("");
  const [password, setPassword] = useState("");

  const [confirmPassword, setConfirmPassword] = useState("");

  const navigate = useNavigate();

  const { setAlert } = useContext(AlertContext);

  // const [submitted, setSubmitted] = useState(false);
  // const [passwordError, setPasswordError] = useState(false);
  // const [alreadyExists, setAlreadyExists] = useState("")
  // const [emailError, setEmailError] = useState(false)

  // const [errorContent, setErrorContent] = useState("")

  // const handleFirstName = (e) => {
  //   setFirstName(e.target.value);
  //   setSubmitted(false);
  // };

  // const handleLastName = (e) => {
  //   setLastName(e.target.value);
  //   setSubmitted(false);
  // };

  // const handleEmail = (e) => {
  //   setEmail(e.target.value);
  //   setSubmitted(false);
  // };

  // const handlePassword = (e) => {
  //   setPassword(e.target.value);
  //   setSubmitted(false);
  // };

  const handleSubmit = async () => {
    // e.preventDefault();

    console.log({ firstName, lastName, email, password });

    // setPasswordError(false)
    // setAlreadyExists(false)
    // setSubmitted(false)
    // setEmailError(false)
    // setErrorContent("")

    if (
      firstName === "" ||
      lastName === "" ||
      email === "" ||
      password === "" ||
      confirmPassword === ""
    ) {
      // setErrorContent("You need to fill in this form.")
      setAlert({
        state: true,
        severity: "error",
        message: "Vyplňte všetky polia!",
      });
      return;
    }

    const nameRegex = /^[a-zA-ZáäčďéíľĺňóôŕšťúýžÁÄČĎÉÍĽĹŇÓÔŔŠŤÚÝŽ -]+$/;

    if (!nameRegex.test(firstName) || !nameRegex.test(lastName)) {
      // setErrorContent("Name contains inappropriate characters.")
      setAlert({
        state: true,
        severity: "error",
        message: "Meno/priezvisko obsahuje nepovolené znaky.",
      });
      return;
    }

    const emailRegex = /^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,}$/i;

    if (!emailRegex.test(email)) {
      // setEmailError(true)
      setAlert({
        state: true,
        severity: "error",
        message: "Nesprávny tvar e-mailu!",
      });
      return;
    }

    const regex =
      /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*?&])[A-Za-z\d@$!%*?&]{8,}$/;

    if (!regex.test(password)) {
      // setPasswordError(true);
      setAlert({
        state: true,
        severity: "error",
        message:
          "Heslo musí obsahovať aspoň jedno malé písmeno, jedno veľké písmeno, číslo, znak a musí byť dlhé aspoň 8 znakov!"
      });
      return;
    }

    if (password !== confirmPassword) {
      // setPasswordError(true);
      setAlert({
        state: true,
        severity: "error",
        message: "Heslá sa nezhodujú.",
      });
      return;
    }

    let hashedPassword = null;

    try {
      hashedPassword = await hashPassword(password);
    } catch (e) {
      // setErrorContent("Unable to hash password.")
      setAlert({
        state: true,
        severity: "error",
        message: "Nepodarilo sa zahashovať heslo.",
      });
      return;
    }

    const data = await fetch(`${BACKEND_URL}/api/users`, {
      method: "POST",
      headers: { "Content-Type": "application/json" },
      body: JSON.stringify({
        firstName,
        lastName,
        email,
        password: hashedPassword,
      }),
    });

    const response = await data.json();

    if (response.error) {
      // setErrorContent(response.error)
      setAlert({
        state: true,
        severity: "error",
        message: response.error,
      });
      return;
    }

    if (response.user) {
      localStorage.setItem("user", JSON.stringify(response.user));
      setUser(response.user);
    }

    setAlert({
      state: true,
      severity: "success",
      message: `Používateľ ${firstName} ${lastName} úspešne zaregistrovaný!`,
    });

    // setSubmitted(true);
    // setPasswordError(false);
    // setEmailError(false)

    setFirstName("");
    setLastName("");
    setEmail("");
    setPassword("");

    navigate("/objednavky");
  };

  // const successMessage = () => {
  //   return !alreadyExists && (
  //     <div
  //       className="success"
  //       style={{
  //         display: submitted ? '' : 'none',
  //         color: 'green'
  //       }}>
  //       User {firstName} {lastName} successfully registered!
  //     </div>
  //   );
  // };

  // const emailErrorMessage = () => {
  //   return (
  //     <div
  //       className="error"
  //       style={{
  //         display: emailError ? '' : 'none',
  //       }}>
  //       Please enter a valid email address.
  //     </div>
  //   );
  // };

  // const passwordErrorMessage = () => {
  //   return <div className="error" style={{ display: passwordError ? '' : 'none' }}>
  //     Password should contain at least one lower case letter,
  //     one upper case letter, number, symbol and should be at least 8 characters long!
  //   </div>
  // }

  return (
    <Container maxWidth="lg" className="custom-container">
      <div>
        <Typography variant="h3" gutterBottom>
          Registrácia
        </Typography>
        <Divider />
        <div className="inputs-container">
          <div className="one-input-field">
            <InputLabel htmlFor="meno">Meno:</InputLabel>
            <OutlinedInput
              id="meno"
              size="small"
              className="custom-outlined-outlinedInput"
              onChange={(e) => setFirstName(e.target.value)}
              value={firstName}
              placeholder="Meno"
            />
          </div>
          <div className="one-input-field">
            <InputLabel htmlFor="priezvisko">Priezvisko:</InputLabel>
            <OutlinedInput
              id="priezvisko"
              size="small"
              className="custom-outlined-outlinedInput"
              onChange={(e) => setLastName(e.target.value)}
              value={lastName}
              placeholder="Priezvisko"
            />
          </div>
          <div className="one-input-field">
            <InputLabel htmlFor="email">E-mail:</InputLabel>
            <OutlinedInput
              id="email"
              size="small"
              className="custom-outlined-outlinedInput"
              onChange={(e) => setEmail(e.target.value)}
              value={email}
              placeholder="E-mail"
            />
          </div>
          <div className="one-input-field">
            <InputLabel htmlFor="heslo">Heslo:</InputLabel>
            <OutlinedInput
              id="heslo"
              size="small"
              type="password"
              className="custom-outlined-outlinedInput"
              onChange={(e) => setPassword(e.target.value)}
              value={password}
              placeholder="Heslo"
            />
          </div>
          <div className="one-input-field">
            <InputLabel htmlFor="potvrdenieHesla">Potvrdenie hesla:</InputLabel>
            <OutlinedInput
              id="potvrdenieHesla"
              size="small"
              type="password"
              className="custom-outlined-outlinedInput"
              onChange={(e) => setConfirmPassword(e.target.value)}
              value={confirmPassword}
              placeholder="Heslo"
            />
          </div>
        </div>
        {/* TODO: pridaj action na registruj */}
        <Button
          className="dark-button"
          variant="outlined"
          size="medium"
          onClick={handleSubmit}
        >
          Registruj
        </Button>
      </div>
    </Container>
  );
};
