import React, { useState } from "react";
import {
  TextField,
  Typography,
  Rating,
  Divider,
  Modal,
  Box,
} from "@mui/material";

const style = {
  position: "absolute",
  top: "50%",
  left: "50%",
  transform: "translate(-50%, -50%)",
  width: 600,
  bgcolor: "background.paper",
  borderRadius: 5,
  boxShadow: 24,
  p: 4,
};
export const SearchableList = ({ data }) => {
  const [searchTerm, setSearchTerm] = useState("");
  const [chosenFeedback, setChosenFeedback] = useState({
    open: false,
    id: 1,
    name: "Jedlo 1",
    date: "1.11.2023",
    autor: "Maťko",
    stars: 2,
    text: "Matkovi to velmi nechutilo nabuduce by si dal daco lepsie",
  });

  const handleSearchChange = (event) => {
    setSearchTerm(event.target.value);
  };

  const filteredData = data.filter((item) => {
    if (
      item.name.toLowerCase().includes(searchTerm.toLowerCase()) ||
      item.date.toLowerCase().includes(searchTerm.toLowerCase()) ||
      item.autor.toLowerCase().includes(searchTerm.toLowerCase())
    ) {
      return true;
    }
    return false;
  });

  return (
    <div>
      <TextField
        variant="outlined"
        fullWidth
        value={searchTerm}
        onChange={handleSearchChange}
        style={{ marginBottom: "16px" }}
        placeholder="Vyhľadávanie"
      />
      <div className="invoices-container">
        <div className="invoices-table-titles-container">
          <Typography variant="h6">Názov jedla</Typography>
          <Typography variant="h6">Dátum</Typography>
          <Typography variant="h6">Autor</Typography>
          <Typography variant="h6">Počet hviezdičiek</Typography>
        </div>
        <Divider />
        <Divider />
        {filteredData.map((item) => (
          <div
            className="recenzia-row-container row-container"
            key={item.id}
            onClick={() => setChosenFeedback({ ...item, open: true })}
          >
            <Typography variant="p">{item.name}</Typography>
            <Typography variant="p">{item.date}</Typography>
            <Typography variant="p">{item.autor}</Typography>
            <Rating
              name="read-only"
              value={item.stars}
              readOnly
              className="custom-rating"
            />
          </div>
        ))}
      </div>

      <Modal
        open={chosenFeedback.open}
        onClose={() => {
          setChosenFeedback({ ...chosenFeedback, open: false });
        }}
      >
        <Box sx={style}>
          <div className="modal-text-fields">
            <div>
              <Typography variant="p" className="text-label">
                Názov jedla:
              </Typography>
              <Typography variant="p">{chosenFeedback.name}</Typography>
            </div>
            <div>
              <Typography variant="p" className="text-label">
                Počet hviezdičiek:
              </Typography>
              <Typography variant="p">
                <Rating
                  name="read-only"
                  value={chosenFeedback.stars}
                  readOnly
                  className="custom-rating"
                />
              </Typography>
            </div>
            <div>
              <Typography variant="p" className="text-label">
                Dátum:
              </Typography>
              <Typography variant="p">{chosenFeedback.date}</Typography>
            </div>
            <div>
              <Typography variant="p" className="text-label">
                Autor:
              </Typography>
              <Typography variant="p">{chosenFeedback.autor}</Typography>
            </div>
            <div>
              <Typography variant="p" className="text-label">
                Text recenzie:
              </Typography>
              <Typography variant="p">{chosenFeedback.text}</Typography>
            </div>
          </div>
        </Box>
      </Modal>
    </div>
  );
};
