export const backendPublicKey =
  `-----BEGIN PUBLIC KEY-----
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAkQmTx95KpyOIZ2FNzGDR
iy9LpiWV0ZufvovyWPcT/xT4r5lLwyi1EULOwUNb1IoXYXCIB6DPtFjD0knF4fP/
Mitq5Ij7DjPXZSiKzlw9/RPiiLn6M91axwzYhf4+BypuiJOoJpv5k0Y1XXBZZ90D
MZjGDxuapqVguypjpG3leRxgXkdcUoiq0+wtdJknlxMBVGPzx7B7PVEP87X9IQlh
URUy1moAS4m03e1p5ySzA/5huTnzoPZaBqDMjM4/OpD+a9zV2oJgY62CjE2gv/v4
YijgTVxenplhdmfPgXJowbrzcD4SROhz9LvU/RYPXKYsb+vvB47MjUR9iiYE9IPN
dwIDAQAB
-----END PUBLIC KEY-----`

export async function keyPairs() {
  const keyPair = await crypto.subtle.generateKey(
    {
      name: "RSA-OAEP",
      modulusLength: 2048,
      publicExponent: new Uint8Array([1, 0, 1]),
      hash: "SHA-256",
    },
    true,
    ["encrypt", "decrypt"]
  );

  return {
    publicKey: await exportPublicKeyToPem(keyPair.publicKey),
    privateKey: await exportPrivateKeyToPem(keyPair.privateKey),
  };
}

async function exportPublicKeyToPem(publicKey) {
  const publicKeyBuffer = await crypto.subtle.exportKey("spki", publicKey);
  const publicKeyArray = new Uint8Array(publicKeyBuffer);
  const publicKeyBase64 = btoa(String.fromCharCode(...publicKeyArray));
  const pem = `-----BEGIN PUBLIC KEY-----\n${publicKeyBase64}\n-----END PUBLIC KEY-----\n`;
  return pem;
}

async function exportPrivateKeyToPem(privateKey) {
  const privateKeyBuffer = await crypto.subtle.exportKey("pkcs8", privateKey);
  const privateKeyArray = new Uint8Array(privateKeyBuffer);
  const privateKeyBase64 = btoa(String.fromCharCode(...privateKeyArray));
  const pem = `-----BEGIN PRIVATE KEY-----\n${privateKeyBase64}\n-----END PRIVATE KEY-----\n`;
  return pem;
}


export async function hashPassword(password) {
  const pub = await importPublicKey(backendPublicKey);
  const encrypted = await encryptRSA(pub, new TextEncoder().encode(password));
  const encryptedBase64 = window.btoa(ab2str(encrypted));

  async function importPublicKey(spkiPem) {
    return await window.crypto.subtle.importKey(
      "spki",
      getSpkiDer(spkiPem),
      {
        name: "RSA-OAEP",
        hash: "SHA-256",
      },
      true,
      ["encrypt"]
    );
  }

  async function encryptRSA(key, plaintext) {
    let encrypted = await window.crypto.subtle.encrypt(
      {
        name: "RSA-OAEP",
      },
      key,
      plaintext
    );
    return encrypted;
  }

  function getSpkiDer(spkiPem) {
    const pemHeader = "-----BEGIN PUBLIC KEY-----";
    const pemFooter = "-----END PUBLIC KEY-----";
    var pemContents = spkiPem.substring(pemHeader.length, spkiPem.length - pemFooter.length);
    var binaryDerString = window.atob(pemContents);
    return str2ab(binaryDerString);
  }

  function str2ab(str) {
    const buf = new ArrayBuffer(str.length);
    const bufView = new Uint8Array(buf);
    for (let i = 0, strLen = str.length; i < strLen; i++) {
      bufView[i] = str.charCodeAt(i);
    }
    return buf;
  }

  function ab2str(buf) {
    return String.fromCharCode.apply(null, new Uint8Array(buf));
  }

  return encryptedBase64
}