import React, { useState } from 'react';
import { hashPassword } from './KeyPairs';
import { ErrorMessage } from './ErrorMessage';
import { BACKEND_URL } from '../App';

export function Login({ user, setUser }) {
    const [email, setEmail] = useState("");
    const [password, setPassword] = useState("");

    const [errorContent, setErrorContent] = useState("")

    const handleEmail = (e) => {
        setEmail(e.target.value);
    };

    const handlePassword = (e) => {
        setPassword(e.target.value);
    };

    const handleLogout = async (e) => {
        localStorage.removeItem("user")
        setUser(null)
    }

    const handleSubmit = async (e) => {
        e.preventDefault();

        setErrorContent("")

        if (email === '' || password === '') {
            setErrorContent("You need to fill in this form before submiting.")
            return
        }

        const emailRegex = /^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,}$/i;

        if (!emailRegex.test(email)) {
            setErrorContent("You need to use right email pattern.")
            return
        }

        let hashedPassword = null

        try {
            hashedPassword = await hashPassword(password)
        }
        catch (e) {
            setErrorContent("Unable to hash password.")
            return
        }

        const data = await fetch(`${BACKEND_URL}/api/users/login`, {
            method: "POST",
            headers: { "Content-Type": "application/json" },
            body: JSON.stringify({ email, password: hashedPassword })
        })

        const response = await data.json()

        if (response.user) {
            localStorage.setItem("user", JSON.stringify(response.user))
            setUser(response.user)
            return
        }

        if (response.error) {
            setErrorContent(response.error)
            return
        }

        if (response.status === 'Too many login attempts, cooldown.') {
            setErrorContent(`${response.status} (5 seconds)`)
            return
        }
    };

    return (
        <div className="form">
            <div>
                <h1>Login</h1>
            </div>

            <ErrorMessage
                condition={errorContent.length !== 0}
                message={<>{errorContent.length !== 0 && errorContent}</>}
            />

            {!user ?
                <>
                    <form>
                        <label className="label">Email: </label>
                        <input
                            onChange={handleEmail}
                            className="input"
                            value={email}
                            type="email"
                        />
                        <br /><br />

                        <label className="label">Password: </label>
                        <input
                            onChange={handlePassword}
                            className="input"
                            value={password}
                            type="password"
                        />
                        <br /><br />

                        <button
                            onClick={handleSubmit}
                            className="btn"
                            type="submit"
                        >
                            Log in
                        </button>
                    </form>
                </>
                : <button onClick={handleLogout}>Log out</button>}
        </div>
    );
}
