const express = require('express');
const route = express.Router();
const bodyParser = require("body-parser");
const { Invoice, User, Food} = require("../models");

const { getInvoice } = require('../controllers/getInvoice');
const { createHash } = require('crypto');
const { decryptHashedPassword } = require('../commands/decryptHashedPasswd');

route.use(bodyParser.json());

route.get('/invoices/:userId', async (req, res) => {
    const { userId } = req.params

    try {
        const invoices = await Invoice.findAll({
            where: { userId },
            order: [['createdAt', 'DESC']]
        })

        const filtered = invoices.map((invoice, index) => {
            return { id: invoice.id, name: `Faktúra_${new Date(invoice.createdAt).getTime()/1000}`, date: invoice.createdAt }
        })

        res.json(filtered);
    } catch (error) {
        res.status(500).json({ error: 'Internal Server Error' });
    }
});

route.post('/invoices/create', async (req, res) => {

    const { userId, foodIds, password } = req.body

    //console.log(req.body);

    const { soupId, mainId } = foodIds;

    if (!userId || !foodIds || !password || userId.trim().length === 0 || password.trim().length === 0) {
        return res.status(404).json({ error: "Zle vyplnené údaje." })
    }

    if (soupId.trim().length === 0 && mainId.trim().length === 0) {
        return res.status(404).json({ error: "Zle vyplnené údaje." })
    }

    let decryptedPassword;
    try {
        decryptedPassword = decryptHashedPassword(password);
    }
    catch (e) {
        return res.status(404).json({ error: "Zadané heslo je v zlom formáte." })
    }

    const user = await User.findOne({ where: { id: userId }, attributes: ['id', 'email', 'password', 'salt', 'firstName', 'lastName'] })

    if (user == null) {
        return res.status(404).json({ error: "Zle vyplnené údaje." })
    }

    const passhash = createHash("sha256")
        .update(decryptedPassword)
        .update(createHash("sha256").update(user.salt, "utf8").digest("hex"))
        .digest("hex");

    if (!user || (user && user.password !== passhash)) {
        return res.status(404).json({ error: "Zle vyplnené údaje." })
    }

    // Fetching 'price' and 'name' columns for the specified IDs
    const foodData = await Food.findAll({
        where: {
            id: [mainId, soupId],
        },
        attributes: ['price', 'name'],
    });

    const combinedDetails = foodData.reduce(
        (result, food) => {
            result.names.push(food.name);

            result.totalPrice += parseFloat(food.price);

            return result;
        },
        { names: [], totalPrice: 0 }
    );

    // Concatenate the names with a plus sign
    const concatenatedNames = combinedDetails.names.join(' + ');

    const totalPrice = combinedDetails.totalPrice;

    const invoice = await Invoice.create({
        userId: userId,
        foodName: concatenatedNames,
        price: totalPrice,
    });

    //console.log(invoice);

    res.status(201).json({ success: true });
})

route.post('/invoices', async (req, res) => {

    const { userId, invoiceId } = req.body

    if (!userId || userId.trim().length === 0 || !invoiceId || invoiceId.trim().length === 0) {
        return res.json({ error: "Zle vyplnené údaje." })
    }

    const bytes = await getInvoice({ userId, invoiceId })

    if (bytes == null) {
        return res.json({ error: "Nepodarilo sa vytvoriť faktúru." })
    }

    return res.json({ status: "Faktúra bola úspešne vytvorená.", bytes })
});

module.exports = route;