const express = require('express');
const route = express.Router();
const bodyParser = require("body-parser");
const { User, Review } = require("../models");
const { createHash } = require('crypto');
const { decryptHashedPassword } = require('../commands/decryptHashedPasswd');

route.use(bodyParser.json());

route.post('/reviews', async (req, res) => {
    try {
        // Extract the user ID from the request parameters

        const { userId, password, food, stars, message } = req.body;

        console.log(req.body);

        const starsCount = parseInt(req.body.stars, 10);

        if (isNaN(starsCount) || starsCount < 0 || starsCount > 5) {
            return res.status(404).json({ error: "Bad request" });
        }

        if (!userId || !password) {
            return res.status(400).json({ error: "Zlé údaje." });
        }

        let decryptedPassword
        try {
            decryptedPassword = decryptHashedPassword(password);
            console.log("Decrypted password: ", decryptedPassword);
        }
        catch (e) {
            return res.status(404).json({ error: "Zlý formát hesla." })
        }

        const user = await User.findOne({ where: { id: userId }, attributes: ['id', 'email', 'password', 'salt', 'firstName', 'lastName'] })

        if (user == null) {
            return res.status(404).json({ error: "Zlé údaje." })
        }

        const passhash = createHash("sha256")
            .update(decryptedPassword)
            .update(createHash("sha256").update(user.salt, "utf8").digest("hex"))
            .digest("hex");

        if (!user || (user && user.password !== passhash)) {
            return res.status(404).json({ error: "Zlé údaje." })
        }

        const review = await Review.create({
            userId: userId,
            firstName: user.get('firstName'),
            lastName: user.get('lastName'),
            food,
            stars: starsCount,
            message,
        });

        res.status(201).json({ success: true });

    } catch (error) {
        return res.status(500).json({ error: 'Internal Server Error' });
    }
});

route.get('/reviews', async (req, res) => {
    try {
        // Retrieve all reviews from the database
        const reviews = await Review.findAll({
            attributes: ['id', 'firstName', 'lastName', 'food', 'stars', 'message', 'createdAt'],
        });

        // Send the reviews as JSON in the response
        res.json(reviews);
    } catch (error) {
        // console.error('Error retrieving reviews:', error);
        // Handle the error and send an appropriate response
        res.status(500).json({ error: 'Internal Server Error' });
    }
});

module.exports = route;

